#import <UIKit/UIKit.h>
#import <sqlite3.h>
#import "Tunebook.h"
#import "Tune.h"
#import "Reachability.h"

@class Playlist;
@class MainViewController;
@class ManageTunesController;
@class SettingsController;
@class PlaylistViewController;
@class TuneViewController;
@class TopLevelViewController;
@class TuneDatabase;

enum {
  kTransitionNone,
  kTransitionCurl,
  kTransitionSlide,
  kTransitionFade,
};

@interface AppDelegate : NSObject <UIApplicationDelegate, UITabBarDelegate>
{
	IBOutlet UIWindow *window;
	IBOutlet UINavigationController *navigationController;
	IBOutlet UIViewController *topViewController;
	IBOutlet UIView *loadingView;
	IBOutlet MainViewController *tunesViewController;
	IBOutlet UIViewController *playlistViewController;
	IBOutlet UIViewController *favouritesViewController;
	IBOutlet UIViewController *searchViewController;
	IBOutlet UIViewController *settingsController;
  IBOutlet ManageTunesController *manageTunesController;
	IBOutlet UITabBar *tabBar;
	IBOutlet UITabBarController *tabBarController;
	IBOutlet UIView *tabView;
	IBOutlet UIView *rootView;
	IBOutlet TopLevelViewController *topLevelViewController;
  TuneViewController *tuneViewController;
	UIViewController *currentViewController;
  NSMutableArray *tunes;
  NSMutableArray *playlists;
  NSMutableArray *filesToImport;
  NSMutableArray* formatFiles;
  NSMutableDictionary *settings;
  Reachability *reachability;
  TuneDatabase* database;

  float playerVolume;
	float playerSpeed;
  int playerVoice;
  int playerVoiceTrebleAccompaniment;
  int playerVoiceBassAccompaniment;
  BOOL guitarChords;
  BOOL loopPlayback;
  BOOL preventSleep;
  BOOL noImportDuplicates;
  BOOL initializedDatabase;
  NSInteger selectedTab;
  int currentTune;
  int currentTunebook;
  int transitionType;
  float tuneScaling;
#if TUNEBOOK_RECENTLY_VIEWED
  int recentlyViewedCount;
#endif
#if TUNEBOOK_RECENTLY_IMPORTED
  int recentlyImportedCount;
#endif
  BOOL playbackCountIn;
}

@property (nonatomic, retain) UINavigationController *navigationController;
@property (nonatomic, retain) UIViewController *topViewController;
@property (nonatomic, retain) TuneViewController *tuneViewController;
@property (nonatomic, retain) TopLevelViewController *topLevelViewController;

@property (nonatomic, retain) UIView *tabBar; // iPhone only
@property (nonatomic, retain) UIView *rootView;

@property (nonatomic, retain) TuneDatabase* database;
@property (nonatomic, retain, readonly) NSMutableArray *tunes;
@property (nonatomic, retain, readonly) NSMutableArray *playlists;
@property (nonatomic, retain) NSMutableDictionary *settings;
@property (nonatomic, retain) Reachability *reachability;

@property (nonatomic, readonly) BOOL initializedDatabase;

@property (nonatomic, assign) float playerVolume;
@property (nonatomic, assign) float playerSpeed;
@property (nonatomic, assign) int playerVoice;
@property (nonatomic, assign) int playerVoiceTrebleAccompaniment;
@property (nonatomic, assign) int playerVoiceBassAccompaniment;
@property (nonatomic, assign) SortOrder tuneSortOrder;
@property (nonatomic, assign) SortOrder tunebookTuneSortOrder;
@property (nonatomic, assign) BOOL guitarChords;
@property (nonatomic, assign) BOOL playbackCountIn;
@property (nonatomic, assign) BOOL loopPlayback;
@property (nonatomic, assign) BOOL preventSleep;
@property (nonatomic, assign) BOOL noImportDuplicates;
@property (nonatomic, assign) int currentTune;
@property (nonatomic, assign) int currentTunebook;
@property (nonatomic, assign) int transitionType;
@property (nonatomic, assign) float tuneScaling;
@property (nonatomic, assign) BOOL zoomTuneWithIndexKey;
#if TUNEBOOK_RECENTLY_VIEWED
@property (nonatomic, assign) int recentlyViewedCount;
#endif
#if TUNEBOOK_RECENTLY_IMPORTED
@property (nonatomic, assign) int recentlyImportedCount;
#endif

@property (nonatomic, readonly) NSMutableArray* formatFiles;

- (void)initializeDatabase;
- (void)startTransaction;
- (void)commitTransaction;

- (void)showTune:(Tune *)tune;
- (void)showTunes:(NSArray *)tunes title:(NSString*)title;

- (IBAction)removeTune:(Tune *)tune;
- (void)addTune:(Tune *)tune;
- (void)deleteAllTunes;
- (void)sortTunes;
- (void)savePreferences;

- (IBAction)onImportTunes:(id)sender;

- (void)addPlaylist:(Playlist *)playlist;
- (void)removePlaylist:(Playlist *)playlist;

- (void)searchForTunes:(NSString *)searchText tunes:(NSArray *)tunelist results:(NSMutableArray *)results;
- (void)searchForTunesWithTitle:(NSString *)titleText key:(NSString *)keyText meter:(NSString *)meterText tunes:(NSArray *)tunelist results:(NSMutableArray *)results;
- (void)searchForDots:(NSString *)searchText tunes:(NSArray *)tunelist results:(NSMutableArray *)results;

- (void)tabBar:(UITabBar *)tabBar didSelectItem:(UITabBarItem *)item;

- (void)threadRun:(id)sender;
- (void)checkDocumentFolderForNewFiles;
- (void)importFiles:(id)sender;
- (void)importFile:(NSString*)path;

- (id)tuneWithPrimaryKey:(NSInteger)pk;

- (SortOrder)sortOrderForPlaylist:(Playlist*)playlist;
- (void)setSortOrder:(SortOrder)order forPlaylist:(Playlist*)playlist;

@end

@interface RootViewController : UIViewController
{
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation;

@end

@interface RotatingNavigationController : UINavigationController
{
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation;

@end
