#import "AppDelegate.h"
#import "TuneDatabase.h"
#import "MainViewController.h"
#import "PlaylistViewController.h"
#import "ImportProgressController.h"
#import "ManageTunesController.h"
#import "Format.h"
#import "TopLevelViewController.h"
#import "TuneViewController.h"
#import "SettingsController.h"
#if TUNEBOOK_WATCH_FOLDER
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/event.h>
#include <sys/time.h>
#include <fcntl.h>
#endif

@implementation AppDelegate

@synthesize database, settings, reachability, playerVolume, playerSpeed, playerVoice, navigationController, topViewController, initializedDatabase, guitarChords, loopPlayback, currentTune, currentTunebook, transitionType;
@synthesize playbackCountIn;
@synthesize playerVoiceTrebleAccompaniment;
@synthesize playerVoiceBassAccompaniment;
@synthesize preventSleep;
@synthesize noImportDuplicates;
@synthesize tuneScaling;
@synthesize tunebookTuneSortOrder;
@synthesize zoomTuneWithIndexKey;
@synthesize formatFiles;
@synthesize tuneViewController;
@synthesize topLevelViewController;
@synthesize tabBar;
@synthesize rootView;
#if TUNEBOOK_RECENTLY_VIEWED
@synthesize recentlyViewedCount;
#endif
#if TUNEBOOK_RECENTLY_IMPORTED
@synthesize recentlyImportedCount;
#endif

- (void)dealloc
{
  [settings release];
  [database release];
	[navigationController release];
  [window release];
  [super dealloc];
}

static NSString *kSelectedTabKey = @"selectedTabKey";
static NSString *kPlayerVolumeKey = @"playerVolumeKey";
static NSString *kPlayerSpeedKey = @"playerSpeedKey";
static NSString *kPlayerVoiceKey = @"playerVoiceKey";
static NSString *kPlayerVoiceTrebleAccompanimentKey = @"playerVoiceTrebleAccompanimentKey";
static NSString *kPlayerVoiceBassAccompanimentKey = @"playerVoiceBassAccompanimentKey";
static NSString *kTuneSortOrderKey = @"tuneSortOrderKey";
static NSString *kTunebookTuneSortOrderKey = @"tunebookTuneSortOrderKey";
static NSString *kGuitarChordsKey = @"guitarChordsKey";
static NSString *kLoopPlaybackKey = @"loopPlaybackKey";
static NSString *kPreventSleepKey = @"preventSleepKey";
static NSString *kCurrentTuneKey = @"currentTuneKey";
static NSString *kCurrentTunebookKey = @"currentTuneBookKey";
static NSString *kTransitionTypeKey = @"transitionType";
static NSString *kTuneScalingKey = @"tuneScaling";
static NSString *kZoomTuneWithIndexKey = @"zoomTuneWithIndexKey";
static NSString *kNoImportDuplicatesKey = @"noImportDuplicates";
#if TUNEBOOK_RECENTLY_VIEWED
static NSString *kRecentlyViewedKey = @"recentlyViewed";
#endif
#if TUNEBOOK_RECENTLY_IMPORTED
static NSString *kRecentlyImportedKey = @"recentlyImported";
#endif
static NSString *kPlaybackCountInKey = @"playbackCountInKey";

int PaperWidth;//FIXME-put somewhere sensible
int PaperHeight;

- (void)applicationDidFinishLaunching:(UIApplication *)application
{
  selectedTab = 0;
  settings = [[NSMutableDictionary alloc] init];
  formatFiles = [[NSMutableArray alloc] init];

  // Initialize preferences
	NSString *testValue = [[NSUserDefaults standardUserDefaults] stringForKey:kPlayerVolumeKey];
	if ( testValue == nil ) {
		NSDictionary *appDefaults =  [NSDictionary dictionaryWithObjectsAndKeys:
      [NSNumber numberWithInteger:kSortTunesByOrder], kTuneSortOrderKey,
      [NSNumber numberWithInteger:kSortTunesByOrder], kTunebookTuneSortOrderKey,
      [NSNumber numberWithFloat:1], kPlayerVolumeKey,
      [NSNumber numberWithInteger:0], kPlayerVoiceKey,
      [NSNumber numberWithInteger:1], kPlayerVoiceTrebleAccompanimentKey,
      [NSNumber numberWithInteger:32], kPlayerVoiceBassAccompanimentKey,
      [NSNumber numberWithFloat:1], kPlayerSpeedKey,
      [NSNumber numberWithBool:1], kGuitarChordsKey,
      [NSNumber numberWithBool:1], kLoopPlaybackKey,
      [NSNumber numberWithBool:0], kPreventSleepKey,
      [NSNumber numberWithBool:1], kNoImportDuplicatesKey,
      [NSNumber numberWithInteger:0], kSelectedTabKey,
      [NSNumber numberWithInteger:-1], kCurrentTuneKey,
      [NSNumber numberWithInteger:-1], kCurrentTunebookKey,
      [NSNumber numberWithInteger:0], kTransitionTypeKey,
      [NSNumber numberWithFloat:1], kTuneScalingKey,
      [NSNumber numberWithBool:1], kZoomTuneWithIndexKey,
#if TUNEBOOK_RECENTLY_VIEWED
      [NSNumber numberWithInteger:10], kRecentlyViewedKey,
#endif
#if TUNEBOOK_RECENTLY_IMPORTED
      [NSNumber numberWithInteger:10], kRecentlyImportedKey,
#endif
      [NSNumber numberWithBool:1], kPlaybackCountInKey,
      nil];
		
		[[NSUserDefaults standardUserDefaults] registerDefaults:appDefaults];
		[[NSUserDefaults standardUserDefaults] synchronize];
	}
	
	guitarChords = [[NSUserDefaults standardUserDefaults] boolForKey:kGuitarChordsKey];
	loopPlayback = [[NSUserDefaults standardUserDefaults] boolForKey:kLoopPlaybackKey];
	preventSleep = [[NSUserDefaults standardUserDefaults] boolForKey:kPreventSleepKey];
	noImportDuplicates = [[NSUserDefaults standardUserDefaults] boolForKey:kNoImportDuplicatesKey];
	playerVolume = [[NSUserDefaults standardUserDefaults] floatForKey:kPlayerVolumeKey];
	playerVoice = (int)[[NSUserDefaults standardUserDefaults] integerForKey:kPlayerVoiceKey];
	playerVoiceTrebleAccompaniment = (int)[[NSUserDefaults standardUserDefaults] integerForKey:kPlayerVoiceTrebleAccompanimentKey];
	playerVoiceBassAccompaniment = (int)[[NSUserDefaults standardUserDefaults] integerForKey:kPlayerVoiceBassAccompanimentKey];
	playerSpeed = [[NSUserDefaults standardUserDefaults] floatForKey:kPlayerSpeedKey];
  if ( playerSpeed < -1 )
    playerSpeed = -1;
  if ( playerSpeed > 1 )
    playerSpeed = 1;
	currentTune = (int)[[NSUserDefaults standardUserDefaults] integerForKey:kCurrentTuneKey];
	currentTunebook = (int)[[NSUserDefaults standardUserDefaults] integerForKey:kCurrentTunebookKey];
	transitionType = (int)[[NSUserDefaults standardUserDefaults] integerForKey:kTransitionTypeKey];
	tuneScaling = [[NSUserDefaults standardUserDefaults] floatForKey:kTuneScalingKey];
  if ( tuneScaling < 1.0f )
    tuneScaling = 1.0f;
  if ( tuneScaling > 2.0f )
    tuneScaling = 2.0f;
	zoomTuneWithIndexKey = [[NSUserDefaults standardUserDefaults] boolForKey:kZoomTuneWithIndexKey];
#if TUNEBOOK_RECENTLY_VIEWED
  recentlyViewedCount =  (int)[[NSUserDefaults standardUserDefaults] integerForKey:kRecentlyViewedKey];
#endif
#if TUNEBOOK_RECENTLY_IMPORTED
  recentlyImportedCount =  (int)[[NSUserDefaults standardUserDefaults] integerForKey:kRecentlyImportedKey];
#endif
  playbackCountIn = [[NSUserDefaults standardUserDefaults] boolForKey:kPlaybackCountInKey];

  if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
    PaperWidth = 542;
    PaperHeight = 842;

    // iOS8 fix for launching in landscape mode
    window.frame = [[UIScreen mainScreen] bounds];

    //FIXME....use IB for this, if we can work out how
    topLevelViewController = [[TopLevelViewController alloc] initWithNibName:@"TopLevel" bundle:nil];
    topLevelViewController.view.frame = [[UIScreen mainScreen] bounds];
    [window setRootViewController:topLevelViewController];
    [window addSubview: topLevelViewController.view];
    tuneViewController = topLevelViewController.tuneViewController;
    [topLevelViewController addChildViewController:tuneViewController];
    self.topLevelViewController = topLevelViewController;
    rootView = topLevelViewController.view;
    loadingView.frame = rootView.bounds;
  }
  else {
    PaperWidth = 592;
    PaperHeight = 842;

    [window setRootViewController:navigationController];

    [navigationController.navigationBar setBarStyle:UIBarStyleDefault];
    [window addSubview: navigationController.view];

    [tabView addSubview: tunesViewController.view];
    tunesViewController.view.frame = tabView.bounds;

    [tabView addSubview: playlistViewController.view];
    playlistViewController.view.frame = tabView.bounds;
    playlistViewController.view.hidden = YES;

    [tabView addSubview: searchViewController.view];
    searchViewController.view.frame = tabView.bounds;
    searchViewController.view.hidden = YES;

    [tabView addSubview: settingsController.view];
    settingsController.view.frame = tabView.bounds;
    settingsController.view.hidden = YES;

    [tabView addSubview: manageTunesController.view];
    manageTunesController.view.frame = tabView.bounds;
    manageTunesController.view.hidden = YES;

    currentViewController = tunesViewController;
  }

  [rootView addSubview:loadingView];
  [rootView bringSubviewToFront:loadingView];
  [window makeKeyAndVisible];

  reachability = [[Reachability reachabilityForInternetConnection] retain];
  [reachability startNotifer];

  if (UI_USER_INTERFACE_IDIOM() != UIUserInterfaceIdiomPad) {
    NSArray *items = tabBar.items;
    selectedTab = [[NSUserDefaults standardUserDefaults] integerForKey:kSelectedTabKey];
    if ( selectedTab >= 0 && selectedTab < [items count] ) {
      UITabBarItem *item = (UITabBarItem*)[items objectAtIndex:selectedTab];
      tabBar.selectedItem = item;
      [self tabBar:tabBar didSelectItem:item];
    }
  }

  if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
    [topLevelViewController zoomModeChanged];

  [UIApplication sharedApplication].idleTimerDisabled = preventSleep;

  [NSThread detachNewThreadSelector:@selector(threadRun:) toTarget:self withObject:nil];
}

- (void)showTune:(Tune *)tune;
{
  if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
    [tuneViewController showTune:tune];
  else {
    if ( [[navigationController topViewController] class] == TuneViewController.class )
      [((TuneViewController*)[navigationController topViewController]) showTune:tune];
    else {
      TuneViewController *controller = [[TuneViewController alloc] initWithTune:tune];
      [navigationController pushViewController:controller animated:YES];
      [controller release];
    }
  }
}

- (void)showTunes:(NSArray *)aTunes title:(NSString*)title;
{
  Tune *tune = [Tune tuneWithTunes:aTunes title:title];
  [self showTune:tune];
}

- (void)threadRun:(id)sender
{
  NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];  
  [self initializeDatabase];
  [self performSelectorOnMainThread:@selector(databaseLoaded:) withObject:self waitUntilDone:NO];
  [pool release];
}

- (void)databaseLoaded:(id)sender
{
  [loadingView removeFromSuperview];
  tunesViewController.playlist = nil;
  [[NSNotificationCenter defaultCenter] postNotificationName:@"databaseLoaded" object:self];

  [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(importFiles:) name:@"importedFile" object:nil]; 

  [self checkDocumentFolderForNewFiles];

  if ( currentTune >= 0 ) {
    [topLevelViewController.tunebookViewController selectTunebook:currentTunebook];

    // Unset the current tune preference. That way, if we get a crash due to a bad ABC file, we won't try to
    // load the file again next time we start up.
    [[NSUserDefaults standardUserDefaults] setInteger:-1 forKey:kCurrentTuneKey];
    [[NSUserDefaults standardUserDefaults] setInteger:-1 forKey:kCurrentTunebookKey];
    [[NSUserDefaults standardUserDefaults] synchronize];

    Tune *tune = [Tune tuneWithPrimaryKey:currentTune database:database.database];
    [self showTune:tune];
  }

#if TUNEBOOK_WATCH_FOLDER
  [self startWatchingFolder];
#endif
}

- (void)checkFolderForNewFiles:(NSString*)directory
{
  NSArray *files = [[NSFileManager defaultManager] contentsOfDirectoryAtPath:directory error:nil];
  for ( NSString *file in files ) {
#if 1
    if ( [file hasSuffix:@".abc"] || [file hasSuffix:@".ABC"] ) {
      if ( !filesToImport )
        filesToImport = [[NSMutableArray alloc] init];
      NSString *path = [directory stringByAppendingPathComponent:file];
      [filesToImport addObject:path];
    }
    else
#endif
    if ( [file hasSuffix:@".fmt"] || [file hasSuffix:@".FMT"] ) {
      NSString *path = [directory stringByAppendingPathComponent:file];
      NSString* text = [NSString stringWithContentsOfFile:path encoding:NSUTF8StringEncoding error:nil];
      if ( text )
        [formatFiles addObject:[[Format alloc] initWithName:[[path lastPathComponent] stringByDeletingPathExtension] text:text]];
    }
  }
}

- (void)checkDocumentFolderForNewFiles
{
  // See if there are any ABC files to be synced
  NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
  NSString *documentsDirectory = [paths objectAtIndex:0];
  [self checkFolderForNewFiles:documentsDirectory];
//  NSString *inboxDirectory = [documentsDirectory stringByAppendingPathComponent:@"Inbox"];
//  [self checkFolderForNewFiles:inboxDirectory];

  if ( [filesToImport count] > 0 ) {
    [self importFiles:self];
  }
}

- (void)importFiles:(id)sender
{
  // Import the next file on our list of files to import
  if ( [filesToImport count] > 0 ) {
    NSString *path = [filesToImport objectAtIndex:0];
    [path retain];
    [filesToImport removeObject:path];
    [self importFile:path];
    [path autorelease];
  }
}

- (void)importFile:(NSString*)path
{
  ImportProgressController *controller = [[ImportProgressController alloc] initWithNibName:@"ImportProgress" bundle:nil];
  if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
    controller.modalPresentationStyle = UIModalPresentationFormSheet;
    [tuneViewController presentViewController:controller animated:YES completion:nil];
  }
  else {
    [navigationController presentViewController:controller animated:YES completion:nil];
  }
  [controller import:[NSURL fileURLWithPath:path isDirectory:NO] animateDismiss:NO];
  [controller release];
}

-(BOOL)application:(UIApplication *)application openURL:(NSURL *)url
  sourceApplication:(NSString *)sourceApplication
  annotation:(id)annotation
{
  if (url != nil && [url isFileURL]) {
    [self importFile:[url path]];
  }    
  return YES;
}

- (BOOL)application:(UIApplication *)app openURL:(NSURL *)url options:(NSDictionary<UIApplicationOpenURLOptionsKey, id> *)options
{
  if (url != nil && [url isFileURL]) {
    [self importFile:[url path]];
  }
  return YES;
}

- (void)initializeDatabase
{
  database = [[TuneDatabase alloc] init];
  [database initializeDatabase];
	database.tuneSortOrder = (int)[[NSUserDefaults standardUserDefaults] integerForKey:kTuneSortOrderKey];
#if TUNEBOOK_RECENTLY_VIEWED
	database.recentlyViewedCount = (int)[[NSUserDefaults standardUserDefaults] integerForKey:kRecentlyViewedKey];
#endif
#if TUNEBOOK_RECENTLY_IMPORTED
	database.recentlyViewedCount = (int)[[NSUserDefaults standardUserDefaults] integerForKey:kRecentlyImportedKey];
#endif
  [Tune sortTunes:database.tunes by:database.tuneSortOrder];
  initializedDatabase = YES;
}

- (void)startTransaction
{
  [database startTransaction];
}

- (void)commitTransaction
{
  [database commitTransaction];
}

- (void)applicationWillTerminate:(UIApplication *)application
{
  [tunes makeObjectsPerformSelector:@selector(dehydrate)];
  [Tune finalizeStatements];
  [Playlist finalizeStatements];
  [database close];
  [self savePreferences];
}

- (void)applicationDidEnterBackground:(UIApplication *)application
{
  // Save the current settings when enetering background
  [self savePreferences];
}

- (void)savePreferences
{
  [[NSUserDefaults standardUserDefaults] setFloat:selectedTab forKey:kSelectedTabKey];
  [[NSUserDefaults standardUserDefaults] setFloat:playerVolume forKey:kPlayerVolumeKey];
  [[NSUserDefaults standardUserDefaults] setFloat:playerSpeed forKey:kPlayerSpeedKey];
  [[NSUserDefaults standardUserDefaults] setInteger:playerVoice forKey:kPlayerVoiceKey];
  [[NSUserDefaults standardUserDefaults] setInteger:playerVoiceTrebleAccompaniment forKey:kPlayerVoiceTrebleAccompanimentKey];
  [[NSUserDefaults standardUserDefaults] setInteger:playerVoiceBassAccompaniment forKey:kPlayerVoiceBassAccompanimentKey];
  [[NSUserDefaults standardUserDefaults] setInteger:self.tuneSortOrder forKey:kTuneSortOrderKey];
  [[NSUserDefaults standardUserDefaults] setInteger:self.tunebookTuneSortOrder forKey:kTunebookTuneSortOrderKey];
  [[NSUserDefaults standardUserDefaults] setBool:guitarChords forKey:kGuitarChordsKey];
  [[NSUserDefaults standardUserDefaults] setBool:loopPlayback forKey:kLoopPlaybackKey];
  [[NSUserDefaults standardUserDefaults] setBool:preventSleep forKey:kPreventSleepKey];
  [[NSUserDefaults standardUserDefaults] setBool:noImportDuplicates forKey:kNoImportDuplicatesKey];
  [[NSUserDefaults standardUserDefaults] setInteger:currentTune forKey:kCurrentTuneKey];
  [[NSUserDefaults standardUserDefaults] setInteger:currentTunebook forKey:kCurrentTunebookKey];
  [[NSUserDefaults standardUserDefaults] setInteger:transitionType forKey:kTransitionTypeKey];
  [[NSUserDefaults standardUserDefaults] setFloat:tuneScaling forKey:kTuneScalingKey];
  [[NSUserDefaults standardUserDefaults] setBool:zoomTuneWithIndexKey forKey:kZoomTuneWithIndexKey];
#if TUNEBOOK_RECENTLY_VIEWED
  [[NSUserDefaults standardUserDefaults] setInteger:recentlyViewedCount forKey:kRecentlyViewedKey];
#endif
#if TUNEBOOK_RECENTLY_IMPORTED
  [[NSUserDefaults standardUserDefaults] setInteger:recentlyImportedCount forKey:kRecentlyImportedKey];
#endif
  [[NSUserDefaults standardUserDefaults] setBool:playbackCountIn forKey:kPlaybackCountInKey];
  [[NSUserDefaults standardUserDefaults] synchronize];
}

- (id)tuneWithPrimaryKey:(NSInteger)pk
{
  return [Tune tuneWithPrimaryKey:pk database:database.database];
}

- (void)removeTune:(Tune *)tune
{
  [database removeTune:tune];
}

- (void)addTune:(Tune *)tune
{
  [database addTune:tune];
}

-(void)onImportTunes:(id)sender
{
  ManageTunesController *controller = [[ManageTunesController alloc] initWithNibName:@"ManageTunes" bundle:nil];
	[navigationController pushViewController:controller animated:YES];
  [controller release];
}

- (void)deleteAllTunes
{
  [database deleteAllTunes];
  [self showTune:nil];
}

- (void)sortTunes
{
  [database sortTunesBy:self.tuneSortOrder];
}

- (void)addPlaylist:(Playlist *)playlist
{
  [database addPlaylist:playlist];
}

- (void)removePlaylist:(Playlist *)playlist
{
  [database removePlaylist:playlist];
}

- (void)searchForTunes:(NSString *)searchText tunes:(NSArray *)tunelist results:(NSMutableArray *)results
{
  [database searchForTunes:searchText tunes:tunelist results:results];
}

- (void)searchForTunesWithTitle:(NSString *)titleText key:(NSString *)keyText meter:(NSString *)meterText tunes:(NSArray *)tunelist results:(NSMutableArray *)results
{
  [database searchForTunesWithTitle:titleText key:keyText meter:meterText tunes:tunelist results:results];
}

- (void)searchForDots:(NSString *)searchText tunes:(NSArray *)tunelist results:(NSMutableArray *)results;
{
  [database searchForDots:searchText tunes:tunelist results:results];
}

- (NSMutableArray *)tunes
{
  return database.tunes;
}

- (NSMutableArray *)playlists
{
  return database.playlists;
}

- (SortOrder) tuneSortOrder
{
  return database.tuneSortOrder;
}

- (void) setTuneSortOrder:(SortOrder)order
{
  if ( order != database.tuneSortOrder ) {
    database.tuneSortOrder = (SortOrder)order;
    [self sortTunes];
  }
}

- (SortOrder)sortOrderForPlaylist:(Playlist*)playlist
{
  if ( playlist )
    return tunebookTuneSortOrder;
  return database.tuneSortOrder;
}

- (void)setSortOrder:(SortOrder)order forPlaylist:(Playlist*)playlist
{
  if ( playlist )
    tunebookTuneSortOrder = order;
  else {
    //FIXME-why do we store this twice?
    self.tuneSortOrder = order;
    database.tuneSortOrder = (SortOrder)order;
  }
}

- (void)tabBar:(UITabBar *)tabBar didSelectItem:(UITabBarItem *)item
{
  selectedTab = [item tag];
  if ( currentViewController )
    [currentViewController viewWillDisappear:NO];
  currentViewController.view.hidden = YES;
  switch ( [item tag] ) {
  case 0:
    currentViewController = tunesViewController;
    topViewController.navigationItem.rightBarButtonItem = currentViewController.editButtonItem;
    break;
  case 1:
    currentViewController = playlistViewController;
    topViewController.navigationItem.rightBarButtonItem = currentViewController.editButtonItem;
    break;
  case 2:
    currentViewController = searchViewController;
    topViewController.navigationItem.rightBarButtonItem = nil;
    break;
  case 3:
    currentViewController = manageTunesController;
    topViewController.navigationItem.rightBarButtonItem = nil;
    break;
  case 4:
    currentViewController = settingsController;
    topViewController.navigationItem.rightBarButtonItem = nil;
    break;
  }
  if ( currentViewController )
    [currentViewController viewWillAppear:NO];
  currentViewController.view.hidden = NO;
}

- (void)refreshView
{
  //FIXME-This is a hacky workround for viewWillAppear not being called on MainView/PlaylistController when going Back from a tune,
  // which causes the Edit button to disappear.
  [self tabBar:tabBar didSelectItem:[tabBar selectedItem]];
  topViewController.navigationItem.rightBarButtonItem = currentViewController.editButtonItem;
}

#if TUNEBOOK_WATCH_FOLDER
static CFFileDescriptorRef _kqRef;

- (void)kqueueFired
{
  int kq;
  struct kevent event;
  struct timespec timeout = { 0, 0 };
  int eventCount;

  kq = CFFileDescriptorGetNativeDescriptor(_kqRef);

  eventCount = kevent(kq, NULL, 0, &event, 1, &timeout);

  if (eventCount == 1) {
    [self performSelector:@selector(checkDocumentFolderForNewFiles) withObject:nil afterDelay:10.0f];
  }

  CFFileDescriptorEnableCallBacks(_kqRef, kCFFileDescriptorReadCallBack);
}

static void KQCallback(CFFileDescriptorRef kqRef, CFOptionFlags callBackTypes, void *info)
{
  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
  [appDelegate kqueueFired];
}

- (void)startWatchingFolder
{
  NSString *              docPath;
  int                     dirFD;
  int                     kq;
  int                     retVal;
  struct kevent           eventToAdd;
  CFFileDescriptorContext context = { 0, self, NULL, NULL, NULL };
  CFRunLoopSourceRef      rls;

  docPath = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
  assert(docPath != 0);

  dirFD = open([docPath fileSystemRepresentation], O_EVTONLY);
  assert(dirFD >= 0);

  kq = kqueue();
  assert(kq >= 0);

  eventToAdd.ident  = dirFD;
  eventToAdd.filter = EVFILT_VNODE;
  eventToAdd.flags  = EV_ADD | EV_CLEAR;
  eventToAdd.fflags = NOTE_WRITE;
  eventToAdd.data   = 0;
  eventToAdd.udata  = NULL;

  retVal = kevent(kq, &eventToAdd, 1, NULL, 0, NULL);
  assert(retVal == 0);
  assert(_kqRef == NULL);

  _kqRef = CFFileDescriptorCreate(NULL, kq, true, KQCallback, &context);
  assert(_kqRef != NULL);
  rls = CFFileDescriptorCreateRunLoopSource(NULL, _kqRef, 0);
  assert(rls != NULL);
  CFRunLoopAddSource(CFRunLoopGetCurrent(), rls, kCFRunLoopDefaultMode);
  CFRelease(rls);
  CFFileDescriptorEnableCallBacks(_kqRef, kCFFileDescriptorReadCallBack);
}
#endif
@end

@implementation RootViewController

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
  return YES;
}

- (BOOL)shouldAutorotate {
  return YES;
}

- (UIInterfaceOrientationMask)supportedInterfaceOrientations
{
  return UIInterfaceOrientationMaskAll;
}

- (void)viewWillAppear:(BOOL)animated
{
  [super viewWillAppear:animated];
  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
  [appDelegate refreshView];
}

// This is overriden to reset the Edit button when a modal view is dismissed.
// The UINavigationController for some reason loses the button when a modal
// view is shown from the first view in the navigation stack.
- (void)viewDidAppear:(BOOL)animated
{
  [super viewDidAppear:animated];
  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
  [appDelegate refreshView];
}
@end

@implementation RotatingNavigationController

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
  return YES;
}

- (BOOL)shouldAutorotate
{
  return YES;
}

- (UIInterfaceOrientationMask)supportedInterfaceOrientations
{
  return UIInterfaceOrientationMaskAll;
}

- (void)viewWillAppear:(BOOL)animated
{
  [super viewWillAppear:animated];
  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
  [appDelegate refreshView];
}

@end

