#import "EditABCController.h"
#import "TuneViewController.h"
#import "Tune.h"
#import "AppDelegate.h"

@implementation EditABCView
@synthesize controller;

- (void) layoutSubviews
{
  [super layoutSubviews];
  [self.controller layoutSubviews];
}
@end

@implementation EditABCController

@synthesize changed;
@synthesize editTune;
@synthesize tuneViewController;


- (void)keyboardWillShow:(NSNotification *)notification
{
    NSDictionary* userInfo = [notification userInfo];
    UIViewAnimationCurve animationCurve;
    CGRect keyboardFrameInWindowsCoordinates;
    NSTimeInterval animationDuration = [[userInfo objectForKey:UIKeyboardAnimationDurationUserInfoKey] floatValue];

    [[userInfo objectForKey:UIKeyboardAnimationCurveUserInfoKey] getValue:&animationCurve];
    [[userInfo objectForKey:UIKeyboardFrameEndUserInfoKey] getValue:&keyboardFrameInWindowsCoordinates];

    [self resizeViewForKeyboard:keyboardFrameInWindowsCoordinates
                             withAnimationDuration:animationDuration
                                    animationCurve:animationCurve];
}

- (void)resizeViewForKeyboard:(CGRect)keyboardFrameInWindowsCoordinates
                           withAnimationDuration:(NSTimeInterval)duration
                                  animationCurve:(UIViewAnimationCurve)curve
{
  CGRect fullFrame = self.view.frame;

  CGRect keyboardFrameInViewCoordinates = [self.view convertRect:keyboardFrameInWindowsCoordinates fromView:nil];

  CGRect keyboardFrameVisibleOnScreen = CGRectIntersection(fullFrame, keyboardFrameInViewCoordinates);
  CGRect newFrame = fullFrame;

  newFrame.size.height -= keyboardFrameVisibleOnScreen.size.height;

  if (duration)
  {
    [UIView beginAnimations:nil context:nil];
    [UIView setAnimationDuration:duration];
    [UIView setAnimationCurve:curve];
    [UIView setAnimationBeginsFromCurrentState:YES];
  }

  self.view.frame = newFrame;

  if (duration)
  {
    [UIView commitAnimations];
  }
}

- (void)keyboardWillHide:(id)sender
{
  if ( viewAppeared ) {
    // On iOS 5, this gets called during view initialization but when in landscape
    // mode, the main screen frame is portrait! Ignore this if it happens before
    // we've shown the view.
    self.view.frame = [self.view window].frame;
    [self layoutViews:self.interfaceOrientation];
  }
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
  return YES;
}

- (void)didRotateFromInterfaceOrientation:(UIInterfaceOrientation)fromInterfaceOrientation;
{
  [self layoutViews:self.interfaceOrientation];
  [tuneViewController zoomToFit:self];
}

- (void)layoutSubviews
{
  [self layoutViews:self.interfaceOrientation];
  if ( _aboutToAppear )
    [tuneViewController zoomToFit:self];
}

- (void)layoutViews:(UIInterfaceOrientation)orientation
{
  CGRect bounds = _containerView.bounds;
  float w = bounds.size.width;
  float h = bounds.size.height;
  if ( UIInterfaceOrientationIsPortrait(orientation) ) {
    textView.frame = CGRectMake( 0, h/2+1, w, h/2-1 );
    tuneViewController.view.frame = CGRectMake( 0, 0, w, h/2 );
  } else {
    textView.frame = CGRectMake( 0, 0, w/2-1, h );
    tuneViewController.view.frame = CGRectMake( w/2, 0, w/2, h );
  }
  textView.autoresizingMask = UIViewAutoresizingNone;
  tuneViewController.view.autoresizingMask = UIViewAutoresizingNone;
}

// For iOS 6.0
- (BOOL)shouldAutorotate
{
  return YES;
}

- (UIInterfaceOrientationMask)supportedInterfaceOrientations
{
  return UIInterfaceOrientationMaskAll;
}

- (BOOL)prefersStatusBarHidden
{
  if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
    return NO;
  return YES;
}

- (void)didReceiveMemoryWarning
{
	[super didReceiveMemoryWarning];
}

-(void)dealloc
{
  [tune dealloc];
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[super dealloc];
}

#pragma mark UIViewController delegate methods

-(void)viewDidLoad
{
  _containerView.controller = self;

  if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
    if ( [self respondsToSelector:@selector(setEdgesForExtendedLayout:)] )
      self.edgesForExtendedLayout = UIRectEdgeNone;
  }

#if 1
  //XCode 11.2 fix. We have to create UITextView in code because XIB loading is broken.
  textView = [[UITextView alloc] initWithFrame:CGRectMake(0, 0, 100, 100)];
  [_containerView addSubview:textView];
#endif

  textView.text = editTune.dots;
  textView.delegate = self;
  [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(keyboardWillShow:) name:UIKeyboardWillShowNotification object:nil];
  [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(keyboardWillHide:) name:UIKeyboardWillHideNotification object:nil];
  tune = [[Tune alloc] init];
  tune.temporary = YES;
  tune.hydrated = 1;
  tune.dots = editTune.dots;
  tuneViewController.paginationOn = YES;
  tuneViewController.editingABC = YES;
  [tuneViewController showTune:tune];

  // No room for keybard accessory on iPhone
  if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
    textView.inputAccessoryView = accessoryView;
  viewAppeared = NO;
  _aboutToAppear = NO;
}

- (UIStatusBarStyle)preferredStatusBarStyle
{
  return UIStatusBarStyleLightContent;
}

- (void)viewWillAppear:(BOOL)animated
{
  [super viewWillAppear:animated];
  _aboutToAppear = YES;
}

- (void)viewDidAppear:(BOOL)animated
{
  [super viewDidAppear:animated];
  viewAppeared = YES;
  _aboutToAppear = NO;
  [tuneViewController zoomToFit:self];
}

#pragma mark UITextViewDelegate delegate methods

#if TUNEBOOK_SYNTAX_HIGHLIGHTING
- (void)scrollViewDidScroll:(UIScrollView *)scrollView
{
	[scrollView setNeedsDisplay];
}
#endif

- (void)textViewDidChange:(UITextView *)aTextView
{
#if TUNEBOOK_SYNTAX_HIGHLIGHTING
  [aTextView setNeedsDisplay];
#endif
  changed = YES;
  [self previewPressed:self];
}

-(void)setText:(NSString*)text
{
  textView.text = text;
  tune.dots = text;
  [tuneViewController dotsChanged];
  changed = NO;
}

-(NSString *)text
{
  return textView.text;
}

-(void)transpose:(NSString*)aTune spec:(const char*)spec
{
  [self setText:[Tune transposeTune:aTune spec:spec]];
}

- (void)insertText:(NSString*)text
{
  [textView insertText:text];
}

- (IBAction)accessoryKeyPressed:(id)sender
{
  UIBarButtonItem* item = (UIBarButtonItem*)sender;
  NSString* text = item.title;
  if ( [text isEqualToString:@"\u2190"] ) {
    UITextRange *selectedRange = [textView selectedTextRange];
    UITextPosition *newPosition = [textView positionFromPosition:selectedRange.start offset:-1];
    UITextRange *newRange = [textView textRangeFromPosition:newPosition toPosition:newPosition];
    [textView setSelectedTextRange:newRange];
  }
  else if ( [text isEqualToString:@"\u2192"] ) {
    UITextRange *selectedRange = [textView selectedTextRange];
    UITextPosition *newPosition = [textView positionFromPosition:selectedRange.start offset:1];
    UITextRange *newRange = [textView textRangeFromPosition:newPosition toPosition:newPosition];
    [textView setSelectedTextRange:newRange];
  }
  else if ( [text isEqualToString:@":|"] ) {
    // For some reason, pasting a string starting with a colon pastes percent-escaped text
    [self insertText:@":"];
    [self insertText:@"|"];
  }
  else
    [self insertText:item.title];
}

- (IBAction)previewPressed:(id)sender
{
  tune.dots = self.text;
  [tuneViewController dotsChanged];
}

- (IBAction)cancelPressed:(id)sender
{
  viewAppeared = NO; // On iOS 5 we get loads of keyboard show/hide messages while dismissing
  [self dismissViewControllerAnimated:YES completion:nil];
}

- (IBAction)donePressed:(id)sender
{
  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
  numTunes = 0;
  parsedTunes = [[NSMutableArray alloc] init];
  [parsedTunes retain];
  TuneParser *parser = [[TuneParser alloc] init];
  parser.delegate = self;
  NSString *dots = textView.text;
  [parser parseString:dots];
  if ( numTunes == 0 ) {
    UIAlertController* alertController = [UIAlertController alertControllerWithTitle:@"Bad ABC" message:@"The ABC does not contain any valid tunes." preferredStyle:UIAlertControllerStyleAlert];
    [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil]];
    [self presentViewController:alertController animated:YES completion:nil];
    [parsedTunes release];
  } else if ( numTunes > 1 ) {
    UIAlertController* alertController = [UIAlertController
      alertControllerWithTitle:@"Multiple tunes"
      message:@"The ABC contains more than one tune."
      preferredStyle:UIAlertControllerStyleAlert];
    [alertController addAction:[UIAlertAction actionWithTitle:@"Cancel" style:UIAlertActionStyleDefault handler:nil]];
    [alertController
      addAction:[UIAlertAction
        actionWithTitle:@"Add Extra Tunes"
        style:UIAlertActionStyleDestructive
        handler:^(UIAlertAction *action) {
          [self addExtraTunes];
        }
      ]
    ];
    [self presentViewController:alertController animated:YES completion:nil];
  } else {
    Tune *t = (Tune *)[parsedTunes objectAtIndex:0];
    editTune.title = t.title;//FIXME-method to do this...
    editTune.dots = t.dots;
    editTune.rating = t.rating;
    editTune.key = t.key;
    editTune.numSharps = t.numSharps;
    editTune.timeSigNum = t.timeSigNum;
    editTune.timeSigDen = t.timeSigDen;
    if ( [editTune inDatabase] ) {
      [editTune dehydrate];
      [editTune hydrate];
    } else
      [appDelegate addTune:editTune];
    [parsedTunes release];
    [[NSNotificationCenter defaultCenter] postNotificationName:@"databaseChanged" object:self]; 
    [appDelegate showTune:editTune];
    [self dismissViewControllerAnimated:YES completion:nil];
  }
  
  [appDelegate sortTunes];
}

- (void)addExtraTunes
{
  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
  Tune *t = (Tune *)[parsedTunes objectAtIndex:0];
  editTune.title = t.title;
  editTune.dots = t.dots;
  editTune.rating = t.rating;
  editTune.key = t.key;
  editTune.numSharps = t.numSharps;
  editTune.timeSigNum = t.timeSigNum;
  editTune.timeSigDen = t.timeSigDen;
  if ( [editTune inDatabase] )
    [editTune dehydrate];
  else
    [appDelegate addTune:editTune];

  // Add the extra tunes
  for ( int i = 1; i < [parsedTunes count]; i++ )
    [appDelegate addTune:(Tune *)[parsedTunes objectAtIndex:i]];
  [[NSNotificationCenter defaultCenter] postNotificationName:@"databaseChanged" object:self];
  [parsedTunes release];
  [self dismissViewControllerAnimated:YES completion:nil];
}

- (void) parsedTune:(TuneParser *)parser tune:(Tune *)t
{
  numTunes++;
  [parsedTunes addObject:t];
}

@end
