#import "ImportProgressController.h"
#import "AppDelegate.h"
#import "Tune.h"
#import "TuneDatabase.h"
#import "PlaylistViewController.h"

@implementation ImportProgressController

enum {
  kTagNewTunebook = 1000
};

-(void)viewDidLoad
{
  animateDismiss = YES;
}

- (void)import:(NSURL *)url animateDismiss:(BOOL)animate
{
  animateDismiss = animate;
  progressView.progress = 0;
  _url = url;
  [_url retain];
  parsedTunes = [[NSMutableArray alloc] init];
  [parsedTunes retain];
  fragment = nil;
  lastWasCR = NO;
  _encoding = NSISOLatin1StringEncoding;
  titleLabel.text = [NSString stringWithFormat:@"Importing %@", [[_url path] lastPathComponent]];
  [NSThread detachNewThreadSelector:@selector(threadRun:) toTarget:self withObject:nil];
}

- (void)import:(NSURL *)url
{
  [self import:url animateDismiss:YES];
}

- (void) importRequest:(NSURLRequest *)request
{
  _request = [request retain];
  animateDismiss = YES;
  progressView.progress = 0;
  _url = [request URL];
  parsedTunes = [[NSMutableArray alloc] init];
  [parsedTunes retain];
  fragment = nil;
  lastWasCR = NO;
  _encoding = NSISOLatin1StringEncoding;
  titleLabel.text = [NSString stringWithFormat:@"Importing %@", [[_url path] lastPathComponent]];
  [NSThread detachNewThreadSelector:@selector(threadRun:) toTarget:self withObject:nil];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
  return YES;
}

- (void)didReceiveMemoryWarning
{
	[super didReceiveMemoryWarning];
}

-(void)dealloc
{
  [_url release];
  [_request release];
  [parsedTunes release];
  [fragment release];
	[super dealloc];
}

- (IBAction) onCancel:(id)sender
{
  cancelled = true;
  finished = true;
  if ( urlConnection ) {
    [urlConnection cancel];
    urlConnection = NULL;
  }
  [self connectionDidFinishLoading:NULL];
}

- (void)threadRun:(id)sender
{
  NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];  

  cancelled = false;
  finished = false;
  statusCode = 0;

  NSURLRequest *theRequest;
  if ( _request )
    theRequest = _request;
  else
    theRequest = [NSURLRequest requestWithURL:_url cachePolicy:NSURLRequestReloadIgnoringLocalCacheData timeoutInterval:60];

  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
  [appDelegate startTransaction];
  parser = [[TuneParser alloc] init];
  parser.delegate = self;

  _numTunes = 0;
  _numDuplicateTunes = 0;
  bytesRead = 0;
  bytesProcessed = 0;
  _encoding = NSISOLatin1StringEncoding;

  urlConnection = [NSURLConnection connectionWithRequest:theRequest delegate:self];
  if ( urlConnection == nil ) {
    UIAlertController* alertController = [UIAlertController
      alertControllerWithTitle:@"Unable to initiate request"
      message:@"NSURLConnection initialization method failed"
      preferredStyle:UIAlertControllerStyleAlert];
    [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil]];
    [self presentViewController:alertController animated:YES completion:nil];
  }

  BOOL done = NO;
  do {
    SInt32 result = CFRunLoopRunInMode( kCFRunLoopDefaultMode, 1, YES );
    if ( (result == kCFRunLoopRunStopped) || (result == kCFRunLoopRunFinished) ) {
      done = YES;
    }
    if ( cancelled || finished )
      done = YES;
  } while ( !done );

  // If it was a local file, delete it when finished
  if ( [_url isFileURL] ) {
    NSFileManager *fileManager = [NSFileManager defaultManager];
    [fileManager removeItemAtPath:[_url path] error:nil];
  }

  [pool release];
}

#pragma mark UIViewController delegate methods

- (void)viewWillAppear:(BOOL)animated
{
  [super viewWillAppear:animated];
}

#pragma mark NSURLConnection delegate methods

- (void) connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response
{
  bytesToRead = (long)[response expectedContentLength];
  if ( [response isKindOfClass:[NSHTTPURLResponse class]] ) {
    statusCode = [(NSHTTPURLResponse *)response statusCode];
    if ( statusCode >= 400 )
      cancelled = true;
  }
}

- (void) updateProgress:(id)sender
{
  progressView.progress = (float)bytesProcessed / bytesToRead;
  statusLabel.text = [NSString stringWithFormat:@"Read %.1fKb, found %lu tunes", (float)bytesProcessed/1024, (unsigned long)_numTunes];
}

- (void) connection:(NSURLConnection *)connection didReceiveData:(NSData *)data
{
  const char *b = (const char *)data.bytes;
  int length = (int)data.length;
  int start = 0;

  if ( cancelled )
    return;

  bytesRead += length;

  NSInteger n = _numTunes;
  int i = 0;
  for ( ; i < length && !cancelled; i++ ) {
    bytesProcessed++;
    if ( lastWasCR && b[i] == '\n' ) {
      lastWasCR = NO;
      start = i+1;
    } else if ( b[i] == '\r' || b[i] == '\n' ) {
      NSString *line = [[NSString alloc] initWithBytes:b+start length:i-start encoding:_encoding];
      if ( [line hasPrefix:@"%%abc-charset"] ) {
        NSString* charSet = [line substringFromIndex:13];
        charSet = [charSet stringByTrimmingCharactersInSet:
                                  [NSCharacterSet whitespaceCharacterSet]];
        if ( [charSet isEqualToString:@"utf-8"] )
          _encoding = NSUTF8StringEncoding;
        else if ( [charSet isEqualToString:@"iso-8859-1"] )
          _encoding = NSISOLatin1StringEncoding;
        else if ( [charSet isEqualToString:@"iso-8859-2"] )
          _encoding = NSISOLatin2StringEncoding;
        else if ( [charSet isEqualToString:@"us-ascii"] )
          _encoding = NSASCIIStringEncoding;
      }

      if ( line == nil )
        line = [[NSString alloc] initWithBytes:b+start length:i-start encoding:NSUTF8StringEncoding];
      if ( fragment ) {
        [fragment appendString:line];
        [parser parseLine:fragment];
        [fragment release];
        fragment = nil;
      } else
        [parser parseLine:line];
      if ( _numTunes >= n+10 ) {
        [self performSelectorOnMainThread:@selector(updateProgress:) withObject:nil waitUntilDone:NO];
        n = _numTunes;
      }

      [line release];
      if ( b[i] == '\r' )
        lastWasCR = YES;
      start = i+1;
    }
  }
  if ( start != i ) {
    if ( fragment == nil )
      fragment = [[NSMutableString alloc] init];
    
    NSString *s = [[NSString alloc] initWithBytes:b+start length:i-start encoding:_encoding];
    if ( s == nil )
      s = [[NSString alloc] initWithBytes:b+start length:i-start encoding:NSUTF8StringEncoding];
    [fragment appendString:s];
    [s release];
  }

  [self performSelectorOnMainThread:@selector(updateProgress:) withObject:nil waitUntilDone:NO];
}

- (void) connection:(NSURLConnection *)connection didFailWithError:(NSError *)error
{
  UIAlertController* alertController = [UIAlertController alertControllerWithTitle:@"Import Failed" message:[error localizedDescription] preferredStyle:UIAlertControllerStyleAlert];
  [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil]];
  [self presentViewController:alertController animated:YES completion:nil];
	[connection release];
  [self dismissViewControllerAnimated:YES completion:nil];
}

- (NSCachedURLResponse *) connection:(NSURLConnection *)connection willCacheResponse:(NSCachedURLResponse *)cachedResponse
{
  return nil;
}

- (void) connectionDidFinishLoading:(NSURLConnection *)connection
{
  // Deal with files which don't have a trailing newline
  if ( fragment ) {
    [fragment appendString:@"\n"];
    [parser parseLine:fragment];
    [fragment release];
    fragment = nil;
  }
  [parser parseLine:@""]; // Force the last tune to end
  [self performSelectorOnMainThread:@selector(finishedImport:) withObject:nil waitUntilDone:NO];
}

- (void) finishedImport:(id)sender
{
  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
  [appDelegate commitTransaction];

  NSString* filename = [[_url path] lastPathComponent];

  if ( statusCode >= 400 ) {
    UIAlertController* alertController = [UIAlertController
      alertControllerWithTitle:@"Import Failed"
      message:[NSString stringWithFormat:@"An error (%lu) occurred when importing %@: %@", (unsigned long)statusCode, filename, [NSHTTPURLResponse localizedStringForStatusCode:statusCode]]
      preferredStyle:UIAlertControllerStyleAlert];
    [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil]];
    [self presentViewController:alertController animated:YES completion:nil];
  } else if ( _numTunes != 0 || _numDuplicateTunes != 0 ) {
    NSString* message = nil;
    if ( _numDuplicateTunes != 0 ) {
      message = [NSString stringWithFormat:@"Imported %lu new tunes from %@, and ignored %lu you already have.\nCreate a tunebook with these tunes?", (unsigned long)_numTunes, filename, (unsigned long)_numDuplicateTunes];
    }
    else {
      message = [NSString stringWithFormat:@"Imported %lu tunes.\nCreate a tunebook for them?", (unsigned long)_numTunes];
    }
    UIAlertView *alert = [[UIAlertView alloc]
      initWithTitle: cancelled ? @"Import Stopped" : @"Import Succeeded" 
      message:message
      delegate:self 
      cancelButtonTitle:@"No"
      otherButtonTitles:@"Create", nil]; 
    alert.tag = kTagNewTunebook;
    [alert setAlertViewStyle:UIAlertViewStylePlainTextInput];
    UITextField *textfield = [alert textFieldAtIndex: 0];
    [textfield setText:[[_url path] lastPathComponent]];
    [alert show];
    [alert release];

    [[NSNotificationCenter defaultCenter] postNotificationName:@"databaseChanged" object:self]; 
  } else if ( _numDuplicateTunes != 0 ) {
    UIAlertController* alertController = [UIAlertController
      alertControllerWithTitle:@"Duplicate Tunes"
      message:[NSString stringWithFormat:@"The file %@ contained %lu tunes you already have and no new tunes, so no tunes have been imported.", filename, (unsigned long)_numDuplicateTunes]
      preferredStyle:UIAlertControllerStyleAlert];
    [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil]];
    [self presentViewController:alertController animated:YES completion:nil];
  } else {
    UIAlertController* alertController = [UIAlertController
      alertControllerWithTitle:@"Import Failed"
      message:[NSString stringWithFormat:@"The file %@ was imported, but appeared to contain no tunes.", filename]
      preferredStyle:UIAlertControllerStyleAlert];
    [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil]];
    [self presentViewController:alertController animated:YES completion:nil];
  }

  finished = true;
}

#pragma mark TuneParser delegate methods

-(void) parsedTune:(TuneParser *)parser tune:(Tune *)tune
{
  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
#if TUNEBOOK_CHECK_DUPLICATES
  if ( appDelegate.noImportDuplicates ) {
    Tune* duplicateTune = [appDelegate.database searchForTune:tune];
    if ( duplicateTune != nil ) {
      _numDuplicateTunes ++;
      [parsedTunes addObject:duplicateTune];
      return;
    }
  }
#endif
  [appDelegate addTune:tune];
  [parsedTunes addObject:tune];
  _numTunes++;
}

#pragma mark UIAlertView delegate methods

- (void)alertView:(UIAlertView *)alertView didDismissWithButtonIndex:(NSInteger)buttonIndex
{
  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
  if ( alertView.tag == kTagNewTunebook && buttonIndex == 1 && parsedTunes && [parsedTunes count] != 0 ) {
    Playlist *playlist = [[Playlist alloc] init];
    UITextField *textfield = [alertView textFieldAtIndex: 0];
    playlist.name = textfield.text;
    if ( [playlist.name length] == 0 )
      playlist.name = @"Imported Tunes";
    [appDelegate startTransaction];
    [appDelegate addPlaylist:playlist];
    for ( Tune* tune in parsedTunes )
      [playlist addTune:tune];
    [appDelegate commitTransaction];
    [parsedTunes release];
    parsedTunes = nil;
    [[NSNotificationCenter defaultCenter] postNotificationName:@"playlistsChanged" object:self]; 
  }

  [self dismissViewControllerAnimated:animateDismiss completion:nil];
  [[NSNotificationCenter defaultCenter] postNotificationName:@"importedFile" object:nil]; 
  [appDelegate sortTunes];
}

@end
