#import "KeyboardHandler.h"

@implementation KeyboardHandler

@synthesize delegate;

static NSString* kKeyboardText = @"\nab\ncd\nef\n";

- (void) dealloc
{
  [super dealloc];
}

- (BOOL)textView:(UITextView *)textView shouldChangeCharactersInRange:(NSRange)range replacementString:(NSString *)string
{
  [[self delegate] didReceiveKey:string];
  return false;
}

- (BOOL)textViewShouldClear:(UITextView *)textView
{   
  return false;
}

- (BOOL)textViewShouldBeginEditing:(UITextView *)textView
{   
  return true;
}

- (BOOL)textViewShouldEndEditing:(UITextView *)textView
{   
  return true;
}

- (void)resetTextView
{
  _handleArrows = NO; // Prevent recursion
  if ( ![_textView.text isEqualToString:kKeyboardText] )
    _textView.text = kKeyboardText;
  _textView.selectedRange = NSMakeRange(5, 0);
  _handleArrows = YES;
}

- (void)textViewDidChangeSelection:(UITextView *)textView
{
  if ( _handleArrows ) {
    NSUInteger selectionLocation = textView.selectedRange.location;

    if (selectionLocation == 4)
      [delegate didReceiveArrowKey:kArrowKeyLeft];
    else if (selectionLocation == 6)
      [delegate didReceiveArrowKey:kArrowKeyRight];
    else if (selectionLocation == 2)
      [delegate didReceiveArrowKey:kArrowKeyUp];
    else if (selectionLocation == 8)
      [delegate didReceiveArrowKey:kArrowKeyDown];

    [self performSelector:@selector(resetTextView) withObject:nil afterDelay:0.0];
  }
}

-(void)makeFirstResponder
{
  [_textView resignFirstResponder];
  [self resetTextView];
  [_textView becomeFirstResponder];
}

-(id)initWithDelegate:(id)aDelegate
{
  self = [super init];
	
	if (self) {
    delegate = aDelegate;
    _inputView = [[UIView alloc] initWithFrame:CGRectZero];
    _textView = [[UITextView alloc] initWithFrame:CGRectMake(0, 0, 100, 100)];
    [_textView setInputView:_inputView];
		_textView.autocorrectionType = UITextAutocorrectionTypeNo;
		_textView.autocapitalizationType = UITextAutocapitalizationTypeNone;
		_textView.scrollsToTop = NO;
    // Prevent the shortcuts bar showing for this hidden text field
    if ([_textView respondsToSelector:@selector(inputAssistantItem)]) {
      UITextInputAssistantItem *inputAssistantItem = [_textView inputAssistantItem];
      inputAssistantItem.leadingBarButtonGroups = nil;
      inputAssistantItem.trailingBarButtonGroups = nil;
    }
    _handleArrows = NO; // Prevent recursion
    _textView.delegate = self;
    _textView.hidden = YES;
    [self addSubview:_textView];
    [_textView becomeFirstResponder];
    _handleArrows = YES;
	}
	
	return self;
}

@end
