#import "MainViewController.h"
#import "TuneViewController.h"
#import "PlaylistViewController.h"
#import "AppDelegate.h"
#import "Tune.h"

#import "TopLevelViewController.h"
#import "TunebookCell.h"
#import "SearchViewController.h"
#import "SettingsController.h"
#import "ManageTunesController.h"
#import "TuneSettingsController.h"
#import "EditABCController.h"

#define kCellIdentifier @"com.jhlabs.Tunebook.TuneCellIdentifier"
#define kAddCellIdentifier @"com.jhlabs.Tunebook.AddTuneCellIdentifier"

#if TUNEBOOK_MULTI_SELECT
#define kHasAddTuneRow NO  // We show the "Add New Tune" row when editing.
#else
#define kHasAddTuneRow YES  // We show the "Add New Tune" row when editing.
#endif

#define kActionSection 0
#define kAllTunesRow 0
#define kAddTunesRow 0

enum {
  kActionSheetNone,
  kActionSheetBookmark,
  kActionSheetMove,
};

enum {
  kAlertAddToTunebook,
  kAlertConfirmDelete,
};


// The MainViewController displays a list of tunes from a Playlist, or all tunes if
// playlist is nil.
// The UITableView always has at least two sections: section 0 has operations such as "Add Tunes" or "All Tunes" and the tunes are in the following sections.

@implementation MainViewController

@synthesize listContent, filteredListContent, tableView, searchBar, choosingTunes;

static MainViewController* sActiveController;

- (CGSize)contentSizeForViewInPopoverView {
  return CGSizeMake(320.0, 440.0);
}

- (void)dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[tableView release];
	[searchBar release];
	[listContent release];
	[filteredListContent release];
	[playItems release];
	[stopItems release];
  [sectionIndexTitles release];

  [super dealloc];
}

-(void)awakeFromNib
{
  [super awakeFromNib];
//  if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
//    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(databaseChanged:) name:@"databaseChanged" object:nil];
}

-(void)viewDidLoad
{
  transitionDirection = 1;

  if ( [self respondsToSelector:@selector(setEdgesForExtendedLayout:)] )
    self.edgesForExtendedLayout = UIRectEdgeNone;

	searchBar.autocorrectionType = UITextAutocorrectionTypeNo;
	searchBar.autocapitalizationType = UITextAutocapitalizationTypeNone;
	searchBar.showsCancelButton = UI_USER_INTERFACE_IDIOM() != UIUserInterfaceIdiomPad;

  // iOS 7: Changed height of segmented control
  if (floor(NSFoundationVersionNumber) <= NSFoundationVersionNumber_iOS_6_1) {
    CGRect r = tableView.bounds;
    r.size.height++;
    tableView.bounds = r;
  }

  tableView.rowHeight = 36;
  tableView.scrollsToTop = YES;

  searchBar.userInteractionEnabled = YES;
  manageButton.enabled = playlist != nil;

  if ( playlist == nil || [playlist isEditable] ) {
    addTuneButton.enabled = YES;
    self.navigationItem.rightBarButtonItem = self.editButtonItem;
  }
  else {
    addTuneButton.enabled = NO;
    self.navigationItem.rightBarButtonItem = nil;
  }
  [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(databaseChanged:) name:@"databaseChanged" object:nil];
  [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(startedPlaying:) name:@"startedPlaying" object:nil];
  [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(finishedPlaying:) name:@"finishedPlaying" object:nil];

  if ( sortOrder != nil )
    sortOrder.selectedSegmentIndex = [self tuneSortOrder];

  if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
    UISwipeGestureRecognizer *recognizer1;
    recognizer1 = [[UISwipeGestureRecognizer alloc] initWithTarget:self action:@selector(swipeLeft:)];
    [recognizer1 setDirection:UISwipeGestureRecognizerDirectionLeft];
    [[self view] addGestureRecognizer:recognizer1];
    [recognizer1 release];

    // Set up the arrays of toolbar items we'll need
    NSArray *items = toolbar.items;
    playItems = [NSMutableArray arrayWithArray:items];
    [playItems removeObject:playButton];
    [playItems retain];
    stopItems = [NSMutableArray arrayWithArray:items];
    [stopItems removeObject:stopButton];
    [stopItems retain];
    toolbar.items = stopItems;
  }

#if TUNEBOOK_MULTI_SELECT
  [self enableButtons];
#endif
}

-(void)viewDidAppear:(BOOL)animated
{
  [super viewDidAppear:animated];
  if ( sActiveController != self ) {
    [sActiveController release];
    sActiveController = [self retain];
  }

  if ( playlist ) {
    AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
    [appDelegate showTunes:filteredListContent title:playlist.name];
  }
}

-(void)viewDidDisappear:(BOOL)animated
{
  [super viewDidDisappear:animated];
  [sActiveController release];
  sActiveController = nil;
}

+ (MainViewController*)activeController
{
  return sActiveController;
}

- (Tune*)selectedTune
{
  NSIndexPath* indexPath = [tableView indexPathForSelectedRow];
  if ( indexPath && indexPath.section != kActionSection ) {
    NSInteger row = indexPath.row;

    row += sectionIndex[indexPath.section];
    return (Tune *)[filteredListContent objectAtIndex:row];
  }
  return nil;
}

- (NSArray*)selectedTunes
{
  NSArray* selectedRows = [tableView indexPathsForSelectedRows];
  NSMutableArray* tunes = [NSMutableArray arrayWithCapacity:[selectedRows count]];
  NSIndexPath* indexPath;
  for (indexPath in selectedRows) {
    Tune* tune = [self tuneForIndexPath:indexPath];
    if ( tune != nil )
      [tunes addObject:tune];
  }
  return tunes;
}

- (void)selectTune:(Tune*)aTune
{
#if 1 //FIXME
  if ( aTune != nil ) {
    NSInteger index = sectionIndex[1];
    NSInteger numSections = [self numberOfSectionsInTableView:tableView];
    for ( int section = 1; section < numSections; section++ ) {
      for ( int row = 0; row < sectionIndex[section+1]-sectionIndex[section]; row++ ) {
        Tune* tune = [filteredListContent objectAtIndex:index];
        if ( tune == aTune ) {
          NSIndexPath* indexPath = [NSIndexPath indexPathForRow:row inSection:section];
          [tableView selectRowAtIndexPath:indexPath animated:YES scrollPosition:UITableViewScrollPositionMiddle];
//        [[tableView delegate] tableView:tableView didSelectRowAtIndexPath:indexPath];
          return;
        }
        index++;
      }
    }
  }
#endif
}

- (IBAction)prevTune:(id)sender
{
  NSIndexPath* indexPath = [tableView indexPathForSelectedRow];
  if ( indexPath ) {
    if ( indexPath.row <= 0 ) {
      NSInteger numRows;
      do {
        numRows = [tableView numberOfRowsInSection:indexPath.section-1];
        indexPath = [NSIndexPath indexPathForRow:numRows-1 inSection:indexPath.section-1];
      } while ( numRows == 0 && indexPath.section > 0 );
      if ( indexPath.section == 0 )
        return;
    }
    else
      indexPath = [NSIndexPath indexPathForRow:indexPath.row-1 inSection:indexPath.section];

    [tableView selectRowAtIndexPath:indexPath animated:NO scrollPosition:UITableViewScrollPositionMiddle];
    transitionDirection = -1;
    [self tableView:tableView didSelectRowAtIndexPath:indexPath];
    transitionDirection = 1;
    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
      AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
      [appDelegate.tuneViewController showMessage:[NSString stringWithFormat:@"Tune %d of %d", (int)indexPath.row+1, (int)[filteredListContent count]]];
    }
  }
}

- (IBAction)nextTune:(id)sender
{
  NSIndexPath* indexPath = [tableView indexPathForSelectedRow];
  if ( indexPath ) {
    NSInteger numRows = [tableView numberOfRowsInSection:indexPath.section];
    if ( indexPath.row >= numRows-1 ) {
      do {
        indexPath = [NSIndexPath indexPathForRow:0 inSection:indexPath.section+1];
        numRows = [tableView numberOfRowsInSection:indexPath.section];
      } while ( numRows == 0 && indexPath.section < [tableView numberOfSections]-1 );
      if ( indexPath.section >= [tableView numberOfSections] )
        return;
    }
    else
      indexPath = [NSIndexPath indexPathForRow:indexPath.row+1 inSection:indexPath.section];

    [tableView selectRowAtIndexPath:indexPath animated:NO scrollPosition:UITableViewScrollPositionMiddle];
    [self tableView:tableView didSelectRowAtIndexPath:indexPath];
    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
      AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
      [appDelegate.tuneViewController showMessage:[NSString stringWithFormat:@"Tune %d of %d", (int)indexPath.row+1, (int)[filteredListContent count]]];
    }
  }
}

- (void)swipeLeft:(UISwipeGestureRecognizer *)swipeGesture
{
  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
  [appDelegate.topLevelViewController showLeftView:NO];
}

- (void)databaseChanged:(id)sender {
  if ( playlist )
    self.listContent = [playlist loadTunes];
  [self filterTunes];
}

- (void)setPlaylist:(Playlist *)p
{
  Playlist *old = playlist;
  playlist = p;
  [playlist retain];
  [old release];

  if (UI_USER_INTERFACE_IDIOM() != UIUserInterfaceIdiomPad)
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(databaseChanged:) name:@"databaseChanged" object:nil];

  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
  manageButton.enabled = YES;
  loading = NO;

	searchBar.autocorrectionType = UITextAutocorrectionTypeYes;
	searchBar.autocapitalizationType = UITextAutocapitalizationTypeAllCharacters;

  tableView.rowHeight = 36;
  tableView.scrollsToTop = YES;

  if ( playlist ) {
    self.listContent = [playlist loadTunes];

    searchBar.userInteractionEnabled = YES;
    manageButton.enabled = YES;
    if ( addTuneButton != nil )
      addTuneButton.enabled = [playlist isEditable];
    loading = NO;

    self.navigationItem.title = playlist.name;
    if ( [playlist isEditable] )
      self.navigationItem.rightBarButtonItem = self.editButtonItem;
    else
      self.navigationItem.rightBarButtonItem = nil;
  } else {
    self.navigationItem.title = @"All Tunes";

    self.listContent = appDelegate.tunes;
    addTuneButton.enabled = YES;
  }

  [filteredListContent release];
  filteredListContent = [[NSMutableArray alloc] initWithCapacity: [listContent count]];
  [self filterTunes];
//  [filteredListContent addObjectsFromArray: listContent];
//  [self buildSectionIndex];
//
//	[tableView reloadData];
}

- (Playlist *)playlist
{
  return playlist;
}

- (void)filterTunes
{
  [filteredListContent removeAllObjects];

  NSString* searchText = searchBar.text;
  if ( [searchText length] != 0 ) {
    AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
    [appDelegate searchForTunes:searchText tunes:listContent results:filteredListContent];
  }
  else {
    [filteredListContent addObjectsFromArray: listContent];
  }
  int index = 0;
  for ( Tune* tune in listContent )
    tune.order = ++index;

  if ( playlist && [self tuneSortOrder] != kSortTunesByOrder )
    [Tune sortTunes:filteredListContent by:(SortOrder)/*FIXME*/[self tuneSortOrder]];
  [self buildSectionIndex];
	[tableView reloadData];
}

- (void)setEditing:(BOOL)editing animated:(BOOL)animated
{
  [super setEditing:editing animated:animated];
#if TUNEBOOK_MULTI_SELECT
  tableView.allowsMultipleSelectionDuringEditing = editing;
#endif
  if ( kHasAddTuneRow ) {
    NSArray *indexPaths = [NSArray arrayWithObjects:
      [NSIndexPath indexPathForRow:0 inSection:0],
      nil
    ];
    [tableView beginUpdates];
    [tableView setEditing:editing animated:YES];
    if ( !playlist ) {
      if ( editing ) {
        // Show the placeholder rows
        [tableView insertRowsAtIndexPaths:indexPaths withRowAnimation:UITableViewRowAnimationTop];
      } else {
        // Hide the placeholder rows.
        [tableView deleteRowsAtIndexPaths:indexPaths withRowAnimation:UITableViewRowAnimationTop];
      }
    } else
      [tableView reloadData];
    [tableView endUpdates];
  }
  else {
    [tableView setEditing:editing animated:YES];
  }
//FIXME-we don't need this any more?  searchBar.userInteractionEnabled = !editing;
  self.navigationItem.title = playlist.name;
#if TUNEBOOK_MULTI_SELECT
  // Refresh the first row (the "42 Tunes" row)
  if ( playlist )
    [tableView reloadRowsAtIndexPaths:@[[NSIndexPath indexPathForRow:kAllTunesRow inSection:kActionSection]] withRowAnimation:UITableViewRowAnimationNone];

  //FIXME-iPhone version of this...
  if ( editing ) {
    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
      editingToolbar.frame = toolbar.frame;
      [self.view addSubview:editingToolbar];
    }
    else {
      AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
      if ( playlist ) {
        CGRect r = tableView.frame;
        CGRect tr = editingToolbar.frame;
        tr.size.width = r.size.width;
        r.size.height -= tr.size.height;
        tr.origin.y = r.origin.y + r.size.height;
        tableView.frame = r;
        editingToolbar.frame = tr;
//        tableView.tableFooterView = editingToolbar;
        [self.view addSubview:editingToolbar];
      }
      else {
        editingToolbar.frame = appDelegate.tabBar.frame;
        [[appDelegate.tabBar superview] addSubview:editingToolbar];
      }
    }
  }
  else {
    if (UI_USER_INTERFACE_IDIOM() != UIUserInterfaceIdiomPad && playlist ) {
//      tableView.tableFooterView = nil;
        CGRect r = tableView.frame;
        CGRect tr = editingToolbar.frame;
        r.size.height += tr.size.height;
        tableView.frame = r;
        [editingToolbar removeFromSuperview];
    }
    else
      [editingToolbar removeFromSuperview];
  }
  [self enableButtons];
#endif
}

#pragma mark UIViewController delegate

- (void)viewWillAppear:(BOOL)animated
{
  [super viewWillAppear:animated];
//	[[UIApplication sharedApplication] setStatusBarStyle:UIStatusBarStyleDefault animated:YES];
  if ( playlist && !playlist.saved )
    self.navigationItem.rightBarButtonItem = bookmarkButtonItem;
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
  return YES;
}

- (BOOL)shouldAutorotate {
  return YES;
}

- (UIInterfaceOrientationMask)supportedInterfaceOrientations {
  return UIInterfaceOrientationMaskAll;
}

- (int)tuneSortOrder
{
  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
  return [appDelegate sortOrderForPlaylist:playlist];
}

#pragma mark UITableView delegate methods

- (void)tableView:(UITableView *)tv didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
	if ( [searchBar isFirstResponder] )
    [searchBar resignFirstResponder];

#if TUNEBOOK_MULTI_SELECT
  // Don't push to the tune view on iPhone
  if ( [self isEditing] && UI_USER_INTERFACE_IDIOM() != UIUserInterfaceIdiomPad) {
    [self enableButtons];
    return;
  }
#endif

  if (UI_USER_INTERFACE_IDIOM() != UIUserInterfaceIdiomPad)
    [tv deselectRowAtIndexPath:indexPath animated:YES];
  NSInteger row = indexPath.row;

  if ( playlist ) {
    // Did the user hit the "All Tunes" row?
    if ( indexPath.section == kActionSection ) {
      if ( indexPath.row == kAllTunesRow ) {
        AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
        [appDelegate showTunes:filteredListContent title:playlist.name];
      }
      return;
    }
  }

  row += sectionIndex[indexPath.section];

  Tune *tune = (Tune *)[filteredListContent objectAtIndex:row];
  if ( choosingTunes ) {
    tune.selected = !tune.selected;
    [tableView reloadData];
  } else {
#if TUNEBOOK_MULTI_SELECT
    if ( [self isEditing] && UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
      [self showSelectedTunes];
      [self enableButtons];
      return;
    }
#endif
    AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
      [appDelegate.tuneViewController showTune:tune fromPlayList:playlist direction: transitionDirection];
    else {
      TuneViewController *controller = [[TuneViewController alloc] initWithTune:tune fromPlayList:playlist];
      [appDelegate.navigationController pushViewController:controller animated:YES];
      [controller autorelease];
    }
  }

#if TUNEBOOK_MULTI_SELECT
  [self enableButtons];
#endif
}

- (void)showSelectedTunes
{
  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
  NSArray* selectedRows = [tableView indexPathsForSelectedRows];
  NSInteger count = [selectedRows count];
  if ( count == 1 )
    [appDelegate showTune:[self selectedTune]];
  else if ( count > 1 )
    [appDelegate showTunes:[self selectedTunes] title:@"Multiple Tunes"];
}

- (void)tableView:(UITableView *)tableView didDeselectRowAtIndexPath:(NSIndexPath *)indexPath;
{
#if TUNEBOOK_MULTI_SELECT
  [self enableButtons];
  if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
    [self showSelectedTunes];
#endif
}

- (NSIndexPath *)tableView:(UITableView *)tableView willSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
  if ( playlist && indexPath.section == kActionSection && indexPath.row == kAllTunesRow && [filteredListContent count] == 0 )
    return nil;
#if TUNEBOOK_MULTI_SELECT
  if ( playlist && indexPath.section == kActionSection && [self isEditing] )
    return nil;
#endif
  return indexPath;
}

#if TUNEBOOK_MULTI_SELECT
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath {

  if ( playlist && indexPath.section == kActionSection && [self isEditing] )
    return NO;

  return YES;
}

- (void)enableButtons
{
  NSArray* selectedRows = [tableView indexPathsForSelectedRows];
  bool selected = [self isEditing] && [selectedRows count] > 0;
  [_editMoveButton setEnabled:selected];
  [_editDeleteButton setEnabled:selected];
}
#endif

#pragma mark UITableView data source methods

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
  return sectionCount;
}

- (NSInteger)tableView:(UITableView *)aTableView numberOfRowsInSection:(NSInteger)section
{
  if ( loading )
    return 0;

  if ( section == kActionSection ) {
    if ( !kHasAddTuneRow ) {
      if ( playlist )
        return 1;
    }
    else {
      if ( [aTableView isEditing] || playlist )
        return 1;
    }
    return 0;
  }

  return sectionIndex[section+1] - sectionIndex[section];
}

#define ROW_HEIGHT 24
#define kTitleTag 1
#define kKeyTag 2
#define kTimeTag 3

- (UITableViewCell *)tableView:(UITableView *)tv cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
	UITableViewCell *cell;
  NSInteger row = indexPath.row;

  if ( indexPath.section == kActionSection ) {
    cell = [tableView dequeueReusableCellWithIdentifier:kAddCellIdentifier];

    if ( cell == nil ) {
      cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:kAddCellIdentifier] autorelease];
    }
  } else {
    cell = [tableView dequeueReusableCellWithIdentifier:kCellIdentifier];
	
    if ( cell == nil )
      cell = [self tableviewCellWithReuseIdentifier:kCellIdentifier];
  }
	
  if ( indexPath.section == kActionSection ) {
    if ( kHasAddTuneRow && self.editing && row == kAddTunesRow ) {
      cell.textLabel.text = playlist ? @"Add tunes" : @"Add new tune";
      cell.accessoryType = UITableViewCellAccessoryNone;
    } else if ( (!kHasAddTuneRow || !self.editing) && playlist && row == 0 ) {
      NSInteger count = [filteredListContent count];
#if TUNEBOOK_MULTI_SELECT
      if ( [self isEditing] ) {
        cell.textLabel.text = [NSString stringWithFormat:@"%d Tunes", (int)count];
      }
      else
#endif
      if ( count == 0 ) {
        cell.textLabel.text = [NSString stringWithFormat:@"No Tunes"];
        cell.accessoryType = UITableViewCellAccessoryNone;
      } else if ( count == 1 ) {
        cell.textLabel.text = [NSString stringWithFormat:@"1 Tune"];
        cell.accessoryType = UITableViewCellAccessoryNone;
      } else if ( count <= kMaxTunesInSet ) {
        cell.textLabel.text = [NSString stringWithFormat:@"All %d Tunes", (int)count];
        cell.accessoryType = UITableViewCellAccessoryNone;
      } else {
        cell.textLabel.text = [NSString stringWithFormat:@"First %d Tunes (of %d)", kMaxTunesInSet, (int)count];
        cell.accessoryType = UITableViewCellAccessoryNone;
      }
//      cell.textLabel.textColor = [UIColor darkGrayColor];
    }
	} else {
    row += sectionIndex[indexPath.section];
    Tune *tune = (Tune *)[filteredListContent objectAtIndex:row];
    UILabel *label;
    label = (UILabel *)[cell viewWithTag:kTitleTag];
    label.text = tune.title;

    label = (UILabel *)[cell viewWithTag:kKeyTag];
    label.text = tune.key;
    if ( tune.timeSigDen != 0 )
      label.text = [NSString stringWithFormat:@"%d/%d %@", tune.timeSigNum, tune.timeSigDen, label.text];

    cell.textLabel.text = @"";
    if ( choosingTunes )
      cell.accessoryType = tune.selected ? UITableViewCellAccessoryCheckmark : UITableViewCellAccessoryNone;
    else
      cell.accessoryType = UITableViewCellAccessoryNone;
  }

	return cell;
}

- (UITableViewCellEditingStyle)tableView:(UITableView *)tv editingStyleForRowAtIndexPath:(NSIndexPath *)indexPath
{
  if ( self.editing ) {
    if ( indexPath.section == kActionSection ) {
      if ( kHasAddTuneRow )
        return UITableViewCellEditingStyleInsert;
      return UITableViewCellEditingStyleNone;
    }
  }
  return UITableViewCellEditingStyleDelete;
}

- (void)tableView:(UITableView *)tv commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath
{
  if (editingStyle == UITableViewCellEditingStyleDelete) {
    NSInteger row = indexPath.row;
    row += sectionIndex[indexPath.section];
    Tune *tune = (Tune *)[filteredListContent objectAtIndex:row];
    if ( playlist ) {
      [playlist removeTune:tune];
    } else {
      AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
      [appDelegate removeTune:tune];
    }
    [listContent removeObject:tune];
    [filteredListContent removeObjectAtIndex:row];
    [self buildSectionIndex];
    [tv reloadData];
    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
      AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
      if ( appDelegate.tuneViewController.tune == tune )
        [appDelegate.tuneViewController showTune:nil];
    }
  } else if (editingStyle == UITableViewCellEditingStyleInsert) {
    [self addNewTune:self];
	}
}

- (UITableViewCell *)tableviewCellWithReuseIdentifier:(NSString *)identifier
{
  CGRect rect;
  CGRect bounds = [tableView bounds];
	
	UITableViewCell *cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:identifier] autorelease];
	
	UILabel *label;
	
	rect = CGRectMake( 10, 2, bounds.size.width-32, 18 );
	label = [[UILabel alloc] initWithFrame:rect];
	label.tag = kTitleTag;
	label.font = [UIFont boldSystemFontOfSize:15];
  label.opaque = NO;
	label.adjustsFontSizeToFitWidth = NO;
	[cell.contentView addSubview:label];
	label.highlightedTextColor = [UIColor whiteColor];
	label.backgroundColor = [UIColor clearColor];
	[label release];
	
	rect = CGRectMake( 10, 22, bounds.size.width-32, 10);
	label = [[UILabel alloc] initWithFrame:rect];
	label.tag = kKeyTag;
	label.font = [UIFont systemFontOfSize:12];
	label.adjustsFontSizeToFitWidth = NO;
  label.opaque = NO;
	[cell.contentView addSubview:label];
	label.highlightedTextColor = [UIColor whiteColor];
	label.backgroundColor = [UIColor clearColor];
	[label release];

	return cell;
}

- (void)buildSectionIndex
{
  int order = [self tuneSortOrder];
  sectionIndexTitles = [NSMutableArray arrayWithCapacity:27];
  [sectionIndexTitles retain];

  sectionIndex[0] = 0;
  sectionCount = 1;
  if ( order == kSortTunesByOrder ) {
    sectionCount++;
  }

  NSString* lastKey = @"";
  int lastNum = 0;
  int lastDen = 0;
  NSInteger count = [filteredListContent count];
  for ( int i = 0; i < count; i++ ) {
    Tune *tune = (Tune *)[filteredListContent objectAtIndex:i];
    if ( order == kSortTunesByTitle ) {
      NSString* title = [tune title];
      if ( [title length] > 0 ) {
        title = [title substringToIndex:1];
        if ( [title caseInsensitiveCompare:lastKey] != NSOrderedSame ) {
          if ( sectionCount < 100 ) {
            [sectionIndexTitles addObject:title];
            sectionIndex[sectionCount++] = i;
          }
        }
        lastKey = title;
      }
    }
    else if ( order == kSortTunesByKey ) {
      NSString* key = [tune key];
      if ([key length] > 6)
        key = [key substringToIndex:6];
      if ( [key caseInsensitiveCompare:lastKey] != NSOrderedSame ) {
        if ( sectionCount < 100 ) {
          [sectionIndexTitles addObject:key];
          sectionIndex[sectionCount++] = i;
        }
      }
      lastKey = key;
    }
    else if ( order == kSortTunesByMeter ) {
      if ( lastNum != tune.timeSigNum || lastDen != tune.timeSigDen ) {
        if ( sectionCount < 100 ) {
          [sectionIndexTitles addObject:[tune meter]];
          sectionIndex[sectionCount++] = i;
        }
      }
      lastNum = tune.timeSigNum;
      lastDen = tune.timeSigDen;
    }
  }
  sectionIndex[sectionCount] = count;
}

- (NSArray *)sectionIndexTitlesForTableView:(UITableView *)tableView
{
  if ( [self tuneSortOrder] == kSortTunesByOrder )
    return nil;
  return sectionIndexTitles;
}

- (NSInteger)tableView:(UITableView *)tableView sectionForSectionIndexTitle:(NSString *)title atIndex:(NSInteger)index
{
  return index+1;
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
{
  // No title for the "All Tunes" section
  if ( section == 0 )
    return nil;
  if ( [self tuneSortOrder] == kSortTunesByOrder )
    return nil;
  return [sectionIndexTitles objectAtIndex:section-1];
}

- (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath
{
  return [self tuneSortOrder] == kSortTunesByOrder &&
    playlist != nil &&
    indexPath.section != kActionSection &&
    [listContent count] == [filteredListContent count];
}

- (NSIndexPath *)tableView:(UITableView *)tableView targetIndexPathForMoveFromRowAtIndexPath:(NSIndexPath *)sourceIndexPath toProposedIndexPath:(NSIndexPath *)proposedDestinationIndexPath
{
  // Check for moving to the action section. If so, retarget to just below that row.
  if ( proposedDestinationIndexPath.section == kActionSection )
    return [NSIndexPath indexPathForRow:0 inSection:sourceIndexPath.section];
  return proposedDestinationIndexPath;
}

- (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)fromIndexPath toIndexPath:(NSIndexPath *)toIndexPath
{
  if ( toIndexPath.row <= [listContent count] ) {
    id item = [[listContent objectAtIndex:fromIndexPath.row] retain];
    [listContent removeObjectAtIndex:fromIndexPath.row];
    [listContent insertObject:item atIndex:toIndexPath.row];
    [item release];
    [playlist updateSortOrder:listContent];
    [self filterTunes];
  }
}

- (void)searchBarTextDidBeginEditing:(UISearchBar *)bar
{
}

- (void)searchBarTextDidEndEditing:(UISearchBar *)bar
{
}

- (void)searchBar:(UISearchBar *)bar textDidChange:(NSString *)searchText
{
  [self filterTunes];
}

- (void)searchBarCancelButtonClicked:(UISearchBar *)bar
{
	if ( bar.text.length > 0 ) {
    [self filterTunes];
	}
	
  [self buildSectionIndex];
	[tableView reloadData];

	[bar resignFirstResponder];
	bar.text = @"";
}

- (void)searchBarSearchButtonClicked:(UISearchBar *)bar
{
	[bar resignFirstResponder];
}

- (void)searchForDots:(NSString *)searchText
{
  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
	[filteredListContent removeAllObjects];
  [appDelegate searchForDots:searchText tunes:listContent results:filteredListContent];
  [self buildSectionIndex];
	[tableView reloadData];
}

- (IBAction)donePressed:(id)sender
{
  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
  [appDelegate startTransaction];
  for ( Tune *tune in listContent ) {
    if ( tune.selected )
      [choosingTunes addTune:tune];
    tune.selected = NO;
  }
  [appDelegate commitTransaction];
  [[NSNotificationCenter defaultCenter] postNotificationName:@"databaseChanged" object:self]; 
  [self dismissViewControllerAnimated:YES completion:nil];
}

- (IBAction)cancelPressed:(id)sender
{
  for ( Tune *tune in listContent )
    tune.selected = NO;
  [self dismissViewControllerAnimated:YES completion:nil];
}

- (IBAction)bookmarkPressed:(id)sender
{
  _activeActionSheetType = kActionSheetBookmark;
  UIActionSheet *sheet = [[UIActionSheet alloc]
    initWithTitle:@"Create a smart tunebook with this search?"
    delegate:self
    cancelButtonTitle:@"Don't Create"
    destructiveButtonTitle:@"Create"
    otherButtonTitles:nil
  ];
  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
  [sheet showInView:appDelegate.rootView];
}

- (IBAction)addNewTune:(id)sender;
{
  if ( playlist == nil ) {
    Tune *tune = [[Tune alloc] init];

    NSArray* selectedRows = [tableView indexPathsForSelectedRows];
    if ( [self isEditing] && [selectedRows count] == 1 ) {
      NSIndexPath* indexPath = [selectedRows firstObject];
      Tune* t = [self tuneForIndexPath:indexPath];
      tune.title = t.title;
      tune.key = t.key;
      tune.dots = t.dots;
    }
    else {
      tune.title = @"New Tune";
      tune.key = @"D";
      tune.dots = @"X: 1\nT: New Tune\nL: 1/8\nM:4/4\nK: D\n|";
    }

    AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
    EditABCController* editABCController = [EditABCController alloc];
    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
      [editABCController initWithNibName:@"EditABC" bundle:nil];
    else
      [editABCController initWithNibName:@"EditABC_iphone" bundle:nil];
    editABCController.modalPresentationStyle = UIModalPresentationFullScreen;
    editABCController.editTune = tune;
    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
      [appDelegate.topLevelViewController presentViewController:editABCController animated:YES completion:nil];
    else
      [appDelegate.navigationController presentViewController:editABCController animated:YES completion:nil];
    [editABCController release];
    return;
  }

  MainViewController *controller = [[MainViewController alloc] initWithNibName:@"AddTunes" bundle:nil];
  controller.playlist = nil;
  controller.choosingTunes = playlist;
  if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
    controller.modalPresentationStyle = UIModalPresentationFormSheet;
  [self presentViewController:controller animated:YES completion:nil];
  [controller release];
  [self buildSectionIndex];
  [tableView reloadData];
}

#pragma mark UIActionSheetDelegate methods

- (void)actionSheet:(UIActionSheet *)actionSheet didDismissWithButtonIndex:(NSInteger)buttonIndex
{
  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
  if (_activeActionSheetType == kActionSheetBookmark ) {
    if ( buttonIndex == 0 ) {
      [appDelegate addPlaylist:playlist];
    }
  }
#if TUNEBOOK_MULTI_SELECT
  else if (_activeActionSheetType == kActionSheetMove ) {
    if ( buttonIndex == _addToTunebookIndex ) {
      [self addToTunebook];
    }
    else if ( buttonIndex == _addToNewTunebookIndex ) {
      [self promptForNewTunebookName];
    }
  }
#endif
  _activeActionSheetType = kActionSheetNone;
}

// These actions are only for iPad
- (IBAction)searchPressed:(id)sender
{
  SearchViewController *controller = [[SearchViewController alloc] initWithNibName:@"SearchView" bundle:nil];
  [self.navigationController pushViewController:controller animated:YES];
  [controller release];
}

- (IBAction)settingsPressed:(id)sender
{
  SettingsController *controller = [[SettingsController alloc] initWithNibName:@"Settings" bundle:nil];
  [self.navigationController pushViewController:controller animated:YES];
  [controller release];
}

- (IBAction)playSettingsPressed:(id)sender
{
  TunePlayer *tunePlayer = [TunePlayer tunePlayer];

  TuneSettingsController*settingsController = [[TuneSettingsController alloc] initWithPlayer:tunePlayer];
  settingsController.contentSizeForViewInPopover = settingsController.view.bounds.size;
  UIPopoverController* activePopoverController = [[UIPopoverController alloc] initWithContentViewController:settingsController];
  [activePopoverController presentPopoverFromBarButtonItem:playSettingsItem
      permittedArrowDirections:UIPopoverArrowDirectionAny animated:YES];
  [settingsController release];
}

- (IBAction)actionPressed:(id)sender
{
  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
  if ( !appDelegate.tuneViewController.tune )
    return;
  [appDelegate.tuneViewController actionPressed:sender];
}

- (void)startedPlaying:(id)sender
{
//  toolbar.items = playItems;
}

- (void)finishedPlaying:(id)sender
{
//  toolbar.items = stopItems;
}

- (IBAction)playPressed:(id)sender
{
  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
//  if ( !appDelegate.tuneViewController.tune )
//    return;
//  toolbar.items = playItems;
//  [appDelegate.tuneViewController startPlaying:self];
  [appDelegate.tuneViewController toggleStatusBar];
}

- (IBAction)stopPressed:(id)sender
{
//  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
//  if ( !appDelegate.tuneViewController.tune )
//    return;
//  toolbar.items = stopItems;
////  [appDelegate.tuneViewController startPlaying:self];
//  [appDelegate.tuneViewController pausePlaying:sender];
}

- (IBAction)editPressed:(id)sender
{
//  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
//  [appDelegate.tuneViewController editPressed:self];
}

- (IBAction)emailPressed:(id)sender
{
//  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
//  [appDelegate.tuneViewController printTune:self];
}

- (IBAction)sortOrderChanged:(id)sender
{
  Tune* tune = [self selectedTune];
  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
  NSInteger order = sortOrder.selectedSegmentIndex;
  [appDelegate setSortOrder:order forPlaylist:playlist];
  if ( playlist )
    [self filterTunes];
  [self selectTune:tune];
}

#if TUNEBOOK_MULTI_SELECT
- (Tune*)tuneForIndexPath:(NSIndexPath*)indexPath
{
  NSInteger row = indexPath.row;
  row += sectionIndex[indexPath.section];
  Tune *tune = (Tune *)[filteredListContent objectAtIndex:row];
  return tune;
}

- (IBAction)deletePressed:(id)sender
{
  NSArray* selectedRows = [tableView indexPathsForSelectedRows];
  if ( playlist == nil && [selectedRows count] > 1 ) {
    UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:NSLocalizedString(@"Delete Tunes", @"")
        message:[NSString stringWithFormat:@"Permanently delete %lu tunes?", (unsigned long)[selectedRows count], nil]
        delegate:self
        cancelButtonTitle:@"Cancel"
        otherButtonTitles:NSLocalizedString(@"Delete", @""), nil];
    alertView.tag = kAlertConfirmDelete;
    [alertView show];
    [alertView release];
  }
  else
    [self deleteSelectedTunes:sender];
}

- (IBAction)deleteSelectedTunes:(id)sender
{
  NSArray* selectedRows = [tableView indexPathsForSelectedRows];
  NSIndexPath* indexPath;
  if ( [selectedRows count] > 0 ) {
    AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
    [appDelegate startTransaction];
    for (indexPath in selectedRows) {
      Tune* tune = [self tuneForIndexPath:indexPath];
      if ( tune != nil ) {
        if ( playlist ) {
          [playlist removeTune:tune];
        } else {
          AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
          [appDelegate removeTune:tune];
        }

        if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
          AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
          if ( appDelegate.tuneViewController.tune == tune )
            [appDelegate.tuneViewController showTune:nil];
        }
      }
    }
    [appDelegate commitTransaction];
    [[NSNotificationCenter defaultCenter] postNotificationName:@"databaseChanged" object:self]; 
  }
}

- (IBAction)movePressed:(id)sender
{
  NSArray* selectedRows = [tableView indexPathsForSelectedRows];
  if ( [selectedRows count] > 0 ) {
    _activeActionSheetType = kActionSheetMove;
    UIActionSheet* actionActionSheet = [[UIActionSheet alloc]
                    initWithTitle:nil
                         delegate:self
                cancelButtonTitle:nil
           destructiveButtonTitle:nil
                otherButtonTitles:nil];

    actionActionSheet.tag = kActionSheetMove;
    _addToNewTunebookIndex = _addToTunebookIndex = -1000;

    _addToNewTunebookIndex = [actionActionSheet addButtonWithTitle:@"Add to New Tunebook..."];
    _addToTunebookIndex = [actionActionSheet addButtonWithTitle:@"Add to Tunebooks..."];

    [actionActionSheet addButtonWithTitle:@"Cancel"];
    [actionActionSheet showFromBarButtonItem:sender animated:YES];
  }
}

- (void)addToTunebook
{
  NSArray* tunes = [self selectedTunes];

  PlaylistViewController *addTunesController = [[PlaylistViewController alloc] initWithNibName:@"AddToTunebook" bundle:nil];
  addTunesController.addingTunes = tunes;
  [addTunesController databaseChanged:self];//fixme - in viewdidload?
  if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
    addTunesController.modalPresentationStyle = UIModalPresentationFormSheet;
    [self presentViewController:addTunesController animated:YES completion:nil];
  }
  else {
    AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
    [appDelegate.navigationController presentViewController:addTunesController animated:YES completion:nil];
  }
}

- (void)promptForNewTunebookName
{
  UIAlertView *prompt = [[UIAlertView alloc] initWithTitle:NSLocalizedString(@"New Tunebook", @"") message:@"Enter the name of the new tunebook:" delegate:self cancelButtonTitle:@"Cancel" otherButtonTitles:NSLocalizedString(@"OK", @""), nil];
  prompt.tag = kAlertAddToTunebook;
  [prompt setAlertViewStyle:UIAlertViewStylePlainTextInput];
  UITextField *textfield =  [prompt textFieldAtIndex: 0];
  [textfield setText:@"New"];
  [prompt show];
//  [textfield selectAll:self];
//  [UIMenuController sharedMenuController].menuVisible = NO;
  [prompt release];
}

- (void)alertView:(UIAlertView *)alertView didDismissWithButtonIndex:(NSInteger)buttonIndex
{
  if ( alertView.tag == kAlertAddToTunebook ) {
    if ( buttonIndex == 1 ) {
      NSArray* selectedRows = [tableView indexPathsForSelectedRows];
      NSIndexPath* indexPath;
      if ( [selectedRows count] > 0 ) {
        UITextField *textfield =  [alertView textFieldAtIndex: 0];
        NSString* title = [textfield text];
        if ( title.length == 0 )
          title = @"New Tunebook";
        Playlist *newTunebook = [[Playlist alloc] init];
        newTunebook.name = title;
        AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
        [appDelegate startTransaction];
        [appDelegate addPlaylist:newTunebook];

        NSUInteger count = 0;
        for (indexPath in selectedRows) {
          Tune* tune = [self tuneForIndexPath:indexPath];
          if ( tune != nil ) {
            [newTunebook addTune:tune];
            count++;
          }
        }
        [appDelegate commitTransaction];
        [[NSNotificationCenter defaultCenter] postNotificationName:@"databaseChanged" object:self]; 

        NSString* message;
        if ( count == 1 )
          message = [NSString stringWithFormat:@"1 tune was added to new tunebook \"%@\".", title, nil];
        else
          message = [NSString stringWithFormat:@"%lu tunes were added to new tunebook \"%@\".", (unsigned long)count, title, nil];
        UIAlertView *alert = [[UIAlertView alloc]
          initWithTitle: @"New Tunebook"
          message:message
          delegate:nil
          cancelButtonTitle:nil 
          otherButtonTitles:@"OK", nil];
        [alert show];
      }
    }
  }
  else if ( alertView.tag == kAlertConfirmDelete ) {
    if ( buttonIndex == 1 )
      [self deleteSelectedTunes:nil];
  }
}
#endif
@end
