#import "ManageTunesController.h"
#import "ImportABCController.h"
#import "WebServerController.h"
#import "WebViewController.h"
#import "TuneViewController.h"
#import "ImportProgressController.h"
#import "AppDelegate.h"

@implementation ManageTunesController

#define kCellIdentifier @"ManageCellIdentifier"

enum {
  kImportTunes = 0,
  kImportTunesFromWeb,
  kDeleteTunes,
  kNewTune,
  kNumRows,
};

#define kShareTunes   0

-(void)viewDidLoad
{
  tableView.scrollsToTop = NO;
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
  return YES;
}

- (void)didReceiveMemoryWarning
{
	[super didReceiveMemoryWarning]; // Releases the view if it doesn't have a superview.
	// Release anything that's not essential, such as cached data.
}

-(void)dealloc
{
	[super dealloc];
}

#pragma mark UITableView delegate methods

- (void)tableView:(UITableView *)tv didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
	[tv deselectRowAtIndexPath:indexPath animated:YES];

  NSInteger row = indexPath.row;
  if ( indexPath.section == 0 ) {
    AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
    if ( row == kImportTunes ) {
      [self showImportTunes];
    }
    if ( row == kImportTunesFromWeb ) {
      NetworkStatus netStatus = [appDelegate.reachability currentReachabilityStatus];
      if ( netStatus == NotReachable ) {
        UIAlertController* alertController = [UIAlertController alertControllerWithTitle:@"Error" message:@"You can't browse the Web because you have no Internet connection." preferredStyle:UIAlertControllerStyleAlert];
        [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil]];
        [self presentViewController:alertController animated:YES completion:nil];
      } else {
        WebViewController *controller = [[WebViewController alloc] initWithNibName:@"WebView" bundle:nil];
        controller.modalPresentationStyle = UIModalPresentationFullScreen;
        [appDelegate.navigationController presentViewController:controller animated:YES completion:nil];
      }
    } else if ( row == kDeleteTunes ) {
      UIActionSheet *sheet = [[UIActionSheet alloc]
        initWithTitle:@"Delete Tunes"
        delegate:self
        cancelButtonTitle:@"Don't Delete"
        destructiveButtonTitle:@"Delete"
        otherButtonTitles:nil
      ];
      [sheet showInView:appDelegate.rootView];
    } else if ( row == kNewTune ) {
      Tune *tune = [[Tune alloc] init];
      tune.title = @"New Tune";
      tune.key = @"D";
      tune.dots = @"X: 1\nT: New Tune\nL: 1/8\nM:4/4\nK: D\n|";
      TuneViewController *controller = [[TuneViewController alloc] initWithTune:tune];
      AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
      [appDelegate.navigationController pushViewController:controller animated:YES];
      [controller setEditing:YES];
      [controller autorelease];
    }
  } else {
    if ( row == kShareTunes ) {
      AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
      NetworkStatus netStatus = [appDelegate.reachability currentReachabilityStatus];
      if ( netStatus == NotReachable ) {
        UIAlertController* alertController = [UIAlertController alertControllerWithTitle:@"Error" message:@"Couldn't start a Web server because you have no Internet connection." preferredStyle:UIAlertControllerStyleAlert];
        [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil]];
        [self presentViewController:alertController animated:YES completion:nil];
      } else {
        WebServerController *controller = [[WebServerController alloc] initWithNibName:@"WebServer" bundle:nil];
        [appDelegate.navigationController presentViewController:controller animated:YES completion:nil];
      }
    } else {
    }
  }
}

#pragma mark UITableView data source methods

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
	return 2;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
  return section == 0 ? kNumRows : 1;
}

- (NSString *)tableView:(UITableView *)aTableView titleForHeaderInSection:(NSInteger)section {
	return section == 0 ? @"Manage Tunes" : @"Sharing";
}

- (UITableViewCell *)tableView:(UITableView *)tv cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
	UITableViewCell *cell = [tv dequeueReusableCellWithIdentifier:kCellIdentifier];
	if (cell == nil)
		cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:kCellIdentifier] autorelease];

  NSInteger row = indexPath.row;
  if ( indexPath.section == 0 ) {
    if ( row == kImportTunes )
      cell.textLabel.text = @"Import Tunes";
    else if ( row == kImportTunesFromWeb )
      cell.textLabel.text = @"Import Tunes from Web";
    else if ( row == kDeleteTunes )
      cell.textLabel.text = @"Delete All Tunes";
    else if ( row == kNewTune )
      cell.textLabel.text = @"Add New Tune";
  } else {
    if ( row == kShareTunes )
      cell.textLabel.text = @"Start Web Server";
  }
  cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
	return cell;
}

#pragma mark UIActionSheetDelegate methods

- (void)actionSheet:(UIActionSheet *)actionSheet didDismissWithButtonIndex:(NSInteger)buttonIndex
{
  if ( buttonIndex == 0 ) {
    AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
    [appDelegate deleteAllTunes];
  }
}

//FIXME-this code should be shared with SettingController
- (void)documentPicker:(UIDocumentPickerViewController *)controller didPickDocumentAtURL:(NSURL *)url {
  if (controller.documentPickerMode == UIDocumentPickerModeImport) {
    ImportProgressController *controller = [[ImportProgressController alloc] initWithNibName:@"ImportProgress" bundle:nil];
    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
      controller.modalPresentationStyle = UIModalPresentationFormSheet;
    [self presentViewController:controller animated:YES completion:nil];
    [controller import:url];
  }
}

- (void)showImportTunes
{
  UIDocumentPickerViewController *documentPicker = [[UIDocumentPickerViewController alloc] initWithDocumentTypes:@[@"public.text", @"public.plain-text", @"public.utf8-plain-text", @"public.alembic"]
    inMode:UIDocumentPickerModeImport];
  documentPicker.delegate = self;
  documentPicker.modalPresentationStyle = UIModalPresentationFormSheet;
  [self presentViewController:documentPicker animated:YES completion:nil];
}

@end
