#import "PlaylistViewController.h"
#import "MainViewController.h"
#import "AppDelegate.h"
#import "TuneDatabase.h"
#import "EditTextTableCell.h"

#import "TunebookCell.h"
#import "SearchViewController.h"
#import "SettingsController.h"
#import "ManageTunesController.h"
#import "TuneSettingsController.h"
#import "TuneViewController.h"
#import "QuartzCore/CAGradientLayer.h"

#define kCellIdentifier @"com.jhlabs.tunebook.PlaylistCellIdentifier"

@implementation PlaylistViewController

@synthesize listContent, tableView;
@synthesize searchBar;
@synthesize addingTunes;

#define kHasAddTuneRow YES  // We show the "Add New Tune" row when editing.

#if TARGET_IPAD// We can probably get rid of this...
#define TB_HAS_ALL_TUNES_ROW 0
#else
#define TB_HAS_ALL_TUNES_ROW 0
#endif

- (CGSize)contentSizeForViewInPopoverView
{
  return CGSizeMake(320.0, 440.0);
}

-(void)awakeFromNib
{
  [super awakeFromNib];
}

-(void)viewDidLoad
{
  [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(databaseChanged:) name:@"databaseChanged" object:nil]; 
  [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(databaseChanged:) name:@"databaseLoaded" object:nil]; 
  [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(databaseChanged:) name:@"playlistsChanged" object:nil]; 
  [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(startedPlaying:) name:@"startedPlaying" object:nil];
  [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(finishedPlaying:) name:@"finishedPlaying" object:nil];

	searchBar.autocorrectionType = UITextAutocorrectionTypeNo;
	searchBar.autocapitalizationType = UITextAutocapitalizationTypeNone;
  searchBar.userInteractionEnabled = YES;
	searchBar.showsCancelButton = UI_USER_INTERFACE_IDIOM() != UIUserInterfaceIdiomPad;

  // Set up the sticky row for "All Tunes"
  allTunesCell.textLabel.text = @"All Tunes";
  allTunesCell.textLabel.font = [UIFont boldSystemFontOfSize:20];
  allTunesCell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
//  allTunesCell.textLabel.backgroundColor = [UIColor redColor];
  UITapGestureRecognizer *tapGesture = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(showAllTunes:)];
  tapGesture.numberOfTapsRequired = 1;
  [allTunesCell addGestureRecognizer:tapGesture];
  [tapGesture release];

  self.tableView.scrollsToTop = YES;
  if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
    self.navigationItem.rightBarButtonItem = self.editButtonItem;
    self.navigationItem.title = @"Tunebooks";

    // Set up the arrays of toolbar items we'll need
    NSArray *items = toolbar.items;
    playItems = [NSMutableArray arrayWithArray:items];
    [playItems removeObject:playButton];
    [playItems retain];
    stopItems = [NSMutableArray arrayWithArray:items];
    [stopItems removeObject:stopButton];
    [stopItems retain];
    toolbar.items = stopItems;
  }
  _ignoreAccessoryTap = false;
}

- (void)dealloc
{
	[tableView dealloc];
	[searchBar release];
	[filteredListContent release];
	[listContent release];
	[playItems release];
	[stopItems release];

  [super dealloc];
}

- (void)databaseChanged:(id)sender
{
  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
  listContent = [appDelegate.playlists retain];
  [self filterTunebooks];

  if ( addingTunes != nil ) {
    for ( Playlist* playlist in listContent )
      playlist.selected = NO;
  }
}

#pragma mark UIViewController delegate

- (void)viewWillAppear:(BOOL)animated
{
  [super viewWillAppear:animated];
	NSIndexPath *tableSelection = [tableView indexPathForSelectedRow];
	[tableView deselectRowAtIndexPath:tableSelection animated:NO];
//	[[UIApplication sharedApplication] setStatusBarStyle:UIStatusBarStyleDefault animated:YES];
  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
  appDelegate.topViewController.navigationItem.rightBarButtonItem = self.editButtonItem;
}

- (void)viewWillDisappear:(BOOL)animated
{
  [super viewWillDisappear:animated];
  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
  appDelegate.topViewController.navigationItem.rightBarButtonItem = nil;
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
  return YES;
}

-(void)showAllTunes:(UIGestureRecognizer*) gesture
{
  [self setEditing:NO animated:YES];
  allTunesCell.selected = YES;
  if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
    AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
    appDelegate.currentTunebook = -1;
    MainViewController *controller = [[MainViewController alloc] initWithNibName:@"MainViewToolbar" bundle:nil];
    controller.playlist = nil;
    [[self navigationController] pushViewController:controller animated:YES];
    [controller release];
  }
  else {
    AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
    MainViewController *controller = [[MainViewController alloc] initWithNibName:@"MainView" bundle:nil];
    controller.playlist = nil;
    [appDelegate.navigationController pushViewController:controller animated:YES];
  }
  allTunesCell.selected = NO;
}

- (void)filterTunebooks
{
  if ( !filteredListContent )
    filteredListContent = [[NSMutableArray alloc] initWithCapacity: [listContent count]];
  [filteredListContent removeAllObjects];

  NSString* searchText = searchBar.text;
  if ( addingTunes != nil || [searchText length] != 0 ) {
    Playlist *playlist;

    for ( playlist in listContent ) {
      if ( addingTunes == nil || playlist.type == kPlaylistTypeSimple ) {
        if ( [searchText length] != 0 ) {
          NSRange range = [playlist.name rangeOfString:searchText options:NSCaseInsensitiveSearch|NSDiacriticInsensitiveSearch];
          if ( range.length != 0 )
            [filteredListContent addObject:playlist];
        }
        else
          [filteredListContent addObject:playlist];
      }
    }
  }
  else {
    [filteredListContent addObjectsFromArray: listContent];
  }

	[tableView reloadData];
}

- (void)setEditing:(BOOL)editing animated:(BOOL)animated
{
  if ( editing == [self isEditing] )
    return;

  [super setEditing:editing animated:animated];
  if ( kHasAddTuneRow ) {
    NSArray *indexPaths = [NSArray arrayWithObjects:
      [NSIndexPath indexPathForRow:0 inSection:1],
      nil
    ];

    [tableView beginUpdates];
    [tableView setEditing:editing animated:YES];

    if (editing) {
      // Show the placeholder rows
      [tableView insertRowsAtIndexPaths:indexPaths withRowAnimation:UITableViewRowAnimationTop];
    } else {
      // Hide the placeholder rows.
      [tableView deleteRowsAtIndexPaths:indexPaths withRowAnimation:UITableViewRowAnimationTop];
    }
  }
  else {
    [tableView beginUpdates];
    [tableView setEditing:editing animated:YES];
  }
  [tableView endUpdates];
  searchBar.userInteractionEnabled = !editing;
}

#pragma mark UITableView delegate methods

- (void)tableView:(UITableView *)tv didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
	if ( [searchBar isFirstResponder] )
    [searchBar resignFirstResponder];

  Playlist *playlist = 0;
  if ( !kHasAddTuneRow ) {
    if ( indexPath.section != 0 )
      playlist = (Playlist *)[filteredListContent objectAtIndex:indexPath.row];
  }
  else
    playlist = (Playlist *)[filteredListContent objectAtIndex:indexPath.row];

  if ( addingTunes ) {
    playlist.selected = !playlist.selected;
    [tableView reloadRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationNone];
  } else {
    [tv deselectRowAtIndexPath:indexPath animated:YES];
    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
      AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
      appDelegate.currentTunebook = playlist ? playlist.primaryKey : -1;
      MainViewController *controller = [[MainViewController alloc] initWithNibName:@"MainViewToolbar" bundle:nil];
      controller.playlist = playlist;
      [[self navigationController] pushViewController:controller animated:YES];
      [controller release];
    }
    else {
      AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
      MainViewController *controller = [[MainViewController alloc] initWithNibName:@"MainView" bundle:nil];
      controller.playlist = playlist;
      [appDelegate.navigationController pushViewController:controller animated:YES];
    }
  }
}

- (void)tableView:(UITableView *)tv accessoryButtonTappedForRowWithIndexPath:(NSIndexPath *)indexPath
{
  if ( _ignoreAccessoryTap ) {
    _ignoreAccessoryTap = false;
    return;
  }

  Playlist *playlist = nil;
  if ( !kHasAddTuneRow )
    playlist = (Playlist *)[filteredListContent objectAtIndex:indexPath.row];
  else
    playlist = (Playlist *)[filteredListContent objectAtIndex:indexPath.row-1]; // -1 because we're editing and have the "add" row

#if 1
  [self renameTunebook:playlist];
#else
  detailsController.playlist = playlist;
  detailsController.tableView = tv;
  detailsController.indexPath = indexPath;
  if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
    detailsController.contentSizeForViewInPopover = CGSizeMake(320,480);
    [[self navigationController] pushViewController:detailsController animated:YES];
  }
  else {
    AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
    [appDelegate.navigationController pushViewController:detailsController animated:YES];
  }
#endif
}

#pragma mark UITableView data source methods

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
  return 2;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
  if ( section == 0 )
#if TB_HAS_ALL_TUNES_ROW
    return 1; // The All Tunes row
#else
    return 0;
#endif
  NSInteger count = [filteredListContent count];
  if ( kHasAddTuneRow )
    if (self.editing)
      count++;

  return count;
}

#define kIconTag 1
#define kNameTag 2

- (UITableViewCell *)tableView:(UITableView *)tv cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
	UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:kCellIdentifier];

  if ( cell == nil ) {
    cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:kCellIdentifier] autorelease];
  }

  cell.editingAccessoryType = UITableViewCellAccessoryNone;

#if TB_HAS_ALL_TUNES_ROW
  if ( indexPath.section == 0 ) {
    cell.textLabel.text = @"All Tunes";
    cell.textLabel.font = [UIFont boldSystemFontOfSize:20];
    cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
    return cell;
  }
  cell.textLabel.font = [UIFont systemFontOfSize:20];
#endif

  NSInteger row = indexPath.row;
  if ( kHasAddTuneRow )
    if ( self.editing )
      row--;

  if ( self.editing && row == -1 ) {
    cell.textLabel.text = @"Add new tunebook";
  } else {
    Playlist *playlist = (Playlist *)[filteredListContent objectAtIndex:row];

    cell.textLabel.text = playlist.name;
    if ( addingTunes )
      cell.accessoryType = playlist.selected ? UITableViewCellAccessoryCheckmark : UITableViewCellAccessoryNone;
    else if ( playlist.type == kPlaylistTypeSearch )
      cell.accessoryView = [[UIImageView alloc] initWithImage:[UIImage imageNamed:@"smartbook.png"]];
    else {
      cell.accessoryView = nil;
      cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
    }
    cell.editingAccessoryType = UITableViewCellAccessoryDetailDisclosureButton;
  }

	return cell;
}

- (UITableViewCell *)tableviewCellWithReuseIdentifier:(NSString *)identifier {
	
  CGRect bounds = [tableView bounds];
	
	UITableViewCell *cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:identifier] autorelease];
		
	CGRect rect = CGRectMake( 6, 13, 16, 16 );
	UIImageView *imageView = [[UIImageView alloc] initWithFrame:rect];
	imageView.tag = kIconTag;
  imageView.opaque = NO;
	[cell.contentView addSubview:imageView];
	[imageView release];
	
	rect = CGRectMake( 28, 3, bounds.size.width-60, 34);
	UILabel *label = [[UILabel alloc] initWithFrame:rect];
	label.tag = kNameTag;
	label.font = [UIFont boldSystemFontOfSize:20];
	label.adjustsFontSizeToFitWidth = NO;
  label.opaque = NO;
	[cell.contentView addSubview:label];
	label.highlightedTextColor = [UIColor whiteColor];
	[label release];
	return cell;
}

- (UITableViewCellEditingStyle)tableView:(UITableView *)tv editingStyleForRowAtIndexPath:(NSIndexPath *)indexPath
{
  if ( !kHasAddTuneRow ) {
    if ( self.editing && indexPath.section == 1 )
      return UITableViewCellEditingStyleDelete;
  }
  else {
    if ( self.editing && indexPath.section == 1 ) {
      if ( indexPath.row == 0 ) {
          return UITableViewCellEditingStyleInsert;
      } else {
          return UITableViewCellEditingStyleDelete;
      }
    }
  }
  return UITableViewCellEditingStyleNone;
}

- (void)tableView:(UITableView *)tv commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath
{
  if (editingStyle == UITableViewCellEditingStyleDelete) {
    _indexToDelete = [indexPath retain];
    UIActionSheet *actionSheet = [[UIActionSheet alloc]
                            initWithTitle:@"Delete the tunes this tunebook contains as well?"
                            delegate:self
                            cancelButtonTitle:@"Cancel"
                            destructiveButtonTitle:@"Delete Tunes and Tunebook"
                            otherButtonTitles:@"Delete Tunebook Only", nil
                            ];
    AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
    [actionSheet showInView:appDelegate.rootView];
  } else if (editingStyle == UITableViewCellEditingStyleInsert) {
    [self createNewTunebook];
  }
}

- (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath
{
  if ( !kHasAddTuneRow )
    return indexPath.section == 1 &&
      indexPath.row < [filteredListContent count] &&
      [listContent count] == [filteredListContent count];

  return indexPath.section == 1 && indexPath.row != 0 &&
    indexPath.row <= [filteredListContent count] &&
    [listContent count] == [filteredListContent count];
}

- (NSIndexPath *)tableView:(UITableView *)tableView targetIndexPathForMoveFromRowAtIndexPath:(NSIndexPath *)sourceIndexPath toProposedIndexPath:(NSIndexPath *)proposedDestinationIndexPath
{
  if ( !kHasAddTuneRow ) {
  // Check for moving to the placeholder row. If so, retarget to just below that row.
    if ( proposedDestinationIndexPath.section == 0 || proposedDestinationIndexPath.row == 0 )
      return [NSIndexPath indexPathForRow:1 inSection:sourceIndexPath.section];
  }
  else {
    // Check for moving to the placeholder row. If so, retarget to just below that row.
    if ( proposedDestinationIndexPath.section == 0 )
      return [NSIndexPath indexPathForRow:1 inSection:sourceIndexPath.section];
  }
  return proposedDestinationIndexPath;
}

- (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)fromIndexPath toIndexPath:(NSIndexPath *)toIndexPath
{
  _ignoreAccessoryTap = true; // Ignore the accessory button which is also fired after reordering
  if ( toIndexPath.row <= [listContent count] ) {
    if ( !kHasAddTuneRow ) {
      id item = [[filteredListContent objectAtIndex:fromIndexPath.row] retain];
      [listContent removeObject:item];
      [listContent insertObject:item atIndex:toIndexPath.row];
      [item release];
    }
    else {
      id item = [[filteredListContent objectAtIndex:fromIndexPath.row-1] retain];
      [listContent removeObject:item];
      [listContent insertObject:item atIndex:toIndexPath.row-1];
      [item release];
    }
    AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
    [appDelegate.database updateTunebookSortOrder];
    [self filterTunebooks];
  }
}

- (void)actionSheet:(UIActionSheet *)actionSheet didDismissWithButtonIndex:(NSInteger)buttonIndex
{
#if TUNEBOOK_ASK_DELETE
  if ( buttonIndex == 0 || buttonIndex == 1 ) { // Delete Tunes/Tunebook
    AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
    NSInteger row = _indexToDelete.row;
    if ( kHasAddTuneRow )
      if ( self.editing )
        row--;
    if ( row < [filteredListContent count] ) {
      Playlist *playlist = (Playlist *)[filteredListContent objectAtIndex:row];

      if ( buttonIndex == 0 ) { // Delete Tunes
        Tune* tune;
        for ( tune in [playlist loadTunes] ) {
          [appDelegate removeTune:tune];
        }
      }
      [playlist deleteFromDatabase];
      [listContent removeObjectAtIndex:row];
      [filteredListContent removeObjectAtIndex:row];
    }
    [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:_indexToDelete] withRowAnimation:UITableViewRowAnimationFade];
    [[NSNotificationCenter defaultCenter] postNotificationName:@"databaseChanged" object:self];
  }
  [_indexToDelete release];
#endif
}

- (void)addTunebook:(id)sender
{
  [self createNewTunebook];
}

- (IBAction)searchPressed:(id)sender
{
  SearchViewController *controller = [[SearchViewController alloc] initWithNibName:@"SearchView" bundle:nil];
  [self.navigationController pushViewController:controller animated:YES];
  [controller release];
}

- (IBAction)settingsPressed:(id)sender
{
  SettingsController *controller = [[SettingsController alloc] initWithNibName:@"Settings" bundle:nil];
  [self.navigationController pushViewController:controller animated:YES];
  [controller release];
}

- (IBAction)playSettingsPressed:(id)sender
{
  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
  [appDelegate.tuneViewController showSettings:playSettingsItem];
}

- (void)selectTunebook:(int)aTunebookPrimaryKey
{
  NSIndexPath* indexPath = 0;
  if ( aTunebookPrimaryKey == -1 ) {
    // iPhone has no All Tunes row
//    if (UI_USER_INTERFACE_IDIOM() != UIUserInterfaceIdiomPad)
//      indexPath = [NSIndexPath indexPathForRow:0 inSection:0];
  }
  else {
    int row = 0;
    for (Playlist* playlist in filteredListContent) {
      if ( playlist.primaryKey == aTunebookPrimaryKey ) {
        indexPath = [NSIndexPath indexPathForRow:row inSection:1];
        break;
      }
      row++;
    }
  }
  if ( indexPath != nil ) {
    [tableView selectRowAtIndexPath:indexPath animated:YES scrollPosition:UITableViewScrollPositionMiddle];
    [[tableView delegate] tableView:tableView didSelectRowAtIndexPath:indexPath];
  }
}


//FIXME-this code is duplicated from MainViewController - need a superclass
- (void)startedPlaying:(id)sender
{
//  toolbar.items = playItems;
}

- (void)finishedPlaying:(id)sender
{
//  toolbar.items = stopItems;
}

- (IBAction)actionPressed:(id)sender
{
  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
  if ( !appDelegate.tuneViewController.tune )
    return;
  [appDelegate.tuneViewController actionPressed:sender];
}

- (IBAction)playPressed:(id)sender
{
  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
//  if ( !appDelegate.tuneViewController.tune )
//    return;
//  toolbar.items = playItems;
//  [appDelegate.tuneViewController startPlaying:self];
  [appDelegate.tuneViewController toggleStatusBar];
}

- (IBAction)stopPressed:(id)sender
{
//  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
//  if ( !appDelegate.tuneViewController.tune )
//    return;
//  toolbar.items = stopItems;
////  [appDelegate.tuneViewController startPlaying:self];
//  [appDelegate.tuneViewController pausePlaying:sender];
}

- (void)searchBarTextDidBeginEditing:(UISearchBar *)bar
{
}

- (void)searchBarTextDidEndEditing:(UISearchBar *)bar
{
}

- (void)searchBar:(UISearchBar *)bar textDidChange:(NSString *)searchText
{
  [self filterTunebooks];
}

- (void)searchBarCancelButtonClicked:(UISearchBar *)bar
{
  [self filterTunebooks];
	
	[bar resignFirstResponder];
	bar.text = @"";
}

- (void)searchBarSearchButtonClicked:(UISearchBar *)bar
{
	[bar resignFirstResponder];
}

// Called when the Done button is pressed when choosing a playlist to add to
- (IBAction)donePressed:(id)sender
{
  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
  [appDelegate startTransaction];
  NSUInteger tunebookCount = 0;
  NSUInteger tuneCount = [addingTunes count];
  for ( Playlist *tunebook in listContent ) {
    if ( tunebook.selected ) {
      tunebookCount++;
      for (Tune* tune in addingTunes) {
        [tunebook addTune:tune];
      }
    }
    tunebook.selected = NO;
  }
  [appDelegate commitTransaction];
  self.addingTunes = nil;

  [[NSNotificationCenter defaultCenter] postNotificationName:@"databaseChanged" object:self];
  [self dismissViewControllerAnimated:YES completion:nil];

  NSString* message;
  if ( tuneCount == 1 && tunebookCount == 1 )
    message = @"1 tune was added to a tunebook.";
  else if ( tuneCount == 1 )
    message = [NSString stringWithFormat:@"1 tune was added to %lu tunebooks.", (unsigned long)tunebookCount, nil];
  else if ( tunebookCount == 1 )
    message = [NSString stringWithFormat:@"%lu tunes were added to a tunebook.", (unsigned long)tuneCount, nil];
  else
    message = [NSString stringWithFormat:@"%lu tunes were added to %lu tunebooks.", (unsigned long)tuneCount, (unsigned long)tunebookCount, nil];
  UIAlertView *alert = [[UIAlertView alloc]
    initWithTitle: @"Add to Tunebook"
    message:message
    delegate:nil
    cancelButtonTitle:nil 
    otherButtonTitles:@"OK", nil];
  [alert show];
}

- (IBAction)cancelPressed:(id)sender
{
  for ( Playlist *tunebook in listContent )
    tunebook.selected = NO;
  [self dismissViewControllerAnimated:YES completion:nil];
}

enum {
  kTagCreateTunebook = 1000,
  kTagRenameTunebook,
};

- (void)createNewTunebook
{
  UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:NSLocalizedString(@"New Tunebook", @"") message:@"Enter the name of the new tunebook:" delegate:self cancelButtonTitle:@"Cancel" otherButtonTitles:NSLocalizedString(@"OK", @""), nil];
  alertView.tag = kTagCreateTunebook;
  [alertView setAlertViewStyle:UIAlertViewStylePlainTextInput];
  UITextField *textfield = [alertView textFieldAtIndex: 0];
  [textfield setText:@"New Tunebook"];
  [alertView show];
//  [textfield selectAll:self];
//  [UIMenuController sharedMenuController].menuVisible = NO;
  [alertView release];
}

- (void)createNewTunebookWithName:(NSString*)name
{
  Playlist *playlist = [[Playlist alloc] init];
  playlist.name = name;
  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
  [appDelegate addPlaylist:playlist];
  [self filterTunebooks];
}

static Playlist* sRenameTunebook;

- (void)renameTunebook:(Playlist*)tunebook
{
  sRenameTunebook = tunebook;
  UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:NSLocalizedString(@"Rename Tunebook", @"") message:@"Enter the new name for the tunebook:" delegate:self cancelButtonTitle:@"Cancel" otherButtonTitles:NSLocalizedString(@"OK", @""), nil];
  alertView.tag = kTagRenameTunebook;
  [alertView setAlertViewStyle:UIAlertViewStylePlainTextInput];
  UITextField *textfield =  [alertView textFieldAtIndex: 0];
  [textfield setText:tunebook.name];
  [alertView show];
//  [textfield selectAll:self];
//  [UIMenuController sharedMenuController].menuVisible = NO;
  [alertView release];
}

- (void)renameTunebookWithName:(NSString*)name
{
  if ( ![sRenameTunebook.name isEqualToString:name] ) {
    [sRenameTunebook updateName:name];
    [[NSNotificationCenter defaultCenter] postNotificationName:@"playlistsChanged" object:self];
  }
}

- (void)alertView:(UIAlertView *)alertView didDismissWithButtonIndex:(NSInteger)buttonIndex
{
  if ( alertView.tag == kTagCreateTunebook ) {
    if ( buttonIndex == 1 ) {
      UITextField *textfield =  [alertView textFieldAtIndex: 0];
      NSString* title = [textfield text];
      if ( title.length == 0 )
        title = @"New Tunebook";
      [self createNewTunebookWithName:title];
    }
  }
  else if ( alertView.tag == kTagRenameTunebook ) {
    if ( buttonIndex == 1 ) {
      UITextField *textfield =  [alertView textFieldAtIndex: 0];
      NSString* title = [textfield text];
      if ( title.length != 0 )
        [self renameTunebookWithName:title];
    }
  }
}
@end



@implementation PlaylistDetailsController

@synthesize tableView, indexPath;

- (void)dealloc
{	
	[super dealloc];
}

- (Playlist*)playlist
{
  return playlist;
}

- (void)setPlaylist:(Playlist*)p
{
	playlist = p;
  [self.tableView reloadData];
  // Set the text early so we don't get a flash of the old text
  if ( textField )
    textField.text = playlist.name;
}

- (void)viewDidLoad
{	
  [super viewDidLoad];
  self.tableView.scrollsToTop = NO;
}

- (void)viewDidAppear:(BOOL)animated
{
  textField.text = playlist.name;
  [super viewDidAppear:animated];
  if ( [playlist.name isEqualToString:@"New Tunebook"] )
    [textField selectAll:self];
  [textField becomeFirstResponder];
}

- (void)viewWillDisappear:(BOOL)animated
{
  [textField resignFirstResponder];
  if ( ![playlist.name isEqualToString:textField.text] ) {
    [playlist updateName:textField.text];
    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
      [tableView reloadData];
    else
      [[NSNotificationCenter defaultCenter] postNotificationName:@"playlistsChanged" object:self];
  }
}

#pragma mark UITableViewDataSource

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
  return 1;
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
{
  return @"Tunebook Name";
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
  return 1;
}

- (UITableViewCell *)tableView:(UITableView *)tv cellForRowAtIndexPath:(NSIndexPath *)indexPath {
  static NSString *kNameCellIdentifier = @"NameCell";

  EditTextTableCell* cell = (EditTextTableCell*)[tv dequeueReusableCellWithIdentifier:kNameCellIdentifier];
  if ( cell == nil ) {
    cell = [[[EditTextTableCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:kNameCellIdentifier] autorelease];
    cell.textField.returnKeyType = UIReturnKeyDone;
    cell.textField.delegate = self;
    cell.textField.placeholder = @"New Tunebook";
  }
  cell.textField.text = playlist.name;
  textField = cell.textField;

  return cell;
}

-(BOOL)textFieldShouldBeginEditing:(UITextField *)textField
{
  // Try to hide the edit menu when selecting all for new tunebooks
  [UIMenuController sharedMenuController].menuVisible = NO;
  return YES;
}

- (BOOL)textFieldShouldReturn:(UITextField *)textField
{
  if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad && playlist.type == kPlaylistTypeSimple ) {
    // New way of creating tunebooks: go straight to playlist and start adding tunes.
    // Don't do it for iPhone - it looks too confusing as all views are fullscreen.
    MainViewController *controller = [[MainViewController alloc] initWithNibName:@"MainViewToolbar" bundle:nil];
    controller.playlist = playlist;
    UINavigationController *navigationController = self.navigationController;
    [navigationController popViewControllerAnimated:NO];
    [navigationController pushViewController:controller animated:YES];

    MainViewController *addTunesController = [[MainViewController alloc] initWithNibName:@"AddTunes" bundle:nil];
    addTunesController.playlist = nil;
    addTunesController.choosingTunes = playlist;
    addTunesController.modalPresentationStyle = UIModalPresentationFormSheet;
    [self presentViewController:addTunesController animated:YES completion:nil];
    [controller.tableView reloadData];
  }
  else
    [self.navigationController popViewControllerAnimated:YES];
  return true;
}

@end

