#import "SafariWebViewController.h"
#import "ImportProgressController.h"
#import "BrowserViewController.h"

@implementation SafariWebViewController

@synthesize urlField;
@synthesize webView;
@synthesize activityIndicator;
@synthesize currentURLRequest;

- (void)viewDidLoad
{
  urlField.delegate = self;
  webView.delegate = self;

  if ( [self respondsToSelector:@selector(setEdgesForExtendedLayout:)] )
    self.edgesForExtendedLayout = UIRectEdgeNone;

  if ([[[UIDevice currentDevice] systemVersion] floatValue] < 7.0) {
    CGRect r = webView.frame;
    r.origin.y -= 20;
    r.size.height += 20;
    webView.frame = r;

    r = _toolbar.frame;
    r.origin.y -= 20;
    _toolbar.frame = r;

    r = activityIndicator.frame;
    r.origin.y -= 20;
    activityIndicator.frame = r;
  }

  UIImage *image = [UIImage imageNamed:@"bonjour.png"];
  UIBarButtonItem *item = [[UIBarButtonItem alloc] initWithImage:image style:UIBarButtonItemStyleBordered target:self action:@selector(showBonjourBrowser:)];//FIXME
  self.navigationItem.rightBarButtonItem = item;

  webView.delegate = self;
  activityIndicator.hidden = YES;

  backButton.enabled = NO;
  forwardButton.enabled = NO;

  [self loadHomePage];
}

- (void)dealloc
{	
  [webView release];
	[super dealloc];
}

- (void)loadHomePage
{
	NSString *path = [[NSBundle mainBundle] pathForResource:@"index" ofType:@"html"];
  NSURL *baseURL = [NSURL fileURLWithPath:path];
	NSFileHandle *readHandle = [NSFileHandle fileHandleForReadingAtPath:path];
	NSMutableString *htmlString = [[NSMutableString alloc] initWithData:[readHandle readDataToEndOfFile] encoding:NSUTF8StringEncoding];
	[webView loadHTMLString:htmlString baseURL:baseURL];
	[htmlString release];
}

-(void)viewDidLayoutSubviews
{
  // Move the activity indicator over the right side of the address field.
  CGRect urlFrame = urlField.frame;
  CGRect activityFrame = activityIndicator.frame;
  activityFrame.origin.x = urlFrame.origin.x + urlFrame.size.width - activityFrame.size.width - 4;
  activityIndicator.frame = activityFrame;
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
  return YES;
}

- (void) browserViewController:(BrowserViewController*)bvc didResolveInstance:(NSNetService*)ref
{
  [[self navigationController] popViewControllerAnimated:YES];
  NSString *s = [NSString stringWithFormat:@"http://%@:%d", [ref hostName], (int)[ref port]];
  NSURL *url = [NSURL URLWithString:s];
  [urlField setText:s];
  NSURLRequest *request = [NSURLRequest requestWithURL:url];
  [self.webView loadRequest:request];
}

- (IBAction)showBonjourBrowser:(id)sender
{
  BrowserViewController *controller = [[BrowserViewController alloc] initWithTitle:@"Bonjour" showDisclosureIndicators:NO showCancelButton:YES];
  controller.delegate = self;
  if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
    UIPopoverController* popover = [[UIPopoverController alloc] initWithContentViewController:controller];
    [popover setPopoverContentSize:CGSizeMake(320, 480) animated:NO];
    [popover presentPopoverFromBarButtonItem:sender permittedArrowDirections:UIPopoverArrowDirectionUp animated:YES];
  }
  else {
    [[self navigationController] pushViewController:controller animated:YES];
  }
  [controller searchForServicesOfType:@"_http._tcp." inDomain:@"local"];
  [controller autorelease];
}

- (BOOL)webView:(UIWebView *)webView shouldStartLoadWithRequest:(NSURLRequest *)request navigationType:(UIWebViewNavigationType)navigationType;
{
  NSURL *url = [request URL];
  if ( url ) {
    if ( ![[url scheme] isEqualToString:@"file"] )
      [urlField setText:[url absoluteString]];
    NSString *path = [url path];
    if ( path ) {
      NSString *file = [path lastPathComponent];
      NSString *extension = [path pathExtension];
      self.currentURLRequest = request;
      // thesession.org has URLs like "tunes/27/abc"
      if ( [extension length] == 0 && [[url host] isEqualToString:@"thesession.org"] && [path rangeOfString:@"/abc"].length != 0 ) {
        file = @"this link";
        extension = @"abc";
      }
      if ( extension && [extension caseInsensitiveCompare:@"abc"] == 0 ) {
        UIActionSheet *sheet = [[UIActionSheet alloc]
          initWithTitle:[NSString stringWithFormat:@"Import tunes from '%@'?", file]
          delegate:self
          cancelButtonTitle:@"Cancel"
          destructiveButtonTitle:@"Import"
          otherButtonTitles:nil
        ];
        [sheet showInView:self.view];
        return NO;
      }
    }
  }
  return YES;
}

- (BOOL)textFieldShouldReturn:(UITextField *)textField
{
  [urlField resignFirstResponder];
  NSURL *url = [NSURL URLWithString:[urlField text]];
  if ( [url scheme] == nil )
    url = [NSURL URLWithString:[@"http://" stringByAppendingString:[urlField text]]];
  NSURLRequest *request = [NSURLRequest requestWithURL:url];
  [self.webView loadRequest:request];
  return YES;
}

- (void)webViewDidStartLoad:(UIWebView *)webView
{
  activityIndicator.hidden = NO;
  [activityIndicator startAnimating];
}

- (void)webViewDidFinishLoad:(UIWebView *)view
{
  [activityIndicator stopAnimating];
  activityIndicator.hidden = YES;
  backButton.enabled = view.canGoBack;
  forwardButton.enabled = view.canGoForward;
  importButton.enabled = currentURLRequest != nil;
}

- (void)webView:(UIWebView *)webView didFailLoadWithError:(NSError *)error
{
}

- (IBAction) back:(id)sender
{
  if (![webView canGoBack]) {
    [self loadHomePage];
    return;
  }
  [webView goBack];
}

- (IBAction) forward:(id)sender
{
  [webView goForward];
}

- (IBAction) importABC:(id)sender
{
  ImportProgressController *controller = [[ImportProgressController alloc] initWithNibName:@"ImportProgress" bundle:nil];
  if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
    controller.modalPresentationStyle = UIModalPresentationFormSheet;
  [self presentViewController:controller animated:YES completion:nil];
//  [self presentModalViewController:controller animated:YES];
  [controller importRequest:currentURLRequest];
  [currentURLRequest release];
}


- (void)actionSheet:(UIActionSheet *)actionSheet didDismissWithButtonIndex:(NSInteger)buttonIndex
{
  if ( buttonIndex == 0 ) {
    ImportProgressController *controller = [[ImportProgressController alloc] initWithNibName:@"ImportProgress" bundle:nil];
    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
      controller.modalPresentationStyle = UIModalPresentationFormSheet;
    [self presentViewController:controller animated:YES completion:nil];
//    [self presentModalViewController:controller animated:YES];
    [controller importRequest:currentURLRequest];
    [currentURLRequest release];
  }
}

- (IBAction) donePressed:(id)sender
{
  [self dismissModalViewControllerAnimated:YES];
}

@end
