#import "SettingsController.h"
#import "AppDelegate.h"
#import "OnOffTableCell.h"
#import "SliderTableCell.h"
#import "WebServerController.h"
#import "WebViewController.h"
#import "TuneViewController.h"
#import "TopLevelViewController.h"
#import "ImportProgressController.h"
#import "TunePlayer.h"

static int kShareSection = -1;
static int kTunesSection = -1;
static int kPlayerSection = -1;
static int kViewSection = -1;
#if ABC_FONTS
static int kMusicSection = -1;
#endif
static int kManageSection = -1;
static int kInfoSection = -1;
static int kNumSections = -1;

// kPlayerSection
enum {
  kGuitarChordsRow,
#ifdef TUNEBOOK_COUN_IN
  kCountInRow,
#endif
  kLoopRow,
  kMelodyRow,
  kTrebleRow,
  kBassRow,
  kPlayerSectionNumRows
};

// kShareSection
enum {
  kShareImportRow,
  kShareImportWebRow,
  kShareWebServerRow,
  kShareNumRows
};

// kManageSection
enum {
  kManageDeleteAllRow,
  kManageNumRows
};

// kViewSection
enum {
  kViewTransitionsRow,
  kViewLeftRightRow,
  kViewScalingRow,
#if TUNEBOOK_FORMAT_FILES
  kViewFormatRow,
#endif
  kViewNumRows
};

UIFont* gFonts[4];//FIXME

#if ABC_FONTS
@interface FontPickerController : UITableViewController {
  NSArray* fontNames;
  NSInteger row;
}

@property (nonatomic) NSInteger row;
@end

@implementation FontPickerController
@synthesize row;

- (id)init {
	self = [super init];
	if ( self ) {
    fontNames = [[UIFont familyNames] sortedArrayUsingSelector:@selector(compare:)];
    [fontNames retain];
	}
	return self;
}

-(void) dealloc {
  [fontNames release];
  [super dealloc];
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
  return [fontNames count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
  UITableViewCell* cell = [tableView dequeueReusableCellWithIdentifier:@"FontPickerCell"];
  if (cell == nil) {
    cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:@"FontPickerCell"] autorelease];
  }
  cell.textLabel.text = [fontNames objectAtIndex:indexPath.row];
  cell.textLabel.font = [UIFont fontWithName:[fontNames objectAtIndex:indexPath.row] size:20];
  return cell;
}

- (void)tableView:(UITableView *)tv didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
  gFonts[row] = [UIFont fontWithName:[fontNames objectAtIndex:indexPath.row] size:20];
//  [self.navigationController popViewControllerAnimated:YES];
  [[NSNotificationCenter defaultCenter] postNotificationName:@"musicPreferencesChanged" object:nil];
}

@end
#endif

#if TUNEBOOK_FORMAT_FILES
#import "Format.h"

@interface FormatPickerController : UITableViewController {
}
@end

@implementation FormatPickerController

- (id)init
{
	self = [super init];
	if ( self ) {
	}
	return self;
}

-(void) dealloc
{
  [super dealloc];
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
  NSMutableArray* formats = appDelegate.formatFiles;
  return [formats count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
  NSMutableArray* formats = appDelegate.formatFiles;
  Format* format = [formats objectAtIndex:indexPath.row];
  UITableViewCell* cell = [tableView dequeueReusableCellWithIdentifier:@"FormatPickerCell"];
  if (cell == nil) {
    cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:@"FormatPickerCell"] autorelease];
  }
  cell.textLabel.text = format.name;
  return cell;
}

- (void)tableView:(UITableView *)tv didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
  extern int gFormat;//FIXME
  gFormat = indexPath.row;
//  [self.navigationController popViewControllerAnimated:YES];
  [[NSNotificationCenter defaultCenter] postNotificationName:@"musicPreferencesChanged" object:nil];
}

@end
#endif

@interface VoicePickerController : UITableViewController {
  NSInteger _row;
}

- (id)initWithRow:(NSInteger)row;
@end

@implementation VoicePickerController

- (id)initWithRow:(NSInteger)row
{
  self = [super init];
  if ( self )
    _row = row;
  return self;
}

- (void)viewDidLoad
{
  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
  int voice = -1;
  if ( _row == kMelodyRow )
    voice = appDelegate.playerVoice;
  if ( _row == kTrebleRow )
    voice = appDelegate.playerVoiceTrebleAccompaniment;
  if ( _row == kBassRow )
    voice = appDelegate.playerVoiceBassAccompaniment;

  self.clearsSelectionOnViewWillAppear = NO;
  self.tableView.allowsSelection = YES;
  if ( voice != -1 ) {
    NSIndexPath *indexPath = [NSIndexPath indexPathForRow:voice inSection:0];

    [self.tableView selectRowAtIndexPath:indexPath
                       animated:NO 
                 scrollPosition:UITableViewScrollPositionMiddle];
  }
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
  return [[TunePlayer tunePlayer] numVoices];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
  NSString* voice = [[TunePlayer tunePlayer] voiceName:(int)indexPath.row];
  UITableViewCell* cell = [tableView dequeueReusableCellWithIdentifier:@"VoicePickerCell"];
  if (cell == nil) {
    cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:@"VoicePickerCell"] autorelease];
  }
  cell.textLabel.text = voice;
  return cell;
}

- (void)tableView:(UITableView *)tv didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
  TunePlayer* tunePlayer = [TunePlayer tunePlayer];
  if ( _row == kMelodyRow )
    [tunePlayer setVoice:(int)indexPath.row];
  if ( _row == kTrebleRow )
    [tunePlayer setTrebleVoice:(int)indexPath.row];
  if ( _row == kBassRow )
    [tunePlayer setBassVoice:(int)indexPath.row];
  [[NSNotificationCenter defaultCenter] postNotificationName:@"playbackSettingsChanged" object:nil];
}

@end

@interface TransitionsTableCell : UITableViewCell {
	UILabel *label;
	UISegmentedControl *control;
	NSString* notification;
}

@property (nonatomic, retain) UILabel *label;
@property (nonatomic, retain) UISegmentedControl *control;
@property (nonatomic, retain) NSString *notification;

@end

@implementation TransitionsTableCell

@synthesize label;
@synthesize control;
@synthesize notification;

- (id)initWithStyle:(UITableViewCellStyle)style reuseIdentifier:(NSString *)reuseIdentifier
{
	self = [super initWithStyle:UITableViewCellStyleDefault reuseIdentifier:reuseIdentifier];
	if ( self ) {
    CGRect frame = CGRectMake(18.0, 44.0, 264.0, 32.0);
    control = [[UISegmentedControl alloc] initWithFrame:frame];
    [control insertSegmentWithTitle:@"None" atIndex:0 animated:NO];
    [control insertSegmentWithTitle:@"Curl" atIndex:1 animated:NO];
    [control insertSegmentWithTitle:@"Slide" atIndex:2 animated:NO];
    [control insertSegmentWithTitle:@"Fade" atIndex:3 animated:NO];
    [control addTarget:self action:@selector(sortAction:) forControlEvents:UIControlEventValueChanged];
		[[self contentView] addSubview:control];

    label = [[UILabel alloc] initWithFrame:CGRectMake(14.0, 2.0, 150.0, 40.0)];
    label.backgroundColor = [UIColor clearColor];
    label.font = [UIFont systemFontOfSize:18];
		[[self contentView] addSubview:label];

		self.selectionStyle = UITableViewCellSelectionStyleNone;
	}
	return self;
}

- (void)sortAction:(UISwitch*)c {
	[[NSNotificationCenter defaultCenter] postNotificationName:self.notification object:c];
}

- (void)setSelected:(BOOL)selected animated:(BOOL)animated {
	[super setSelected:selected animated:animated];
}

@end


@interface FontTableCell : UITableViewCell<UIPickerViewDataSource, UIPickerViewDelegate> {
	UILabel *label;
	UIPickerView *control;
	NSString* notification;
  NSArray* fontNames;
}

@property (nonatomic, retain) UILabel *label;
@property (nonatomic, retain) UIPickerView *control;
@property (nonatomic, retain) NSString *notification;

@end

@implementation FontTableCell

@synthesize label;
@synthesize control;
@synthesize notification;

- (id)initWithStyle:(UITableViewCellStyle)style reuseIdentifier:(NSString *)reuseIdentifier
{
	self = [super initWithStyle:UITableViewCellStyleDefault reuseIdentifier:reuseIdentifier];
	if ( self ) {
    fontNames = [UIFont familyNames];
    [fontNames retain];
    CGRect frame = CGRectMake(18.0, 44.0, 300.0, 32.0);
    control = [[UIPickerView alloc] initWithFrame:frame];
    [control setDelegate:self];
		[[self contentView] addSubview:control];
		
		label = [[UILabel alloc] initWithFrame:CGRectMake(20.0, 2.0, 150.0, 40.0)];
    label.backgroundColor = [UIColor clearColor];
		[[self contentView] addSubview:label];
		self.selectionStyle = UITableViewCellSelectionStyleNone;
	}
	return self;
}

#pragma mark UIPickerView delegate methods

- (NSInteger)numberOfComponentsInPickerView:(UIPickerView *)picker
{
  return 1;
}

- (NSInteger)pickerView:(UIPickerView *)picker numberOfRowsInComponent:(NSInteger)col
{
  return [fontNames count];
}

//- (int)rowForFont:(NSString *)fontName
//{
//  int count = [fontNames count];
//  for (int i = 0; i < count; i++)
//    if ([[fontNames objectAtIndex:i] isEqualToString:fontName])
//      return i;
//  return 0;
//}

#pragma mark UIPickerView delegate methods

//- (CGFloat)pickerView:(UIPickerView *)pickerView widthForComponent:(NSInteger)component
//{
//  return 40.0f;
//}

//- (CGFloat)pickerView:(UIPickerView *)pickerView rowHeightForComponent:(NSInteger)component
//{
//  return 20.0f;
//}

- (NSString *)pickerView:(UIPickerView *)pickerView titleForRow:(NSInteger)row forComponent:(NSInteger)component
{
  return [fontNames objectAtIndex:row];
}


@end


@implementation SettingsController

- (void)dealloc
{	
	[super dealloc];
}

- (void)viewDidLoad
{	
  [super viewDidLoad];

  int section = 0;
  if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
    kShareSection = section++;
  kTunesSection = section++;
  kPlayerSection = section++;
  kViewSection = section++;
#if ABC_FONTS
  kMusicSection = section++;
#endif
  if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
    kManageSection = section++;
  kInfoSection = section++;
  kNumSections = section++;

  self.tableView.scrollsToTop = NO;
  [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(playbackSettingsChanged) name:@"playbackSettingsChanged" object:nil];
  self.navigationItem.title = @"Settings";
}

- (void)playbackSettingsChanged
{
  [self.tableView reloadData];
}

- (CGSize)contentSizeForViewInPopoverView
{
  return CGSizeMake(320.0, 440.0);
}

#pragma mark -
#pragma mark UITableViewDataSource

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
  return kNumSections;
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
{
  if ( section == kTunesSection )
    return @"General";
  if ( section == kPlayerSection )
    return @"Player";
  if ( section == kViewSection )
    return @"View";
#if ABC_FONTS
  if ( section == kMusicSection )
    return @"Music";
#endif
  if ( section == kShareSection )
    return @"Sharing";
  if ( section == kManageSection )
    return @"Tunes";
  if ( section == kInfoSection )
    return @"Info";
  return @"Other";
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
  if ( section == kTunesSection ) {
    return 2;
  }
  if ( section == kPlayerSection )
    return kPlayerSectionNumRows;
  if ( section == kViewSection ) {
    int count = 1;
    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
      count = 3;
#if TUNEBOOK_FORMAT_FILES
    AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
    NSMutableArray* formats = appDelegate.formatFiles;
    if ( formats != nil && [formats count] > 0 )
      count++;
#endif
    return count;
  }
#if ABC_FONTS
  if ( section == kMusicSection )
    return 4;
#endif
  if ( section == kShareSection ) {
    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
      return kShareNumRows;
    return 0;
  }
  if ( section == kManageSection ) {
    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
      return kManageNumRows;
    return 0;
	}
  if ( section == kInfoSection ) {
    return 1;
	}
  return 0;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
{
  if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
    if (indexPath.section == kViewSection ) {
      if (  indexPath.row == 0 )
        return 90.0f;
      else if (  indexPath.row == 1 )
        return 44.0f;
      else
        return 75.0f;
    }
  }
  else {
    if (indexPath.section == kViewSection && indexPath.row == 0 )
      return 70.0f;
  }
  return 44.0f;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
  NSInteger row = indexPath.row;

	if ( indexPath.section == kPlayerSection) {
    if ( indexPath.row == kGuitarChordsRow || indexPath.row == kLoopRow
#ifdef TUNEBOOK_COUN_IN
      || indexPath.row == kCountInRow
#endif
    ) {
      OnOffTableCell* cell = (OnOffTableCell*)[tableView dequeueReusableCellWithIdentifier:@"OnOffCell"];
      if ( cell == nil )
        cell = [[[OnOffTableCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:@"OnOffCell"] autorelease];
      [cell.switchControl removeTarget:self action:nil forControlEvents:UIControlEventValueChanged];

      if ( indexPath.row == kGuitarChordsRow ) {
        cell.textLabel.text = @"Guitar Chords";
        cell.switchControl.on = appDelegate.guitarChords;
        [cell.switchControl addTarget:self action:@selector(guitarChordsAction:) forControlEvents:UIControlEventValueChanged];
      }
#ifdef TUNEBOOK_COUN_IN
      else if ( indexPath.row == kCountInRow ) {
        cell.textLabel.text = @"Count in Playback";
        cell.switchControl.on = appDelegate.playbackCountIn;
        [cell.switchControl addTarget:self action:@selector(countInPlaybackAction:) forControlEvents:UIControlEventValueChanged];
      }
#endif
      else if ( indexPath.row == kLoopRow ) {
        cell.textLabel.text = @"Loop Playback";
        cell.switchControl.on = appDelegate.loopPlayback;
        [cell.switchControl addTarget:self action:@selector(loopPlaybackAction:) forControlEvents:UIControlEventValueChanged];
      }
      return cell;
    }
    else {
      UITableViewCell* cell = [tableView dequeueReusableCellWithIdentifier:@"VoiceCell"];
      if (cell == nil)
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:@"VoiceCell"] autorelease];
      cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;

      switch ( indexPath.row ) {
      case kMelodyRow:
        cell.textLabel.text = @"Melody";
        cell.detailTextLabel.text = [[TunePlayer tunePlayer] voiceName:appDelegate.playerVoice];
        break;
      case kTrebleRow:
        cell.textLabel.text = @"Treble Accompaniment";
        cell.detailTextLabel.text = [[TunePlayer tunePlayer] voiceName:appDelegate.playerVoiceTrebleAccompaniment];
        break;
      case kBassRow:
        cell.textLabel.text = @"Bass Accompaniment";
        cell.detailTextLabel.text = [[TunePlayer tunePlayer] voiceName:appDelegate.playerVoiceBassAccompaniment];
        break;
      }

      return cell;
    }
	}
  else if ( indexPath.section == kViewSection) {
    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
      if ( row == 0 ) {
        TransitionsTableCell* cell = (TransitionsTableCell*)[tableView dequeueReusableCellWithIdentifier:@"TransitionsCell"];
        if ( cell == nil ) {
          cell = [[[TransitionsTableCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:@"TransitionsCell"] autorelease];
          cell.label.text = @"Transitions";
          cell.control.selectedSegmentIndex = appDelegate.transitionType;
          [cell.control addTarget:self action:@selector(transitionAction:) forControlEvents:UIControlEventValueChanged];
        }
        return cell;
      }
      else if ( row == 1 ) {
        OnOffTableCell* cell = (OnOffTableCell*)[tableView dequeueReusableCellWithIdentifier:@"OnOffCell"];
        if ( cell == nil )
          cell = [[[OnOffTableCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:@"OnOffCell"] autorelease];
        cell.textLabel.text = @"Left/Right View";
        [cell.switchControl removeTarget:self action:nil forControlEvents:UIControlEventValueChanged];
        cell.switchControl.on = appDelegate.zoomTuneWithIndexKey;
        [cell.switchControl addTarget:self action:@selector(zoomTuneWithIndexKey:) forControlEvents:UIControlEventValueChanged];
        return cell;
      }
      row -= 2;
    }
    if ( row == 0 ) {
      SliderTableCell* cell = (SliderTableCell*)[tableView dequeueReusableCellWithIdentifier:@"SliderTableCell"];
      if ( cell == nil ) {
        cell = [[[SliderTableCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:@"SliderTableCell"] autorelease];
        cell.label.text = @"Note Scaling";
        cell.slider.minimumValue = 1.0f;
        cell.slider.maximumValue = 2.0f;
        cell.slider.value = appDelegate.tuneScaling;
        [cell.slider addTarget:self action:@selector(tuneScalingAction:) forControlEvents:UIControlEventValueChanged];
      }
      return cell;
    }
#if TUNEBOOK_FORMAT_FILES
    if ( row == 1 ) {
      UITableViewCell* cell = [tableView dequeueReusableCellWithIdentifier:@"ManageCell"];
      if (cell == nil)
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:@"ManageCell"] autorelease];
      cell.textLabel.text = @"Format";
      cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
      return cell;
    }
#endif
	}
  else if ( indexPath.section == kTunesSection) {
    {
      if ( row == 0 ) {
        OnOffTableCell* cell = (OnOffTableCell*)[tableView dequeueReusableCellWithIdentifier:@"OnOffCell"];
        if ( cell == nil ) {
          cell = [[[OnOffTableCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:@"OnOffCell"] autorelease];
          cell.textLabel.text = @"Prevent Sleep";
          cell.switchControl.on = appDelegate.preventSleep;
          [cell.switchControl addTarget:self action:@selector(preventSleepAction:) forControlEvents:UIControlEventValueChanged];
        }
        return cell;
      }
      if ( row == 1 ) {
        OnOffTableCell* cell = (OnOffTableCell*)[tableView dequeueReusableCellWithIdentifier:@"NoImportCell"];
        if ( cell == nil ) {
          cell = [[[OnOffTableCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:@"NoImportCell"] autorelease];
          cell.textLabel.text = @"Don't import duplicate tunes";
          cell.switchControl.on = appDelegate.noImportDuplicates;
          [cell.switchControl addTarget:self action:@selector(noImportDuplicatesAction:) forControlEvents:UIControlEventValueChanged];
        }
        return cell;
      }
    }
    return nil;
  }
	else if ( indexPath.section == kShareSection) {
    UITableViewCell* cell = [tableView dequeueReusableCellWithIdentifier:@"ManageCell"];

    if ( cell == nil ) {
      cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:@"ManageCell"] autorelease];
    }
    switch ( indexPath.row ) {
    case kShareImportRow:
      cell.textLabel.text = @"Import Tunes...";
      break;
    case kShareImportWebRow:
      cell.textLabel.text = @"Import Tunes from Web";
      break;
    case kShareWebServerRow:
      cell.textLabel.text = @"Start Web Server";
      break;
    }
    cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
    return cell;
  }
	else if ( indexPath.section == kManageSection) {
    UITableViewCell* cell = [tableView dequeueReusableCellWithIdentifier:@"ManageCell"];

    if ( cell == nil ) {
      cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:@"ManageCell"] autorelease];
    }
    switch ( indexPath.row ) {
    case kManageDeleteAllRow:
      cell.textLabel.text = @"Delete All Tunes";
      break;
    }
    cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
    return cell;
  }
	else if ( indexPath.section == kInfoSection) {
    UITableViewCell* cell = [tableView dequeueReusableCellWithIdentifier:@"ManageCell"];

    if ( cell == nil ) {
      cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:@"ManageCell"] autorelease];
    }
    NSDictionary* info = [[NSBundle mainBundle] infoDictionary];
    
    cell.textLabel.text = [NSString stringWithFormat:@"%@ %@",
      [info objectForKey:@"CFBundleDisplayName"],
      [info objectForKey:@"CFBundleShortVersionString"]
    ];
    cell.accessoryType = UITableViewCellAccessoryNone;
    return cell;
  }
#if ABC_FONTS
  else if ( indexPath.section == kMusicSection) {
		UITableViewCell* cell = [tableView dequeueReusableCellWithIdentifier:@"FontCell"];
		if (cell == nil) {
			cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:@"FontCell"] autorelease];
      switch ( indexPath.row ) {
      case 0:
        cell.textLabel.text = @"Title Font";
        break;
      case 1:
        cell.textLabel.text = @"Subtitle Font";
        break;
      case 2:
        cell.textLabel.text = @"Composer Font";
        break;
      case 3:
        cell.textLabel.text = @"Words Font";
        break;
      }
		}
    cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
		return cell;
  }
#endif

  return nil;
}

- (void)tableView:(UITableView *)tv didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
	if ( indexPath.section == kShareSection) {
    [tv deselectRowAtIndexPath:indexPath animated:NO];
    switch ( indexPath.row ) {
    case kShareImportRow:
      [self showImportTunes];
      break;
    case kShareImportWebRow:
      [self showImportTunesFromWeb];
      break;
    case kShareWebServerRow:
      [self startWebServer];
      break;
    }
  }
	else if ( indexPath.section == kManageSection) {
    [tv deselectRowAtIndexPath:indexPath animated:NO];
    switch ( indexPath.row ) {
    case kManageDeleteAllRow:
      [self deleteAllTunes];
      break;
    }
  }
  [tv deselectRowAtIndexPath:indexPath animated:YES];
#if TUNEBOOK_FORMAT_FILES
  if ( indexPath.section == kViewSection) {
    NSInteger row = indexPath.row;
    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
      row -= 2;
    if ( row == 1 ) {
      FormatPickerController *controller = [[FormatPickerController alloc] init];
      [[self navigationController] pushViewController:controller animated:YES];
    }
  }
#endif
#if ABC_FONTS
	if (indexPath.section == kMusicSection && indexPath.row < 4) {
    FontPickerController *controller = [[FontPickerController alloc] init];
    controller.row = indexPath.row;
    [[self navigationController] pushViewController:controller animated:YES];
  }
#endif
  if ( indexPath.section == kPlayerSection) {
    VoicePickerController *controller = [[VoicePickerController alloc] initWithRow:indexPath.row];
    [[self navigationController] pushViewController:controller animated:YES];
    NSString* title = nil;
    switch ( indexPath.row ) {
    case kMelodyRow:
      title = @"Melody";
      break;
    case kTrebleRow:
      title = @"Treble";
      break;
    case kBassRow:
      title = @"Bass";
      break;
    }
    controller.navigationItem.title = title;
  }
}

// Override this to get the corretc controller.
-(UINavigationController*)navigationController
{
  if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
    return [super navigationController];

  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
  return appDelegate.navigationController;
}

- (IBAction)guitarChordsAction:(id)sender
{
  UISwitch *control = (UISwitch *)sender;
  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
  appDelegate.guitarChords = control.on;
}

- (IBAction)countInPlaybackAction:(id)sender
{
  UISwitch *control = (UISwitch *)sender;
  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
  appDelegate.playbackCountIn = control.on;
}

- (IBAction)loopPlaybackAction:(id)sender
{
  UISwitch *control = (UISwitch *)sender;
  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
  appDelegate.loopPlayback = control.on;
}

- (IBAction)zoomTuneWithIndexKey:(id)sender
{
  UISwitch *control = (UISwitch *)sender;
  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
  appDelegate.zoomTuneWithIndexKey = control.on;
  [appDelegate.topLevelViewController zoomModeChanged];
}

- (IBAction)preventSleepAction:(id)sender
{
  UISwitch *control = (UISwitch *)sender;
  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
  appDelegate.preventSleep = control.on;
  [UIApplication sharedApplication].idleTimerDisabled = appDelegate.preventSleep;
}

- (IBAction)noImportDuplicatesAction:(id)sender
{
  UISwitch *control = (UISwitch *)sender;
  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
  appDelegate.noImportDuplicates = control.on;
}

- (IBAction)transitionAction:(id)sender
{
  UISegmentedControl *control = (UISegmentedControl *)sender;
  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
  appDelegate.transitionType = (int)control.selectedSegmentIndex;
}

- (IBAction)tuneScalingAction:(id)sender
{
  UISlider *control = (UISlider *)sender;
  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
  appDelegate.tuneScaling = control.value;
  [[NSNotificationCenter defaultCenter] postNotificationName:@"musicPreferencesChanged" object:nil];
}

- (void)documentPicker:(UIDocumentPickerViewController *)controller didPickDocumentAtURL:(NSURL *)url {
  if (controller.documentPickerMode == UIDocumentPickerModeImport) {
    ImportProgressController *controller = [[ImportProgressController alloc] initWithNibName:@"ImportProgress" bundle:nil];
    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
      controller.modalPresentationStyle = UIModalPresentationFormSheet;
    [self presentViewController:controller animated:YES completion:nil];
    [controller import:url];
  }
}

- (void)showImportTunes
{
  UIDocumentPickerViewController *documentPicker = [[UIDocumentPickerViewController alloc] initWithDocumentTypes:@[@"public.text", @"public.plain-text", @"public.utf8-plain-text", @"public.alembic"]
    inMode:UIDocumentPickerModeImport];
  documentPicker.delegate = self;
  documentPicker.modalPresentationStyle = UIModalPresentationFormSheet;
  [self presentViewController:documentPicker animated:YES completion:nil];
}

- (void)showImportTunesFromWeb
{
  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
  NetworkStatus netStatus = [appDelegate.reachability currentReachabilityStatus];
  if ( netStatus == NotReachable ) {
    UIAlertController* alertController = [UIAlertController alertControllerWithTitle:@"Error" message:@"You can't browse the Web because you have no Internet connection." preferredStyle:UIAlertControllerStyleAlert];
    [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil]];
    [self presentViewController:alertController animated:YES completion:nil];
  } else {
    WebViewController *controller = [[WebViewController alloc] initWithNibName:@"WebView" bundle:nil];
    controller.modalPresentationStyle = UIModalPresentationFullScreen;
    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
      [appDelegate.tuneViewController dismissPopoverController];
    [appDelegate.topLevelViewController presentViewController:controller animated:YES completion:nil];
  }
}

- (void)startWebServer
{
  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
  NetworkStatus netStatus = [appDelegate.reachability currentReachabilityStatus];
  if ( netStatus == NotReachable ) {
    UIAlertController* alertController = [UIAlertController alertControllerWithTitle:@"Error" message:@"Couldn't start a Web server because you have no Internet connection." preferredStyle:UIAlertControllerStyleAlert];
    [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil]];
    [self presentViewController:alertController animated:YES completion:nil];
  } else {
    WebServerController *controller = [[WebServerController alloc] initWithNibName:@"WebServer" bundle:nil];
    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
      controller.modalPresentationStyle = UIModalPresentationFormSheet;
      [appDelegate.tuneViewController dismissPopoverController];
    }
    [appDelegate.topLevelViewController presentViewController:controller animated:YES completion:nil];
  }
}

- (void)deleteAllTunes
{
  UIActionSheet *sheet = [[UIActionSheet alloc]
    initWithTitle:@"Delete Tunes"
    delegate:self
    cancelButtonTitle:@"Don't Delete"
    destructiveButtonTitle:@"Delete"
    otherButtonTitles:nil
  ];
  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
  [sheet showInView:appDelegate.rootView];
}

#pragma mark UIActionSheetDelegate methods

- (void)actionSheet:(UIActionSheet *)actionSheet didDismissWithButtonIndex:(NSInteger)buttonIndex
{
  if ( buttonIndex == 0 ) {
    AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
    [appDelegate deleteAllTunes];
  }
}

@end

