#import <Foundation/Foundation.h>
#import <AudioToolbox/AudioServices.h>
#import <AudioToolbox/AudioToolbox.h>
#include <fluidsynth.h>

#define TUNEBOOK_AUDIO_TOOLBOX 0

@interface Voice : NSObject {
  fluid_preset_t _preset;
}

- (id)initWithPreset:(fluid_preset_t)preset;
- (NSString*)name;
- (int)bank;
- (int)number;
@end

@interface TunePlayer : NSObject {
  NSString *filePath;
  float speed;
  int transpose;
  int voice;
  int trebleVoice;
  int bassVoice;
  float volume;
  BOOL isPlaying;
  BOOL isPaused;
  int pauseTime;

#if TUNEBOOK_AUDIO_TOOLBOX
  AUGraph graph;
  AudioUnit samplerUnit;
#else
#endif
  fluid_settings_t* settings;
  fluid_synth_t* synth;
  fluid_audio_driver_t* adriver;
  fluid_player_t* player;
  int sfont_id[8];
  NSMutableArray* _voices;
}

@property (nonatomic, retain) NSString *filePath;
@property (nonatomic) int transpose;
@property (nonatomic) int voice;
@property (nonatomic) float volume;

+ (TunePlayer *)tunePlayer;

- (id)init;

- (void)play;   // Start or continue playback
- (void)playFromTime:(float)time;
- (void)stop;   // Stop playback
- (void)pause;  // Pause playback
- (void)reset;  // Unload MIDI file and stop playing

- (BOOL)playing;
- (BOOL)paused;

- (int)numVoices;
- (NSString *)voiceName:(int)voice;
- (void)setVoice:(int)voice;
- (void)setTrebleVoice:(int)voice;
- (void)setBassVoice:(int)voice;

- (float)speed;
- (void)setSpeed:(float)speed;

- (float)time;
- (void)setTime:(float)time;

- (void)playerFinished:(id)sender;

- (NSArray*)voices;

@end
