#import "TuneSettingsController.h"
#import "AppDelegate.h"
#import "TunePlayer.h"
#import <math.h>

@implementation TuneSettingsView

// Our preset instruments
static int sGMInstruments[] = {
  105, // Banjo
  25, // Guitar
  40, // Violin
  73, // Flute
  21, // Accordion
  0, // Piano
  78, // Whistle
  46, // Harp
};

- (void)fillRoundedRect:(CGRect)rect inContext:(CGContextRef)context
{
  float radius = 8;

  CGContextBeginPath(context);
  CGContextSetGrayFillColor(context, 0.1, 0.8);
  CGContextMoveToPoint(context, CGRectGetMinX(rect) + radius, CGRectGetMinY(rect));
  CGContextAddArc(context, CGRectGetMaxX(rect) - radius, CGRectGetMinY(rect) + radius, radius, 3 * M_PI / 2, 0, 0);
  CGContextAddArc(context, CGRectGetMaxX(rect) - radius, CGRectGetMaxY(rect) - radius, radius, 0, M_PI / 2, 0);
  CGContextAddArc(context, CGRectGetMinX(rect) + radius, CGRectGetMaxY(rect) - radius, radius, M_PI / 2, M_PI, 0);
  CGContextAddArc(context, CGRectGetMinX(rect) + radius, CGRectGetMinY(rect) + radius, radius, M_PI, 3 * M_PI / 2, 0);

  CGContextClosePath(context);
  CGContextFillPath(context);
}

- (void)drawRect:(CGRect)rect
{
  if (floor(NSFoundationVersionNumber) <= NSFoundationVersionNumber_iOS_6_1) {
    CGRect boxRect = self.bounds;
    CGContextRef ctxt = UIGraphicsGetCurrentContext();	
    boxRect = CGRectInset(boxRect, 1.0f, 1.0f);
    [self fillRoundedRect:boxRect inContext:ctxt];
  }
}

- (void)dealloc
{
	[super dealloc];
}

@end

@implementation TuneSettingsController

@synthesize speedSlider, pitchSlider, volumeSlider;
@synthesize player;

- (id)initWithPlayer:(TunePlayer *)tunePlayer
{
  self = [[TuneSettingsController alloc] initWithNibName:@"TuneSettings" bundle:nil];
  if ( self ) {
    self.player = tunePlayer;
  }
  return self;
}

- (void)viewDidLoad
{
  instrumentButtons[0] = instrument1Button;
  instrumentButtons[1] = instrument2Button;
  instrumentButtons[2] = instrument3Button;
  instrumentButtons[3] = instrument4Button;
  instrumentButtons[4] = instrument5Button;
  instrumentButtons[5] = instrument6Button;
  instrumentButtons[6] = instrument7Button;
  instrumentButtons[7] = instrument8Button;
  if (floor(NSFoundationVersionNumber) <= NSFoundationVersionNumber_iOS_6_1)
    self.view.backgroundColor = [UIColor clearColor];
  else
    self.view.backgroundColor = [UIColor whiteColor];

  speedSlider.value = player.speed;
  volumeSlider.value = player.volume;
  currentInstrument = player.voice;
  for ( int i = 0; i < 8; i++ ) {
    UIButton *b = instrumentButtons[i];
    [b setSelected:sGMInstruments[i] == currentInstrument];
  }
}

- (CGSize)contentSizeForViewInPopoverView {
  return [self preferredContentSize];
}

- (CGSize)preferredContentSize {
  return CGSizeMake(292.0, 223.0);
}

- (IBAction)donePressed:(id)sender
{
  [self dismissViewControllerAnimated:YES completion:nil];
}

- (IBAction)speedChanged:(id)sender
{
  [player setSpeed:[speedSlider value]];
}

- (IBAction)pitchChanged:(id)sender
{
  player.transpose = (int)([pitchSlider value] * 127) - 64;
}

- (IBAction)volumeChanged:(id)sender
{
  player.volume = [volumeSlider value];
}

- (IBAction)instrumentChanged:(id)sender
{
  for ( int i = 0; i < 8; i++ ) {
    UIButton *b = instrumentButtons[i];
    if ( b == sender ) {
      [b setSelected:YES];
      [player setVoice:sGMInstruments[i]];
    } else {
      [b setSelected:NO];
    }
  }
}

#if 0//SF
- (NSInteger)numberOfComponentsInPickerView:(UIPickerView *)pickerView
{
  return 1;
}

- (NSInteger)pickerView:(UIPickerView *)pickerView numberOfRowsInComponent:(NSInteger)component
{
  return [player numVoices];
}

- (NSString *)pickerView:(UIPickerView *)pickerView titleForRow:(NSInteger)row forComponent:(NSInteger)component
{
  return [player voiceName:row];
}

- (void)pickerView:(UIPickerView *)pickerView didSelectRow:(NSInteger)row inComponent:(NSInteger)component
{
  [player setVoice:row];
}
#endif

@end
