#import <UIKit/UIKit.h>
#import <MessageUI/MessageUI.h>
#import <QuartzCore/QuartzCore.h>
#import "Tunebook.h"
#import "TuneView.h"
#import "TunePlayer.h"
#import "TuneParser.h"

@class Tune;
@class Playlist;
@class EditABCController;
@class TuneSettingsController;

@interface TextLayer : CALayer
{
  NSTextAlignment alignment;
}

@property (nonatomic) NSTextAlignment alignment;

@end

@interface TuneViewController : UIViewController <UIScrollViewDelegate, UIActionSheetDelegate,
  UIPopoverControllerDelegate,
  UIDocumentInteractionControllerDelegate,
  MFMailComposeViewControllerDelegate,
  UIPrintInteractionControllerDelegate,
  CAAnimationDelegate,
  TuneViewDelegate,
  TuneParserDelegate>
{
  UIScrollView *scrollView;
  IBOutlet UIView *mainView;
  IBOutlet UIToolbar *toolbar;
  IBOutlet UIBarButtonItem *rewindButton;
  IBOutlet UIBarButtonItem *playButton;
  IBOutlet UIBarButtonItem *pauseButton;
  IBOutlet UIBarButtonItem *emailButton;
  IBOutlet UIBarButtonItem *abcButton;
  IBOutlet UIBarButtonItem *actionButton;
  IBOutlet UIBarButtonItem *printButton;
  IBOutlet UIBarButtonItem *printSpacer;
  IBOutlet UISlider *timeSlider;
  IBOutlet EditABCController *editController;
  BOOL showingABC;
  BOOL editingABC;
  TuneSettingsController *settingsController;
  BOOL showingSettings;
  BOOL paginationOn;

	TuneView *tuneView;
  Tune *tune;
  Playlist *playlist;
  NSInteger currentPage;
  NSString *savedDots;
  BOOL barsHidden;
  NSMutableArray *playItems;
  NSMutableArray *stopItems;
  int numTunes;
  int numPages;
  int tuneHeight;
  NSMutableArray *parsedTunes;
  UIActionSheet *saveChangesSheet;
  UIActionSheet *multipleTunesSheet;

  IBOutlet UIBarButtonItem *editButton;
  UIActionSheet*       activeActionSheet;
  UIBarButtonItem*     activeBarButtonItem;
  UIPopoverController* activePopoverController;
  UIActionSheet*       rotateActionSheet;
  UIBarButtonItem*     rotateBarButtonItem;
  NSArray*             rotatePassthroughViews;
  UIViewController*    rotateViewController;
  UIImageView* background;
  UIActionSheet* _actionActionSheet;
  UIBarButtonItem* _actionBarButtonItem;
  id _actionSender;
  NSInteger _editTuneIndex;
  NSInteger _emailTuneIndex;
  NSInteger _printTuneIndex;
  NSInteger _transposeTuneIndex;
  NSInteger _addToTunebookIndex;
  NSInteger _shareIndex;
  NSInteger _exportABCIndex;
  NSInteger _exportPDFIndex;
  NSInteger _exportMIDIIndex;

  NSTimer* timeSliderTimer;

  TextLayer* messageLayer;
}

//@property(retain) id<TuneViewDelegate> tuneViewDelegate;
@property(retain,readwrite) Tune *tune;
@property(retain,readwrite) Playlist *playlist;
@property(retain,readwrite) NSString *savedDots;
@property(nonatomic,readwrite) BOOL paginationOn;
@property(nonatomic,readwrite) BOOL editingABC;

- (id)initWithTune:(Tune *)tune;
- (id)initWithTune:(Tune *)tune fromPlayList:(Playlist *)playlist;

- (void)showTune:(Tune *)tune;
- (void)showTune:(Tune *)tune fromPlayList:(Playlist *)playlist;
- (void)showTune:(Tune *)tune fromPlayList:(Playlist *)playlist direction:(int)direction;

- (void)editABC;
- (void)showMusic;
- (void)showABC;
- (void)hideSettings;
- (void)viewTapped:(id)sender;
- (void)viewDoubleTapped:(id)sender;
- (void)finishedPlaying:(id)sender;

- (IBAction)startPlaying:(id)sender;
- (IBAction)stopPlaying:(id)sender;
- (IBAction)pausePlaying:(id)sender;
- (IBAction)rewindPlaying:(id)sender;
- (IBAction)timeSliderChanged:(id)sender;
- (void)updateTimeSlider:(id)info;
- (IBAction)showSettings:(id)sender;
- (IBAction)emailTune:(id)sender;
- (IBAction)printTune:(id)sender;
- (IBAction)zoomToFit:(id)sender;

- (IBAction)prevTune:(id)sender;
- (IBAction)nextTune:(id)sender;

- (IBAction)showStatusBar;
- (IBAction)hideStatusBar;
- (IBAction)toggleStatusBar;

- (IBAction)editPressed:(id)sender;
- (IBAction)actionPressed:(id)sender;

- (void)dotsChanged;

- (void)presentPopoverController:(UIBarButtonItem *)item controller:(UIViewController*)controller;
- (void)dismissPopoverController;

- (void)showMessage:(NSString*)message;

- (UIView *)viewForZoomingInScrollView:(UIScrollView *)scrollView;     // return a view that will be scaled. if delegate returns nil, nothing happens
- (void)scrollViewDidEndZooming:(UIScrollView *)scrollView withView:(UIView *)view atScale:(CGFloat)scale; // scale between minimum and maximum. called after any 'bounce' animations
- (void)actionSheet:(UIActionSheet *)actionSheet didDismissWithButtonIndex:(NSInteger)buttonIndex;

- (void)displayComposerSheet;
- (void)launchMailAppOnDevice;

- (void)mailComposeController:(MFMailComposeViewController*)controller didFinishWithResult:(MFMailComposeResult)result error:(NSError*)error;

- (void) parsedTune:(TuneParser *)parser tune:(Tune *)tune;

@end
