#import "TuneViewController.h"
#import "EditABCController.h"
#import "Tune.h"
#import "Playlist.h"
#import "TunePlayer.h"
#import "TuneParser.h"
#import "TuneSettingsController.h"
#import "AppDelegate.h"

#import "MainViewController.h"
#import "PlaylistViewController.h"
#import "SearchViewController.h"
#import "ManageTunesController.h"
#import "SettingsController.h"
#import "TopLevelViewController.h"
#import "TransposeViewController.h"
#if TUNEBOOK_RECENTLY_VIEWED
#import "TuneDatabase.h"
#endif

#define TUNEBOOK_SLIDE_TOOLBAR 1 // remove when this works.

//FIXME-if this works, put it in a file
#if 1//TUNEBOOK_DOCUMENT_INTERACTION
@interface DataItem : NSObject <UIActivityItemSource>
- (id)initWithData:(NSData *)data type:(NSString *)type subject:(NSString *)subject previewImage:(UIImage *)previewImage;
@end

@implementation DataItem {
    NSData *_data;
    NSString *_type;
    NSString *_subject;
    UIImage *_image;
};

- (id)initWithData:(NSData *)data type:(NSString *)type subject:(NSString *)subject previewImage:(UIImage *)previewImage
{
    self = [super init];
    if (self) {
        _data = data;
        _type = [type copy];
        _subject = [subject copy];
        _image = previewImage;
    }
    return self;
}

#pragma mark - UIActivityItemSource
- (id)activityViewControllerPlaceholderItem:(UIActivityViewController *)activityViewController
{
  return _data;
}

- (id)activityViewController:(UIActivityViewController *)activityViewController itemForActivityType:(NSString *)activityType
{
  return _data;
}

- (NSString *)activityViewController:(UIActivityViewController *)activityViewController dataTypeIdentifierForActivityType:(NSString *)activityType
{
  return _type;
}


- (NSString *)activityViewController:(UIActivityViewController *)activityViewController subjectForActivityType:(NSString *)activityType
{
  return _subject;
}

- (UIImage *)activityViewController:(UIActivityViewController *)activityViewController thumbnailImageForActivityType:(NSString *)activityType suggestedSize:(CGSize)size
{
    // might need to do some scaling of the image to fit within suggestedSize
  return _image;
}

@end
#endif

static float kTuneMargin() {
  if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
    return 0.0f; // This makes the zoom-to-fit view fill the screen exactly
  return 25.0f;
}

//FIXME-PM
int gTuneHeight;

static NSString *kPlayControlsVisibleKey = @"playControlsVisible";

static TunePlayer *tunePlayer;
static BOOL printingEnabled;

@implementation TuneViewController

@synthesize tune;
@synthesize savedDots;
@synthesize playlist;
@synthesize paginationOn;
@synthesize editingABC;

-(id)initWithTune:(Tune *)aTune
{
  [self initWithTune:aTune fromPlayList:nil];
  return self;
}

-(id)initWithTune:(Tune *)aTune fromPlayList:(Playlist *)pl
{
	self = [super initWithNibName:@"TuneView_iphone" bundle:nil];
	if ( self != nil ) {
    [aTune hydrate];
    self.playlist = pl;
    self.tune = aTune;
		self.title = aTune.title;
    saveChangesSheet = NULL;
    multipleTunesSheet = NULL;
    editController = NULL;
    showingABC = NO;
    editingABC = NO;
    settingsController = nil;
    showingSettings = NO;
    tunePlayer = nil;
    numPages = 1;

    AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
    appDelegate.currentTune = aTune.primaryKey;

    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(finishedPlaying:) name:@"finishedPlaying" object:nil];
	}
	return self;
}

-(void)viewDidLoad
{
  barsHidden = NO;
  if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
    barsHidden = ![[NSUserDefaults standardUserDefaults] boolForKey:kPlayControlsVisibleKey];
  toolbar.alpha = editController == nil && barsHidden ? 0.0f : 1.0f;

  if ( [self respondsToSelector:@selector(setEdgesForExtendedLayout:)] )
    self.edgesForExtendedLayout = UIRectEdgeNone;

  CGRect bounds;
#if 1//Measuring
  numPages = 1;
  if ( tune ) {
    UIGraphicsBeginImageContext( CGSizeMake( 8, 8 ) );
    numPages = [Tune drawTune:tune.dots intoContext:UIGraphicsGetCurrentContext() bounds:CGRectMake(0, 0, 8, 8) scale:1.0f paginate:NO];
    if ( numPages < 1 )
      numPages = 1;
    if ( paginationOn )
      bounds = CGRectMake( 0, 0, PaperWidth, numPages * PaperHeight );
    else {
      tuneHeight = gTuneHeight;
//    int h = scrollView.bounds.size.height;
//    numPages = (gTuneHeight+h-1)/h;
      bounds = CGRectMake( 0, 0, PaperWidth, tuneHeight );
    }
    UIGraphicsEndImageContext();
  } else
    bounds = CGRectMake( 0, 0, PaperWidth, PaperHeight );
#else
  bounds = CGRectMake( 0, 0, PaperWidth, PaperHeight );
#endif
  scrollView = [[UIScrollView alloc] initWithFrame:[mainView bounds]];
	[scrollView setContentSize:bounds.size];
	[scrollView setScrollEnabled:YES];
  scrollView.bouncesZoom = NO;
  scrollView.minimumZoomScale = 0.5f;
  scrollView.maximumZoomScale = 4.0f;
  scrollView.delegate = self;
	scrollView.autoresizesSubviews = YES;
  scrollView.clipsToBounds = YES;
	scrollView.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;

  if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
  //FIXME-have this iphone too?
    background = [[UIImageView alloc] initWithImage:[UIImage imageNamed:@"Linen.jpg"]];
    background.contentMode = UIViewContentModeScaleAspectFill;
    background.autoresizingMask = UIViewAutoresizingFlexibleWidth|UIViewAutoresizingFlexibleHeight;
    scrollView.backgroundColor = [UIColor clearColor];
  }

  tuneView = [[TuneView alloc] initWithFrame:CGRectMake(0.0, 0.0, 1.0, 1.0)];
  tuneView.delegate = self;
  tuneView.autoresizingMask = UIViewAutoresizingFlexibleHeight | UIViewAutoresizingFlexibleWidth;
  tuneView.scale = 1.0f;

  if ( [[UIScreen mainScreen] respondsToSelector:@selector(scale)] && [tuneView respondsToSelector:@selector(contentScaleFactor)] )
    tuneView.contentScaleFactor = [UIScreen mainScreen].scale;

	tuneView.frame = bounds;
  tuneView.tune = tune ? [tune dots] : nil;
  tuneView.contentMode = UIViewContentModeRedraw;

  if (UI_USER_INTERFACE_IDIOM() != UIUserInterfaceIdiomPad) {
    // I've no idea why this doesn't need to be done on the iPad, but if you do
    // the view appears twice at different scales.
    [scrollView addSubview:tuneView];
  }
  [mainView addSubview:scrollView];
	[mainView sendSubviewToBack:scrollView];
  if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
    mainView.clipsToBounds = YES;
    [mainView addSubview:background];
    [mainView sendSubviewToBack:background];
  }

  scrollView.contentInset = UIEdgeInsetsMake( 0, 0, 88, 0 );
  scrollView.directionalLockEnabled = YES;

	[scrollView release];

  self.navigationItem.rightBarButtonItem = (tune && tune.temporary) ? nil : self.editButtonItem;


  // Set up the arrays of toolbar items we'll need
  NSArray *items = toolbar.items;
  playItems = [NSMutableArray arrayWithArray:items];
  [playItems removeObject:playButton];
  [playItems retain];
  stopItems = [NSMutableArray arrayWithArray:items];
  [stopItems removeObject:pauseButton];
  [stopItems retain];

  abcButton = [[[UIBarButtonItem alloc]
    initWithTitle:@"ABC"
    style:UIBarButtonItemStylePlain 
    target:self 
    action:@selector(editABC)] autorelease];
  [abcButton retain];

  Class uipc = NSClassFromString(@"UIPrintInteractionController");
  printingEnabled = uipc && [uipc isPrintingAvailable];

  if ( !printingEnabled ) {
    [playItems removeObject:printButton];
    [stopItems removeObject:printButton];
  } else {
    [playItems removeObject:printSpacer];
    [stopItems removeObject:printSpacer];
  }

  toolbar.items = stopItems;

  if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
    printButton.enabled = NO;
    emailButton.enabled = NO;
    editButton.enabled = NO;
    playButton.enabled = NO;
    rewindButton.enabled = NO;
    timeSlider.enabled = NO;
  }

  [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(playerFinished:) name:@"finishedPlaying" object:nil];
  if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(redrawTune:) name:@"musicPreferencesChanged" object:nil];
  }

  messageLayer = [[TextLayer alloc] init];
  messageLayer.position = CGPointMake( self.view.bounds.size.width/2, self.view.bounds.size.height/2 );
  messageLayer.bounds = CGRectMake( 0, 0, 230, 40 );
  messageLayer.opaque = NO;
  [messageLayer setValue:[UIFont systemFontOfSize:22] forKey:@"font"];
  [messageLayer setValue:@"Text" forKey:@"text"];
  [messageLayer setValue:[UIColor whiteColor] forKey:@"textColor"];
  messageLayer.cornerRadius = 16.0f;
  messageLayer.backgroundColor = [UIColor colorWithRed:0.0f green:0.0f blue:0.0f alpha:0.8f].CGColor;
  [messageLayer setNeedsDisplay];
  messageLayer.needsDisplayOnBoundsChange = YES;
  messageLayer.opacity = 0.0f;
  [self.view.layer addSublayer:messageLayer];

  if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
    UISwipeGestureRecognizer *recognizer;
    recognizer = [[UISwipeGestureRecognizer alloc] initWithTarget:self action:@selector(swipeRight:)];
    [recognizer setDirection:UISwipeGestureRecognizerDirectionRight];
    [[self view] addGestureRecognizer:recognizer];
    [recognizer release]; 

    UISwipeGestureRecognizer *recognizer1;
    recognizer1 = [[UISwipeGestureRecognizer alloc] initWithTarget:self action:@selector(swipeLeft:)];
    [recognizer1 setDirection:UISwipeGestureRecognizerDirectionLeft];
    [[self view] addGestureRecognizer:recognizer1];
    [recognizer1 release];
  }
}

-(void)viewDidAppear:(BOOL)animated
{
  [super viewDidAppear:animated];
  [self becomeFirstResponder];
}

- (void)swipeLeft:(UISwipeGestureRecognizer *)swipeGesture
{
  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
  [appDelegate.topLevelViewController showLeftView:NO];
}

- (void)swipeRight:(UISwipeGestureRecognizer *)swipeGesture
{
  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
  [appDelegate.topLevelViewController showLeftView:YES];
}

- (void)dotsChanged
{
  tuneView.tune = tune.dots;
  [tuneView setNeedsDisplay];
}

- (void)redrawTune:(id)sender
{
  [tuneView setNeedsDisplay];
}

- (void)showTune:(Tune *)t
{
  [self showTune:t fromPlayList:nil];
}

- (void)showTune:(Tune *)t fromPlayList:(Playlist *)aPlaylist
{
  [self showTune:t fromPlayList:aPlaylist direction:1];
}

- (void)showTune:(Tune *)aTune fromPlayList:(Playlist *)aPlaylist direction:(int)direction
{
  self.playlist = aPlaylist;

  [self becomeFirstResponder];

  [aTune hydrate];
  [tune dehydrate];

  if ( aTune == nil && self.tune == nil )
    return;

  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
  appDelegate.currentTune = aTune.primaryKey;

  [self stopPlaying:self];

  self.tune = aTune;

  printButton.enabled = printingEnabled && aTune != nil;
  emailButton.enabled = aTune != nil;
  if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
    editButton.enabled = aTune != nil;
  playButton.enabled = aTune != nil;
  rewindButton.enabled = aTune != nil;
  timeSlider.enabled = aTune != nil;

  numPages = 3; // Enough for editing a fairly long tune
  if ( !paginationOn ) {
    if ( tune ) {
      UIGraphicsBeginImageContext( CGSizeMake( 8, 8 ) );
      numPages = [Tune drawTune:tune.dots intoContext:UIGraphicsGetCurrentContext() bounds:CGRectMake(0, 0, 8, 8) scale:1.0f paginate:NO];
      if ( numPages < 1 )
        numPages = 1;
      if ( !paginationOn ) {
        tuneHeight = gTuneHeight;
        int h = scrollView.bounds.size.height;
        numPages = (tuneHeight+h-1)/h;
      }
      UIGraphicsEndImageContext();
    }
  }

  TuneView *newView = [[TuneView alloc] initWithFrame:CGRectMake(0.0, 0.0, 1.0, 1.0)];
  newView.delegate = self;
  newView.autoresizingMask = UIViewAutoresizingFlexibleHeight | UIViewAutoresizingFlexibleWidth;
  newView.scale = 1.0f;

	newView.frame = tuneView.frame;
  newView.tune = aTune ? [aTune dots] : nil;
  newView.contentMode = UIViewContentModeRedraw;

  float scale = scrollView.bounds.size.width / (PaperWidth - 2*kTuneMargin());
  newView.scale = scale;

  CGSize contentSize;
  contentSize.width = PaperWidth * scale;
  if ( paginationOn )
    contentSize.height = PaperHeight * scale * numPages;
  else
    contentSize.height = MAX( tuneHeight * scale, scrollView.bounds.size.height );

  CGSize scrollFrameSize = scrollView.bounds.size;
  CGPoint frameOrigin;
  frameOrigin.x = (contentSize.width < scrollFrameSize.width) ? ((scrollFrameSize.width - contentSize.width) / 2) : 0;
  frameOrigin.y = (contentSize.height < scrollFrameSize.height) ? ((scrollFrameSize.height - contentSize.height) / 2) : 0;

  [newView setTransform:CGAffineTransformIdentity];
  [newView setFrame:CGRectMake(frameOrigin.x, frameOrigin.y, contentSize.width, contentSize.height)];
  [scrollView setContentSize:contentSize];
  scrollView.contentOffset = CGPointMake(kTuneMargin()*scale, 0.0f);
  scrollView.minimumZoomScale = 0.5f;
  scrollView.maximumZoomScale = 4.0f;

  if ( appDelegate.transitionType != kTransitionNone ) {
    if ( appDelegate.transitionType == kTransitionFade )
      newView.alpha = 0.0f;
    else if ( appDelegate.transitionType == kTransitionSlide ) {
      CGRect r = newView.frame;
      r.origin.x += direction*r.size.width;
      newView.frame = r;
    }
    [UIView beginAnimations:nil context:tuneView];
    [UIView setAnimationDuration:0.5f];
  }
  if ( appDelegate.transitionType == kTransitionCurl )
    [UIView setAnimationTransition:direction == -1 ? UIViewAnimationTransitionCurlDown : UIViewAnimationTransitionCurlUp forView:mainView cache:YES];
  else if ( appDelegate.transitionType == kTransitionFade ) {
    tuneView.alpha = 0.0f;
    newView.alpha = 1.0f;
  }
  else if ( appDelegate.transitionType == kTransitionSlide ) {
    CGRect r = newView.frame;
    r.origin.x -= direction*r.size.width;
    newView.frame = r;

    r = tuneView.frame;
    r.origin.x -= direction*r.size.width;
    tuneView.frame = r;
  }
	[UIView setAnimationDelegate:tuneView];
  [UIView setAnimationDidStopSelector:@selector(removeFromSuperview)];
	[scrollView addSubview: newView];
  if ( appDelegate.transitionType != kTransitionNone )
    [UIView commitAnimations];
  else
    [tuneView removeFromSuperview];
  [tuneView release];
  tuneView = newView;

  self.navigationItem.title = tune.title;

#if TUNEBOOK_RECENTLY_VIEWED
	[appDelegate.database addToRecentlyViewed:tune];
#endif
}

- (void)showMusic
{
	if ( showingABC )
    [self editABC];
}

- (void)showABC
{
  [self hideSettings];
	if ( !showingABC )
    [self editABC];
}

- (void)editABC
{
  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
  EditABCController* editABCController = [EditABCController alloc];
  if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
    [editABCController initWithNibName:@"EditABC" bundle:nil];
  else
    [editABCController initWithNibName:@"EditABC_iphone" bundle:nil];
  editABCController.modalPresentationStyle = UIModalPresentationFullScreen;
  editABCController.editTune = tune;
  if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
    [appDelegate.topLevelViewController presentViewController:editABCController animated:YES completion:nil];
  else
    [self presentViewController:editABCController animated:YES completion:nil];
  [editABCController release];
}

- (void)setEditing:(BOOL)editing animated:(BOOL)animated
{
  if ( editing ) {
    [self showABC];
  } else {
    [self showMusic];
    if ( editController.changed ) {
      if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
        saveChangesSheet = [[UIActionSheet alloc]
          initWithTitle:[NSString stringWithFormat:@"Save Changes to '%@'?", tune.title]
          delegate:self
          cancelButtonTitle:@"Don't Save"
          destructiveButtonTitle:@"Save"
          otherButtonTitles:nil
        ];
      }
      else {
        saveChangesSheet = [[UIActionSheet alloc]
          initWithTitle:[NSString stringWithFormat:@"Save Changes to '%@'?", tune.title]
          delegate:self
          cancelButtonTitle:@"Save"
          destructiveButtonTitle:@"Don't Save"
          otherButtonTitles:nil
        ];
      }
      AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
      [saveChangesSheet showInView:appDelegate.rootView];
    } else {
      [editController release];
      editController = NULL;
    }
  }
}

- (void)actionSheet:(UIActionSheet *)actionSheet didDismissWithButtonIndex:(NSInteger)buttonIndex
{
  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
  if ( actionSheet == _actionActionSheet ) {
    if ( buttonIndex == _editTuneIndex )
      [self editPressed:_actionSender];
    else if ( buttonIndex == _emailTuneIndex )
      [self emailTune:_actionSender];
    else if ( buttonIndex == _printTuneIndex )
      [self printTune:_actionSender];
    else if ( buttonIndex == _transposeTuneIndex )
      [self transposeTune];
    else if ( buttonIndex == _addToTunebookIndex )
      [self addToTunebook:_actionSender];
//    else if ( buttonIndex == _shareIndex )
//      [self shareABCDocument:_actionSender];
    else if ( buttonIndex == _exportABCIndex )
      [self openABCDocumentIn:_actionSender];
    else if ( buttonIndex == _exportPDFIndex )
      [self openPDFDocumentIn:_actionSender];
    else if ( buttonIndex == _exportMIDIIndex )
      [self openMIDIDocumentIn:_actionSender];
    return;
  }

  if ( actionSheet == saveChangesSheet ) {
    if ( buttonIndex == (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad ? 2 : 1)) { // Save
      numTunes = 0;
      parsedTunes = [[NSMutableArray alloc] init];
      [parsedTunes retain];
      TuneParser *parser = [[TuneParser alloc] init];
      parser.delegate = self;
      NSString *dots = editController.text;
      [parser parseString:dots];
      if ( numTunes == 0 ) {
        UIAlertController* alertController = [UIAlertController alertControllerWithTitle:@"Bad ABC" message:@"The ABC does not contain any valid tunes." preferredStyle:UIAlertControllerStyleAlert];
        [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil]];
        [self presentViewController:alertController animated:YES completion:nil];
        [parsedTunes release];
      } else if ( numTunes > 1 ) {
        multipleTunesSheet = [[UIActionSheet alloc]
          initWithTitle:[NSString stringWithFormat:@"Multiple tunes in ABC"]
          delegate:self
          cancelButtonTitle:@"Cancel"
          destructiveButtonTitle:@"Add Extra Tunes"
          otherButtonTitles:nil
        ];
        AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
        [multipleTunesSheet showInView:appDelegate.rootView];
      } else {
        Tune *t = (Tune *)[parsedTunes objectAtIndex:0];
        tune.title = t.title;//FIXME-method to do this...
        tune.dots = t.dots;
        tune.rating = t.rating;
        tune.key = t.key;
        tune.numSharps = t.numSharps;
        tune.timeSigNum = t.timeSigNum;
        tune.timeSigDen = t.timeSigDen;
        if ( [tune inDatabase] )
          [tune saveToDatabase];
        else
          [appDelegate addTune:tune];
        [parsedTunes release];
        [[NSNotificationCenter defaultCenter] postNotificationName:@"databaseChanged" object:self]; 
      }
    } else { // Don't Save
      tuneView.tune = [tune dots];
      [tuneView setNeedsDisplay];
//      tune.dots = savedDots;
    }
  } else if ( actionSheet == multipleTunesSheet ) {
    if ( buttonIndex == 0 ) {
      Tune *t = (Tune *)[parsedTunes objectAtIndex:0];
      tune.title = t.title;
      tune.dots = t.dots;
      tune.rating = t.rating;
      tune.key = t.key;
      tune.numSharps = t.numSharps;
      tune.timeSigNum = t.timeSigNum;
      tune.timeSigDen = t.timeSigDen;
      if ( [tune inDatabase] )
        [tune saveToDatabase];
      else
        [appDelegate addTune:tune];

      // Add the extra tunes
      for ( int i = 1; i < [parsedTunes count]; i++ )
        [appDelegate addTune:(Tune *)[parsedTunes objectAtIndex:i]];
      [[NSNotificationCenter defaultCenter] postNotificationName:@"databaseChanged" object:self]; 
    }
    [parsedTunes release];
  }
  [editController release];
  editController = NULL;
}

- (void) parsedTune:(TuneParser *)parser tune:(Tune *)t
{
  numTunes++;
  [parsedTunes addObject:t];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
  return YES;
}

- (BOOL)shouldAutorotate
{
  return YES;
}

- (UIInterfaceOrientationMask)supportedInterfaceOrientations
{
  return UIInterfaceOrientationMaskAll;
}

- (void)didReceiveMemoryWarning
{
	[super didReceiveMemoryWarning];
}

-(void)dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];
  [scrollView retain];
	[tuneView release];
	[playItems release];
	[stopItems release];
  [settingsController release];
  [abcButton release];
	[messageLayer release];
  [background release];
	[super dealloc];
}

- (IBAction)hideStatusBar
{
  if ( barsHidden )
    return;

  if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
#if TUNEBOOK_PLAY_CONTROLS
    if ( editController == nil ) {
      [UIView beginAnimations:nil context:NULL];
      [UIView setAnimationDuration:0.30];
      toolbar.alpha = 0.0;
      [self hideSettings];
      [UIView commitAnimations];
      barsHidden = true;
      [[NSUserDefaults standardUserDefaults] setBool:!barsHidden forKey:kPlayControlsVisibleKey];
    }
#endif
  }
  else {
    [[self navigationController] setNavigationBarHidden:YES animated:YES];
    CGRect bounds = self.view.bounds;
    float w = bounds.size.width;
    float h = bounds.size.height;
    scrollView.frame = CGRectMake( 0, 0, w, h );
    [UIView beginAnimations:nil context:NULL];
    [UIView setAnimationDuration:0.20];
#if TUNEBOOK_SLIDE_TOOLBAR
    CGRect r = toolbar.frame;
    r = CGRectOffset( r, 0, r.size.height );
    toolbar.frame = r;
#else
    toolbar.alpha = 0.0;
#endif
    [self hideSettings];
    [UIView commitAnimations];
    barsHidden = true;
  }
}

- (IBAction)showStatusBar
{
  if ( !barsHidden )
    return;

  if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
#if TUNEBOOK_PLAY_CONTROLS
    if ( editController == nil ) {
      [UIView beginAnimations:nil context:NULL];
      [UIView setAnimationDuration:0.30];
      toolbar.alpha = 1.0;
      [self hideSettings];
      [UIView commitAnimations];
      barsHidden = false;
      [[NSUserDefaults standardUserDefaults] setBool:!barsHidden forKey:kPlayControlsVisibleKey];
    }
#endif
  }
  else {
    [[self navigationController] setNavigationBarHidden:NO animated:YES];
    CGRect bounds = self.view.bounds;
    float w = bounds.size.width;
    float h = bounds.size.height;
    float t = 44; // toolbar height
    scrollView.frame = CGRectMake( 0, 0, w, h-t );
    [UIView beginAnimations:nil context:NULL];
    [UIView setAnimationDuration:0.20];
#if TUNEBOOK_SLIDE_TOOLBAR
    CGRect r = toolbar.frame;
    r = CGRectOffset( r, 0, -r.size.height );
    toolbar.frame = r;
#else
    toolbar.alpha = 1.0;
#endif
    [UIView commitAnimations];
    barsHidden = false;
  }
}

- (IBAction)toggleStatusBar
{
  if ( barsHidden )
    [self showStatusBar];
  else
    [self hideStatusBar];
}

-(BOOL)canBecomeFirstResponder
{
  return YES;
}

- (void)viewTapped:(id)sender
{
  if ( editingABC ) {
    [self becomeFirstResponder];
  }
  else {
    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
      AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
      [appDelegate.topLevelViewController toggleLeftView];
    }
    else {
      [self toggleStatusBar];
    }
  }
}

- (IBAction)zoomToFit:(id)sender
{
  if ( self.tune == nil ) {
    tuneView.frame = scrollView.frame;
    scrollView.contentSize = scrollView.frame.size;
    return;
  }

  float scale = scrollView.bounds.size.width / (PaperWidth - 2*kTuneMargin());
  tuneView.scale = scale;

  tuneView.transform = CGAffineTransformIdentity;
  if ( paginationOn ) {
    tuneView.frame = CGRectMake( 0, 0, PaperWidth*scale, numPages*PaperHeight*scale );
    scrollView.contentSize = CGSizeMake( PaperWidth*scale, numPages*PaperHeight*scale );
  }
  else {
    if ( tuneHeight == 0 )
      tuneHeight = PaperHeight;
    tuneView.frame = CGRectMake( 0, 0, PaperWidth*scale, tuneHeight*scale );
    scrollView.contentSize = CGSizeMake( PaperWidth*scale, MAX( tuneHeight * scale, scrollView.bounds.size.height ) );
  }
  scrollView.contentOffset = CGPointMake(kTuneMargin()*scale, 0.0f);
  [tuneView setNeedsLayout];
  [tuneView setNeedsDisplay];
  if ( self.tune != nil )
    [self scrollViewDidEndZooming:scrollView withView:self.view atScale:1.0f];
}

- (void)viewDoubleTapped:(id)sender {
  if ( !tune.temporary ) {
    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
//      if ( barsHidden )
//        [self showStatusBar];
//      else
//        [self hideStatusBar];
    }
    else
      [self hideStatusBar];
  }
  [self zoomToFit:sender];
}

#pragma mark UIViewController delegate methods

- (void)viewWillAppear:(BOOL)animated
{
  [super viewWillAppear:animated];
  [self zoomToFit:self];
  if ( self.editing )
    [tuneView setNeedsDisplay]; // Force a refresh if we're editing
}

- (void)viewWillDisappear:(BOOL)animated
{
  [super viewWillDisappear:animated];
  [self hideSettings];
  if ( tunePlayer )
    [tunePlayer reset];
}

- (UIView *)viewForZoomingInScrollView:(UIScrollView *)scrollView
{
  return self.tune ? tuneView : nil;
}

- (void)scrollViewDidScroll:(UIScrollView *)aScrollView
{
  // Show or hide the tune sheet if the user scrolls to left or right
//  AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
//
//  if ( aScrollView.contentOffset.x < 0 )
//    [appDelegate.topLevelViewController showLeftView:YES];
//  else if ( aScrollView.contentSize.width-aScrollView.contentOffset.x < aScrollView.frame.size.width )
//    [appDelegate.topLevelViewController showLeftView:NO];
}

- (void)scrollViewDidEndZooming:(UIScrollView *)sv withView:(UIView *)view atScale:(CGFloat)scale
{
  float zoomScale = scrollView.zoomScale;
  float oldScale = tuneView.scale;
  scale *= oldScale;

  tuneView.scale = scale;

  CGSize contentSize = scrollView.contentSize;
  CGPoint newContentOffset = scrollView.contentOffset;
  CGSize oldContentViewSize = [tuneView frame].size;

  float xMult = contentSize.width/oldContentViewSize.width;
  float yMult = contentSize.height/oldContentViewSize.height;

  newContentOffset.x *= xMult;
  newContentOffset.y *= yMult;

  float currentMinZoom = scrollView.minimumZoomScale;
  float currentMaxZoom = scrollView.maximumZoomScale;

  float newMinZoom = currentMinZoom/zoomScale;
  float newMaxZoom = currentMaxZoom/zoomScale;

  [scrollView setMinimumZoomScale:1.0];
  [scrollView setMaximumZoomScale:1.0];
  [scrollView setZoomScale:1.0 animated:NO];

  CGSize scrollFrameSize = sv.bounds.size;
  CGPoint frameOrigin;
  frameOrigin.x = (contentSize.width < scrollFrameSize.width) ? ((scrollFrameSize.width - contentSize.width) / 2) : 0;
  frameOrigin.y = (contentSize.height < scrollFrameSize.height) ? ((scrollFrameSize.height - contentSize.height) / 2) : 0;

  [tuneView setTransform:CGAffineTransformIdentity];
  [tuneView setFrame:CGRectMake(frameOrigin.x, frameOrigin.y, contentSize.width, contentSize.height)];

  scrollView.contentSize = contentSize;
  [scrollView setContentOffset:newContentOffset animated:NO];
  [self scrollViewDidZoom:sv]; // Center if zoomed out

  [scrollView setMinimumZoomScale:newMinZoom];
  [scrollView setMaximumZoomScale:newMaxZoom];
 
  [tuneView setNeedsDisplay];
}

- (void)scrollViewDidZoom:(UIScrollView *)sv
{
  CGSize boundsSize = scrollView.bounds.size;
  CGRect frameToCenter = tuneView.frame;

  // center horizontally
  if (frameToCenter.size.width < boundsSize.width)
    frameToCenter.origin.x = (boundsSize.width - frameToCenter.size.width) / 2;
  else
    frameToCenter.origin.x = 0;

  // center vertically
  if (frameToCenter.size.height < boundsSize.height) 
    frameToCenter.origin.y = (boundsSize.height - frameToCenter.size.height) / 2;
  else
    frameToCenter.origin.y = 0;

  tuneView.frame = frameToCenter;
}

- (IBAction)startPlaying:(id)sender
{
  if ( !tunePlayer )
    tunePlayer = [TunePlayer tunePlayer];

  if ( [tunePlayer playing] ) {
    if ( ![tunePlayer paused] )
      return; // Safety exit if we got called twice
    toolbar.items = playItems;
    [tunePlayer playFromTime:[timeSlider value]];
  } else {
    toolbar.items = playItems;
    AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
    NSString *directory = NSTemporaryDirectory();
    NSString *midiPath = [directory stringByAppendingPathComponent:@"tune.mid"];

    extern int ABCTypesetTune( const char* abc, const char* formatText, const char* midiFile, int flags );
    ABCTypesetTune( [[tune dots] UTF8String], 0, [midiPath UTF8String], appDelegate.guitarChords ? 0x06 : 0x02/*FIXME-use proper flag kABCOutputGuitarChords/kABCOutputMIDI*/ );
    tunePlayer.filePath = midiPath;
    [tunePlayer setTime:0.0f];
    [tunePlayer playFromTime:[timeSlider value]];
  }
  timeSliderTimer = [NSTimer scheduledTimerWithTimeInterval:0.25 target:self selector:@selector(updateTimeSlider:) userInfo:nil repeats:YES];
}

- (void)playerFinished:(id)sender
{
  [timeSliderTimer invalidate];
  timeSliderTimer = nil;
  toolbar.items = stopItems;
}

- (IBAction)stopPlaying:(id)sender
{
//  [timeSliderTimer invalidate];
//  timeSliderTimer = nil;
  toolbar.items = stopItems;
  [timeSlider setValue:0.0f];
  [tunePlayer reset];
}

- (IBAction)pausePlaying:(id)sender
{
  toolbar.items = stopItems;
  [tunePlayer pause];
}

- (IBAction)rewindPlaying:(id)sender
{
  [timeSlider setValue:0.0f];
  [tunePlayer setTime:0.0f];
}

- (void)finishedPlaying:(id)sender
{
  toolbar.items = stopItems;
}

- (IBAction)timeSliderChanged:(id)sender
{
  [tunePlayer setTime:[timeSlider value]];
}

- (void)updateTimeSlider:(id)info
{
//printf("update %f\n", [tunePlayer time]);
  if ( !timeSlider.highlighted )
    [timeSlider setValue:[tunePlayer time]];
}

- (IBAction)showSettings:(id)sender
{
  if ( showingSettings ) {
    [self hideSettings];
  } else {
    if ( !tunePlayer )
      tunePlayer = [TunePlayer tunePlayer];

    if ( !settingsController )
      settingsController = [[[TuneSettingsController alloc] initWithPlayer:tunePlayer] retain];
    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
      settingsController.contentSizeForViewInPopover = settingsController.view.bounds.size;
      [self presentPopoverController:sender controller:settingsController];
    }
    else {
      CGSize s1 = self.view.bounds.size;
      CGSize s2 = settingsController.view.bounds.size;
      [settingsController.view setFrame:CGRectOffset(settingsController.view.bounds, (s1.width-s2.width)/2, (s1.height-s2.height-44)/2)];
      settingsController.view.alpha = 0.0;
      CALayer *layer = settingsController.view.layer;
      layer.borderColor = [[UIColor lightGrayColor] CGColor];
      layer.borderWidth = 1;
      layer.cornerRadius = 5;

      [self.view addSubview:settingsController.view];
      [UIView beginAnimations:nil context:NULL];
      [UIView setAnimationDuration:0.40];
      settingsController.view.alpha = 1.0;
      [UIView commitAnimations];
    }
    showingSettings = YES;
  }
}

- (void)hideSettings
{
  if (UI_USER_INTERFACE_IDIOM() != UIUserInterfaceIdiomPad) {
    if ( settingsController && showingSettings ) {
      [UIView beginAnimations:nil context:NULL];
      [UIView setAnimationDuration:0.40];
      settingsController.view.alpha = 0.0;
      [UIView commitAnimations];
    }
  }
  else
    [self dismissPopoverController];
  showingSettings = NO;
}

- (IBAction)printTune:(id)sender
{
  if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
    [self dismissPopoverController];
  Class uipc = NSClassFromString(@"UIPrintInteractionController");
  Class uipi = NSClassFromString(@"UIPrintInfo");
  UIPrintInteractionController *pic = [uipc sharedPrintController];
  NSData *pdf = [Tune pdfDataForTune:tune.dots];
  if ( pic && [uipc canPrintData:pdf] ) {
      pic.delegate = self;

      UIPrintInfo *printInfo = [uipi printInfo];
      printInfo.outputType = UIPrintInfoOutputGeneral;
      printInfo.jobName = tune.title;
      printInfo.duplex = UIPrintInfoDuplexLongEdge;
      pic.printInfo = printInfo;
      pic.showsPageRange = YES;
      pic.printingItem = pdf;

      void (^completionHandler)(UIPrintInteractionController *, BOOL, NSError *) =
         ^(UIPrintInteractionController *pic, BOOL completed, NSError *error) {
           if (!completed && error)
                NSLog(@"FAILED! due to error in domain %@ with error code %lu",
                error.domain, (long)error.code);
      };
      if ( UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad ) {
//        [pic presentFromRect:self.view.bounds inView:tuneView animated:YES completionHandler:completionHandler];
        [pic presentFromBarButtonItem:sender animated:YES
          completionHandler:completionHandler];
      } else {
      [pic presentAnimated:YES completionHandler:completionHandler];
    }
  }
}

- (UIPrintPaper *)printInteractionController:(UIPrintInteractionController *)printInteractionController choosePaper:(NSArray *)paperList
{
  Class uipp = NSClassFromString(@"UIPrintPaper");
  UIPrintPaper *paper;
  // Choose A4 paper if possible
  for (paper in paperList) {
    CGSize size = paper.paperSize;
    if ( fabs( size.width - 595.275574f ) < 1.0f && fabs( size.height - 841.889771f ) < 1.0f )
      return paper;
  }
  return [uipp bestPaperForPageSize:CGSizeMake(PaperWidth, PaperHeight) withPapersFromArray:paperList];
}

- (IBAction)emailTune:(id)sender
{
	Class mailClass = NSClassFromString(@"MFMailComposeViewController");
	if (mailClass != nil)
	{
		if ([mailClass canSendMail])
			[self displayComposerSheet];
		else
			[self launchMailAppOnDevice];
	}
	else
		[self launchMailAppOnDevice];
}

-(void)transposeTune
{
  TransposeViewController *controller = [[TransposeViewController alloc] initWithNibName:@"Transpose" bundle:nil];
  [controller setEditController:editController];
  controller.view.frame = editController.view.bounds;
  [editController addChildViewController:controller];
  [editController.view addSubview:controller.view];
  controller.view.alpha = 0.0;
  [UIView beginAnimations:nil context:NULL];
  [UIView setAnimationDuration:0.30];
  controller.view.alpha = 1.0;
  [UIView commitAnimations];

  [controller release];
}

- (IBAction)actionPressed:(id)sender
{
  if ( activeActionSheet == 0 ) {
    _actionSender = sender;
    if ( tune ) {
      _actionActionSheet = [[UIActionSheet alloc]
                      initWithTitle:nil
                           delegate:self
                  cancelButtonTitle:nil
             destructiveButtonTitle:nil
                  otherButtonTitles:nil];

      _editTuneIndex = _emailTuneIndex = _printTuneIndex = _transposeTuneIndex = _addToTunebookIndex = _shareIndex = -1000;
      if ( !editingABC && !tune.temporary ) {
        _editTuneIndex = [_actionActionSheet addButtonWithTitle:@"Edit Tune"];
      }
      if ( editingABC )
        _transposeTuneIndex = [_actionActionSheet addButtonWithTitle:@"Transpose Tune"];
      if ( !editingABC ) {
        // On older iOS versions, add Email separately as it doesn't appear in UIDocumentInteraction
        _emailTuneIndex = [_actionActionSheet addButtonWithTitle:@"Email Tune"];
        _exportABCIndex = [_actionActionSheet addButtonWithTitle:@"Share ABC..."];
        _exportPDFIndex = [_actionActionSheet addButtonWithTitle:@"Share PDF..."];
        _exportMIDIIndex = [_actionActionSheet addButtonWithTitle:@"Share MIDI..."];
//        _addToTunebookIndex = [_actionActionSheet addButtonWithTitle:@"Add to Tunebook..."];
      }
      if ([UIPrintInteractionController isPrintingAvailable])
        _printTuneIndex = [_actionActionSheet addButtonWithTitle:@"Print"];
      
      [_actionActionSheet addButtonWithTitle:@"Cancel"];

      activeActionSheet = _actionActionSheet;
      _actionBarButtonItem = sender;
      [_actionActionSheet showFromBarButtonItem:sender animated:YES];
    }
  }
}

- (IBAction)editPressed:(id)sender
{
  [self dismissPopoverController];
  [self setEditing:YES animated:YES];
}

#pragma mark -
#pragma mark Compose Mail

-(void)displayComposerSheet 
{
	MFMailComposeViewController *picker = [[MFMailComposeViewController alloc] init];
	picker.mailComposeDelegate = self;
	
	[picker setSubject:[@"Tune: " stringByAppendingString:tune.title]];

	NSString *emailBody = tune.dots;
	[picker setMessageBody:emailBody isHTML:NO];
  NSData *attachment = [Tune pdfDataForTune:tune.dots];
	[picker addAttachmentData:attachment mimeType:@"application/pdf" fileName:@"tune.pdf"];
	
	[self presentViewController:picker animated:YES completion:nil];
  [picker release];
}

- (void)mailComposeController:(MFMailComposeViewController*)controller didFinishWithResult:(MFMailComposeResult)result error:(NSError*)error 
{	
	switch (result)
	{
  case MFMailComposeResultCancelled:
    break;
  case MFMailComposeResultSaved:
    break;
  case MFMailComposeResultSent:
    {
    UIAlertController* alertController = [UIAlertController alertControllerWithTitle:@"Success" message:@"Your tune has been sent." preferredStyle:UIAlertControllerStyleAlert];
    [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil]];
    [self presentViewController:alertController animated:YES completion:nil];
    }
    break;
  case MFMailComposeResultFailed:
  default:
    {
    UIAlertController* alertController = [UIAlertController alertControllerWithTitle:@"Unable to send email" message:@"Please make sure that email is configured correctly." preferredStyle:UIAlertControllerStyleAlert];
    [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil]];
    [self presentViewController:alertController animated:YES completion:nil];
    }
    break;
	}
	[self dismissViewControllerAnimated:YES completion:nil];
}


#pragma mark -

-(void)launchMailAppOnDevice
{
	NSString *recipients = @"mailto:?&subject=Tune: ";
	NSString *body = [@"&body=" stringByAppendingString:tune.dots];
	
	NSString *email = [NSString stringWithFormat:@"%@%@%@", recipients, tune.title, body];
	email = [email stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding];
	
	[[UIApplication sharedApplication] openURL:[NSURL URLWithString:email]];
}

- (void)actionSheet:(UIActionSheet*)actionSheet willDismissWithButtonIndex:(NSInteger)buttonIndex
{
  if( activeActionSheet == actionSheet ) {
    activeActionSheet = nil;
    activeBarButtonItem = nil;
  }
}

- (BOOL)popoverControllerShouldDismissPopover:(UIPopoverController*)popoverController
{
  if( activePopoverController == popoverController ) {
    activePopoverController = nil;
    activeBarButtonItem = nil;
    showingSettings = NO;
  }
  return YES;
}

- (void)willRotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation duration:(NSTimeInterval)duration
{
  [super willRotateToInterfaceOrientation:toInterfaceOrientation duration:duration];
  if( activePopoverController )
  {
    if( activeBarButtonItem )
    {
      rotateViewController = [activePopoverController contentViewController];
      [rotateViewController retain];
      rotatePassthroughViews = [activePopoverController passthroughViews];
      [rotatePassthroughViews retain];
      rotateBarButtonItem = activeBarButtonItem;
    }
    [activePopoverController dismissPopoverAnimated:YES];
    activePopoverController = nil;
  }
  else if( activeActionSheet )
  {
    if( activeBarButtonItem )
    {
      rotateActionSheet = activeActionSheet;
      [rotateActionSheet retain];
      rotateBarButtonItem = activeBarButtonItem;
    }
    [activeActionSheet dismissWithClickedButtonIndex:-1 animated:YES];
    activeActionSheet = nil;
  }
  activeBarButtonItem = nil;
}

- (void)didRotateFromInterfaceOrientation:(UIInterfaceOrientation)fromInterfaceOrientation
{
  [super didRotateFromInterfaceOrientation:(UIInterfaceOrientation)fromInterfaceOrientation];

  if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
    if( rotateBarButtonItem )
    {
      activeBarButtonItem = rotateBarButtonItem;
      rotateBarButtonItem = nil;
    }
    if( rotateViewController )
    {
      if( activeBarButtonItem )
      {
        activePopoverController = [[UIPopoverController alloc] initWithContentViewController:rotateViewController];
        if( rotatePassthroughViews )
        {
          [activePopoverController setPassthroughViews:rotatePassthroughViews];
        }
        [activePopoverController presentPopoverFromBarButtonItem:activeBarButtonItem
            permittedArrowDirections:UIPopoverArrowDirectionAny animated:YES];
      }
      [rotateViewController release];
      rotateViewController = nil;
      [rotatePassthroughViews release];
      rotatePassthroughViews = nil;
    }
    else if( rotateActionSheet )
    {
      if( activeBarButtonItem )
      {
        activeActionSheet = rotateActionSheet;
        [rotateActionSheet showFromBarButtonItem:activeBarButtonItem animated:YES];
      }
      [rotateActionSheet release];
      rotateActionSheet = nil;
    }
  }

  [self zoomToFit:self];
}

- (void)presentPopoverController:(UIBarButtonItem *)item controller:(UIViewController*)controller
{
  [self dismissPopoverController];
  activePopoverController = [[UIPopoverController alloc] initWithContentViewController:controller];
  [activePopoverController setDelegate:self];
  activeBarButtonItem = item;
  [activePopoverController setPassthroughViews:[NSArray arrayWithObjects:toolbar, nil]];
  [activePopoverController presentPopoverFromBarButtonItem:item
      permittedArrowDirections:UIPopoverArrowDirectionAny animated:YES];
}

- (void)dismissPopoverController
{
  if ( activePopoverController ) {
    [activePopoverController dismissPopoverAnimated:YES];
    activePopoverController = nil;
  }
}

- (IBAction)prevTune:(id)sender
{
  if ( !tune.temporary && !editingABC ) {
    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
    [[MainViewController activeController] prevTune:self];
  }
  else {
      AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
      NSArray *tunes = playlist ? [playlist loadTunes] : appDelegate.tunes;
      NSInteger index = [tunes indexOfObject:tune];
      if ( index != NSNotFound && index > 0 ) {
        Tune *t = [tunes objectAtIndex:index-1];
        [self showTune:t fromPlayList:playlist direction:-1];
        [self showMessage:[NSString stringWithFormat:@"Tune %d of %d", (int)index, (int)[tunes count]]];
      }
    }
  }
}

- (IBAction)nextTune:(id)sender
{
  if ( !tune.temporary && !editingABC ) {
    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
      [[MainViewController activeController] nextTune:self];
    }
    else {
      AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
      NSArray *tunes = playlist ? [playlist loadTunes] : appDelegate.tunes;
      NSInteger index = [tunes indexOfObject:tune];
      if ( index != NSNotFound && index < [tunes count]-1 ) {
        Tune *t = [tunes objectAtIndex:index+1];
        [self showTune:t fromPlayList:playlist direction:1];
        [self showMessage:[NSString stringWithFormat:@"Tune %d of %d", (int)index+2, (int)[tunes count]]];
      }
    }
  }
}

- (void)movePage:(int)delta
{
  if ( paginationOn ) {
    int page = (int)((scrollView.contentOffset.y+1)/(PaperHeight * tuneView.scale));
    int newPage = page+delta;
    newPage = MAX( 0, MIN( numPages-1, newPage ) );
    if ( page != newPage ) {
      CGPoint offset = scrollView.contentOffset;
      offset.y = newPage * PaperHeight * tuneView.scale;
      [scrollView setContentOffset:offset animated:YES];

      [self showMessage:[NSString stringWithFormat:@"Page %d of %d", newPage+1, numPages]];
    }
  }
  else {
    float pageHeight = scrollView.bounds.size.height;
    int page = (int)((scrollView.contentOffset.y+1)/pageHeight);
    int newPage = page+delta;
    newPage = MAX( 0, MIN( numPages-1, newPage ) );
    if ( page != newPage ) {
      CGPoint offset = scrollView.contentOffset;
      offset.y = newPage * pageHeight;
      [scrollView setContentOffset:offset animated:YES];

      [self showMessage:[NSString stringWithFormat:@"Page %d of %d", newPage+1, numPages]];
    }
  }
}

- (IBAction)prevPage:(id)sender
{
  [self movePage:-1];
}

- (IBAction)nextPage:(id)sender
{
  [self movePage:1];
}

-(void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event
{
  [self becomeFirstResponder];
}

-(void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event
{
}

-(void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event
{
  UITouch * t = [touches anyObject];
  if ( [t tapCount] == 1 ) {
    CGPoint location = [t locationInView:mainView];
    CGRect r = mainView.bounds;

    if ( location.y < r.size.height/5.0f )
      [self prevPage:self];
    else if ( location.y > r.size.height*4.0f/5.0f )
      [self nextPage:self];
    else if ( location.x < r.size.width/5.0f )
      [self prevTune:self];
    else if ( location.x > r.size.width*4.0f/5.0f )
      [self nextTune:self];
    else
      [self viewTapped:tuneView];
  } else if( [t tapCount] == 2 )
    [self viewDoubleTapped:tuneView];
}

-(void)touchesCancelled:(NSSet *)touches withEvent:(UIEvent *)event
{
}

- (void)showMessage:(NSString*)message
{
  [CATransaction begin];
  [CATransaction setAnimationDuration:0];
  messageLayer.position = CGPointMake( self.view.bounds.size.width/2, self.view.bounds.size.height/2 );
  [messageLayer setValue:message forKey:@"text"];
  [messageLayer setNeedsDisplay];
  [CATransaction commit];

  float alpha = 0.9f;
  CABasicAnimation *alphaAnimation = [CABasicAnimation animationWithKeyPath: @"opacity"];
  alphaAnimation.beginTime = 0.0;
  alphaAnimation.fromValue = [NSNumber numberWithFloat: 0.0f];
  alphaAnimation.toValue = [NSNumber numberWithFloat: alpha];
  alphaAnimation.duration = 0.2;
  CABasicAnimation *alphaAnimation2 = [CABasicAnimation animationWithKeyPath: @"opacity"];
  alphaAnimation2.beginTime = 0.2;
  alphaAnimation2.duration = 0.4;
  alphaAnimation2.fromValue = [NSNumber numberWithFloat: alpha];
  alphaAnimation2.toValue = [NSNumber numberWithFloat: alpha];
  CABasicAnimation *alphaAnimation3 = [CABasicAnimation animationWithKeyPath: @"opacity"];
  alphaAnimation3.beginTime = 0.6;
  alphaAnimation3.duration = 0.2;
  alphaAnimation3.fromValue = [NSNumber numberWithFloat: alpha];
  alphaAnimation3.toValue = [NSNumber numberWithFloat: 0.0f];

  CAAnimationGroup *fadeAnimation = [CAAnimationGroup animation];
  fadeAnimation.delegate = self;
  fadeAnimation.animations = [NSArray arrayWithObjects:alphaAnimation, alphaAnimation2, alphaAnimation3, nil];
  fadeAnimation.duration = 1.0;
  [CATransaction begin];
  [messageLayer addAnimation:fadeAnimation forKey:@"animateOpacity"];
  [CATransaction commit];
}

- (NSArray *)keyCommands
{
  UIKeyCommand *upArrow = [UIKeyCommand keyCommandWithInput: UIKeyInputUpArrow modifierFlags: 0 action: @selector(upArrow:)];
  UIKeyCommand *downArrow = [UIKeyCommand keyCommandWithInput: UIKeyInputDownArrow modifierFlags: 0 action: @selector(downArrow:)];
  UIKeyCommand *leftArrow = [UIKeyCommand keyCommandWithInput: UIKeyInputLeftArrow modifierFlags: 0 action: @selector(leftArrow:)];
  UIKeyCommand *rightArrow = [UIKeyCommand keyCommandWithInput: UIKeyInputRightArrow modifierFlags: 0 action: @selector(rightArrow:)];
  return [[NSArray alloc] initWithObjects: upArrow, downArrow, leftArrow, rightArrow, nil];
}

- (void) upArrow: (UIKeyCommand *) keyCommand
{
  [self prevTune:self];
}

- (void) downArrow: (UIKeyCommand *) keyCommand
{
  [self nextTune:self];
}

- (void) leftArrow: (UIKeyCommand *) keyCommand
{
  [self prevTune:self];
}

- (void) rightArrow: (UIKeyCommand *) keyCommand
{
  [self nextTune:self];
}

-(void)addToTunebook:(id)sender
{
}

-(NSString*)safeFileName:(NSString*)fileName extension:(NSString*)extension
{
  NSCharacterSet* illegalFileNameCharacters = [NSCharacterSet characterSetWithCharactersInString:@":/\\?%*|\"<>"];
  return [[[fileName componentsSeparatedByCharactersInSet:illegalFileNameCharacters] componentsJoinedByString:@""] stringByAppendingString:extension];
}

static NSURL* sDocumentURL;

-(void)openABCDocumentIn:(id)sender
{
  NSString *filePath = [NSTemporaryDirectory() stringByAppendingPathComponent:[self safeFileName:tune.title extension:@".abc"]];
  NSError *error = NULL;
  if ( [[tune fullDots] writeToFile:filePath atomically:false encoding:NSUTF8StringEncoding error:&error] ) {
    sDocumentURL = [[NSURL fileURLWithPath:filePath] retain];
    UIDocumentInteractionController* documentController =
        [UIDocumentInteractionController interactionControllerWithURL:sDocumentURL];
    documentController.delegate = self;
    documentController.UTI = @"public.utf8-plain-text";
    [documentController retain];
    [documentController presentOpenInMenuFromBarButtonItem:_actionBarButtonItem animated:YES];
  }
}

-(void)openPDFDocumentIn:(id)sender
{
  NSData* data = [tune pdfData];
  NSString *filePath = [NSTemporaryDirectory() stringByAppendingPathComponent:[self safeFileName:tune.title extension:@".pdf"]];
  if ( [data writeToFile:filePath atomically:YES] ) {
    sDocumentURL = [[NSURL fileURLWithPath:filePath] retain];
    UIDocumentInteractionController* documentController =
        [UIDocumentInteractionController interactionControllerWithURL:sDocumentURL];
    documentController.delegate = self;
    documentController.UTI = @"com.adobe.pdf";
    [documentController retain];
    [documentController presentOpenInMenuFromBarButtonItem:_actionBarButtonItem animated:YES];
  }
}

-(void)openMIDIDocumentIn:(id)sender
{
  NSData* data = [tune midiDataWithGuitarChords:YES];
  NSString *filePath = [NSTemporaryDirectory() stringByAppendingPathComponent:[self safeFileName:tune.title extension:@".mid"]];
  if ( [data writeToFile:filePath atomically:YES] ) {
    sDocumentURL = [[NSURL fileURLWithPath:filePath] retain];
    UIDocumentInteractionController* documentController =
        [UIDocumentInteractionController interactionControllerWithURL:sDocumentURL];
    documentController.delegate = self;
    documentController.UTI = @"public.midi-audio";
    [documentController retain];
    [documentController presentOpenInMenuFromBarButtonItem:_actionBarButtonItem animated:YES];
  }
}

-(void)documentInteractionController:(UIDocumentInteractionController *)controller 
         willBeginSendingToApplication:(NSString *)application
{
}
 
-(void)documentInteractionController:(UIDocumentInteractionController *)controller 
             didEndSendingToApplication:(NSString *)application
{ 
}
 
-(void)documentInteractionControllerDidDismissOptionsMenu:
(UIDocumentInteractionController *)controller
{
  if ( sDocumentURL != nil ) {
    NSError* error = nil;
    [[NSFileManager defaultManager] removeItemAtURL:sDocumentURL error:&error];
    [sDocumentURL release];
    sDocumentURL = nil;
  }
}
@end

@implementation TextLayer

@synthesize alignment;

- (id) init
{
  self = [super init];
  if ( self ) {
    alignment = NSTextAlignmentCenter;
  }
  return self;
}

- (void)drawInContext:(CGContextRef)theContext
{
  CGContextSaveGState( theContext );
  NSString *text = [self valueForKey:@"text"];
  UIFont *font = [self valueForKey:@"font"];
  CGContextSetRGBFillColor( theContext, 0.8f, 0.8f, 0.8f, 1 );
  UIGraphicsPushContext( theContext );
  CGSize size = [text sizeWithFont:font constrainedToSize:CGSizeMake(0, 0) lineBreakMode:UILineBreakModeWordWrap];
  CGRect r = self.bounds;
  r = CGRectInset( r, 5, 0 );
  r.origin.y += (r.size.height-size.height)/2;
  [text drawInRect:r withFont:font lineBreakMode:UILineBreakModeWordWrap alignment:alignment];
  UIGraphicsPopContext();
  CGContextRestoreGState( theContext );
}

@end
