#import "ImportProgressController.h"
#import "TuneDatabase.h"
#import "Tune.h"
#import "Playlist.h"

@implementation ImportProgressController

- (void)import:(NSURL *)url intoTunebook:(Playlist*)tunebook withDatabase:(TuneDatabase*)database window:(NSWindow*)window
{
  [progressView setDoubleValue:0.0];
  _url = url;
  [_url retain];
  _tunebook = tunebook;
  [_tunebook retain];
  _database = database;
  [_database retain];
  _window = window;
  [_window retain];
  parsedTunes = [[NSMutableArray alloc] init];
  [parsedTunes retain];
  fragment = nil;
  lastWasCR = NO;
  [titleLabel setStringValue: [NSString stringWithFormat:@"Importing %@", [[_url path] lastPathComponent]]];
  NSWindow* sheet = [self.view window];
  [NSApp beginSheet:sheet
         modalForWindow:window
         modalDelegate: self
         didEndSelector: @selector(didEndSheet:returnCode:contextInfo:)
         contextInfo: NULL];
  [NSThread detachNewThreadSelector:@selector(threadRun:) toTarget:self withObject:nil];
}

-(void)dealloc
{
  [_tunebook release];
  [_url release];
  [_database release];
  [_window release];
  [parsedTunes release];
  [fragment release];
	[super dealloc];
}

- (IBAction) onCancel:(id)sender
{
  cancelled = true;
  finished = true;
  if ( urlConnection ) {
    [urlConnection cancel];
    urlConnection = NULL;
  }
  [self connectionDidFinishLoading:NULL];
}

- (void)threadRun:(id)sender
{
  NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];  

  cancelled = false;
  finished = false;
  statusCode = 0;

  NSURLRequest *theRequest = [NSURLRequest requestWithURL:_url cachePolicy:NSURLRequestReloadIgnoringLocalCacheData timeoutInterval:60];
  
  parser = [[TuneParser alloc] init];
  parser.delegate = self;

  numTunes = 0;
  bytesRead = 0;
  bytesProcessed = 0;

  urlConnection = [NSURLConnection connectionWithRequest:theRequest delegate:self];
  if ( urlConnection == nil ) {
    NSBeginAlertSheet(@"Unable to initiate request", @"OK", nil, nil, _window, self, nil, nil, nil, @"NSURLConnection initialization method failed" );
    return;
  }

  BOOL done = NO;
  do {
    SInt32 result = CFRunLoopRunInMode( kCFRunLoopDefaultMode, 1, YES );
    if ( (result == kCFRunLoopRunStopped) || (result == kCFRunLoopRunFinished) ) {
      done = YES;
    }
    if ( cancelled || finished )
      done = YES;
  } while ( !done );

#if TARGET_OS_IPHONE
  // If it was a local file, delete it when finished
  if ( [_url isFileURL] ) {
    NSFileManager *fileManager = [NSFileManager defaultManager];
    [fileManager removeItemAtPath:[_url path] error:nil];
  }
#endif

  [self release];
  [pool release];
}

#pragma mark UIViewController delegate methods

- (void)viewWillAppear:(BOOL)animated
{
}

#pragma mark NSURLConnection delegate methods

- (void) connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response
{
  bytesToRead = (long)[response expectedContentLength];
  if ( [response isKindOfClass:[NSHTTPURLResponse class]] ) {
    statusCode = [(NSHTTPURLResponse *)response statusCode];
    if ( statusCode >= 400 )
      cancelled = true;
  }
}

- (void) updateProgress:(id)sender
{
  [progressView setDoubleValue: (double)bytesProcessed / bytesToRead];
  [statusLabel setStringValue: [NSString stringWithFormat:@"Read %.1fKb, found %d tunes", (float)bytesProcessed/1024, numTunes]];
}

- (void) connection:(NSURLConnection *)connection didReceiveData:(NSData *)data
{
  const char *b = (const char *)data.bytes;
  int length = data.length;
  int start = 0;

  if ( cancelled )
    return;

  bytesRead += length;

  int n = numTunes;
  int i = 0;
  for ( ; i < length && !cancelled; i++ ) {
    bytesProcessed++;
    if ( lastWasCR && b[i] == '\n' ) {
      lastWasCR = NO;
      start = i+1;
    } else if ( b[i] == '\r' || b[i] == '\n' ) {
      NSString *line = [[NSString alloc] initWithBytes:b+start length:i-start encoding:NSISOLatin1StringEncoding];
      if ( line == nil )
        line = [[NSString alloc] initWithBytes:b+start length:i-start encoding:NSUTF8StringEncoding];
      if ( fragment ) {
        [fragment appendString:line];
        [parser parseLine:fragment];
        [fragment release];
        fragment = nil;
      } else
        [parser parseLine:line];
      if ( numTunes >= n+10 ) {
        [self performSelectorOnMainThread:@selector(updateProgress:) withObject:nil waitUntilDone:NO];
        n = numTunes;
      }

      [line release];
      if ( b[i] == '\r' )
        lastWasCR = YES;
      start = i+1;
    }
  }
  if ( start != i ) {
    if ( fragment == nil )
      fragment = [[NSMutableString alloc] init];
    
    NSString *s = [[NSString alloc] initWithBytes:b+start length:i-start encoding:NSISOLatin1StringEncoding];
    if ( s == nil )
      s = [[NSString alloc] initWithBytes:b+start length:i-start encoding:NSUTF8StringEncoding];
    [fragment appendString:s];
    [s release];
  }
  [self performSelectorOnMainThread:@selector(updateProgress:) withObject:nil waitUntilDone:NO];
}

- (void) connection:(NSURLConnection *)connection didFailWithError:(NSError *)error
{
  [NSApp endSheet:[self.view window] returnCode: NSModalResponseOK];
  NSBeginAlertSheet(@"Import Failed", @"OK", nil, nil, _window, self, nil, nil, nil, @"%@", [error localizedDescription] );
	[connection release];
}

- (NSCachedURLResponse *) connection:(NSURLConnection *)connection willCacheResponse:(NSCachedURLResponse *)cachedResponse
{
  return nil;
}

- (void) connectionDidFinishLoading:(NSURLConnection *)connection
{
  [parser parseLine:@""]; // Force the last tune to end
  [self performSelectorOnMainThread:@selector(finishedImport:) withObject:nil waitUntilDone:NO];
}

- (void) finishedImport:(id)sender
{
  [NSApp endSheet:[self.view window] returnCode: NSModalResponseOK];
  finished = true;
  urlConnection = NULL;

  if ( !cancelled ) {
    NSString* filename = [[_url path] lastPathComponent];
    if ( statusCode >= 400 ) {
      NSBeginAlertSheet(@"Import Failed", @"OK", nil, nil, _window, self, nil, nil, nil,
        @"An error (%ld) occurred when importing %@: %@", (long)statusCode, filename, [NSHTTPURLResponse localizedStringForStatusCode:statusCode] );
    } else if ( numTunes ) {
      [_database startTransaction];
      if ( ![_tunebook inDatabase] )
        [_database addPlaylist:_tunebook];
      for ( Tune* tune in parsedTunes ) {
        [_database addTune:tune];
        if ( _tunebook )
          [_tunebook addTune:tune];
      }
      [_database commitTransaction];
      [_database updateTunebookSortOrder];
      [parsedTunes release];
      parsedTunes = nil;
      [[NSNotificationCenter defaultCenter] postNotificationName:@"playlistsChanged" object:self]; 
    } else {
      NSBeginAlertSheet(@"Import Failed", @"OK", nil, nil, _window, self, nil, nil, nil,
        @"The file %@ was imported, but appeared to contain no tunes.", filename );
    }
  }

  [[NSNotificationCenter defaultCenter] postNotificationName:@"databaseChanged" object:self]; 
}

#pragma mark TuneParser delegate methods

-(void) parsedTune:(TuneParser *)parser tune:(Tune *)tune
{
  [parsedTunes addObject:tune];
  numTunes++;
}

- (void)didEndSheet:(NSWindow *)sheet returnCode:(NSInteger)returnCode contextInfo:(void *)contextInfo;
{
  [sheet orderOut:self];
}

@end
