#import "PluginManager.h"
#import "TuneListViewController.h"
#import <Python/Python.h>

@interface NSObject (PythonPluginInterface)

- (BOOL) loadModuleAtPath:(NSString*)path functionName:(NSString*)funcName arguments:(NSArray*)args;

@end

@implementation PluginManager

+ (PluginManager*) sharedManager {
	static dispatch_once_t pred;
	static PluginManager *sharedManager;
	dispatch_once(&pred, ^{
		sharedManager = [[PluginManager alloc] init];
	});
	return sharedManager;
}

- (BOOL) setupPython
{
	if (Py_IsInitialized())
		return YES;
	
	Py_SetProgramName("/usr/bin/python");
	Py_Initialize();
	PyUnicode_SetDefaultEncoding("utf-8");
	NSString *scriptPath = [[NSBundle mainBundle] pathForResource:@"PluginRunner" ofType:@"py"];
	FILE *mainFile = fopen([scriptPath UTF8String], "r");
	BOOL result = (PyRun_SimpleFile(mainFile, (char *)[[scriptPath lastPathComponent] UTF8String]) == 0);
  if ( PyErr_Occurred() )
    PyErr_Print();
  return result;
}

- (BOOL) loadScriptAtPath:(NSString*)scriptPath runFunction:(NSString*)functionName withArguments:(NSArray*)arguments
{
  if (![self setupPython]) {
    NSLog(@"Error: Can't start up Python plugins.");
    return false;
  }
	Class executor = NSClassFromString(@"PluginRunner");
	return [executor loadModuleAtPath:scriptPath functionName:functionName arguments:arguments];
}

@end


@implementation PluginContext
- (id)initWithController:(TuneListViewController*)aController
{
  self = [super init];
  if ( self ) {
    controller = [aController retain];
  }
  return self;
}

- (void)dealloc
{
  [controller release];
  [super dealloc];
}

- (NSArray*)tunes
{
  return [controller.database tunes];
}

- (NSTextView*)abcTextView
{
  return controller.abcTextView;
}

- (NSString*)selectedText
{
  return [[controller.abcTextView string] substringWithRange:[controller.abcTextView selectedRange]];
}

- (void)replaceSelectedText:(NSString*)text
{
  [controller.abcTextView replaceCharactersInRange:[controller.abcTextView selectedRange] withString:text];
}

- (Tune*)selectedTune
{
  return [controller selectedTune];
}

- (NSArray*)selectedTunes
{
  return [controller selectedTunes];
}

- (Tune*)displayedTunes
{
  return [controller tuneWithSelectedTunes];
}
@end
