//
//  SearchWindowController.m
//  Tunebook-Mac
//
//  Created by Jerry on 26/07/2011.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import "SearchWindowController.h"

@implementation SearchWindowController

@synthesize predicate;
@synthesize delegate;

- (void)awakeFromNib
{
  NSArray *keyPaths = [NSArray arrayWithObjects:
    [NSExpression expressionForKeyPath:@"title"],
    [NSExpression expressionForKeyPath:@"key"],
    [NSExpression expressionForKeyPath:@"meter"],
//    [NSExpression expressionForKeyPath:@"dots"],
     nil];

  NSArray *operators = [NSArray arrayWithObjects:
    [NSNumber numberWithInteger:NSEqualToPredicateOperatorType],
    [NSNumber numberWithInteger:NSNotEqualToPredicateOperatorType],
    [NSNumber numberWithInteger:NSBeginsWithPredicateOperatorType],
    [NSNumber numberWithInteger:NSEndsWithPredicateOperatorType],
    [NSNumber numberWithInteger:NSContainsPredicateOperatorType],
    nil];

  NSPredicateEditorRowTemplate *template = [[NSPredicateEditorRowTemplate alloc] initWithLeftExpressions:keyPaths
    rightExpressionAttributeType:NSStringAttributeType
    modifier:NSDirectPredicateModifier 
    operators:operators 
    options:(NSCaseInsensitivePredicateOption | NSDiacriticInsensitivePredicateOption)];


  NSArray *dotsKeyPaths = [NSArray arrayWithObjects:
    [NSExpression expressionForKeyPath:@"dots"],
     nil];

  NSArray *dotsOperators = [NSArray arrayWithObjects:
//    [NSNumber numberWithInteger:NSEqualToPredicateOperatorType],
    [NSNumber numberWithInteger:NSContainsPredicateOperatorType],
    nil];

  NSPredicateEditorRowTemplate *dotsTemplate = [[NSPredicateEditorRowTemplate alloc] initWithLeftExpressions:dotsKeyPaths
    rightExpressionAttributeType:NSStringAttributeType
    modifier:NSDirectPredicateModifier 
    operators:dotsOperators 
    options:(NSCaseInsensitivePredicateOption | NSDiacriticInsensitivePredicateOption)];

  NSArray *compoundTypes = [NSArray arrayWithObjects:[NSNumber numberWithInteger:NSNotPredicateType],
                                                     [NSNumber numberWithInteger:NSAndPredicateType],
                                                     [NSNumber numberWithInteger:NSOrPredicateType],
                                                     nil];
  NSPredicateEditorRowTemplate *compound = [[NSPredicateEditorRowTemplate alloc] initWithCompoundTypes:compoundTypes];

  NSArray *rowTemplates = [NSArray arrayWithObjects:template, dotsTemplate, compound, nil];
  [template release];
  [compound release];

  [predicateEditor setRowTemplates:rowTemplates];

  [predicateEditor setObjectValue:predicate];
}

- (void)dealloc
{
   [predicate release];
   [delegate release];
   [super dealloc];
}

- (void)showSearch:(NSPredicate*)aPredicate window:(NSWindow*)aWindow
{
  NSPredicate* old = predicate;
  predicate = [aPredicate retain];
  [old release];

  NSWindow* sheet = [self window];
  [NSApp beginSheet:sheet
         modalForWindow:aWindow
         modalDelegate:self
         didEndSelector: @selector(didEndSheet:returnCode:contextInfo:)
         contextInfo: NULL];
}

- (void)searchPressed:(id)sender
{
  NSWindow* sheet = [self window];
  [NSApp endSheet:sheet];
  [delegate searchWithPredicate:[predicateEditor objectValue]];
}

- (void)cancelPressed:(id)sender
{
  NSWindow* sheet = [self window];
  [NSApp endSheet:sheet];
}

- (void)didEndSheet:(NSWindow *)sheet returnCode:(NSInteger)returnCode contextInfo:(void *)contextInfo
{
  [sheet orderOut:self];
}

@end
