//
//  TuneListViewController.m
//  Tunebook-Mac
//
//  Created by Jerry on 22/07/2011.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import "TuneListViewController.h"
#import "TuneDatabase.h"
#import "TuneView.h"
#import "TuneParser.h"
#import "Playlist.h"
#import "CenteringClipView.h"
#import "PlaySettingsWindowController.h"
#import "ImportProgressController.h"
#import "TunePlayer.h"
#import "ImageAndTextCell.h"
#import "SearchWindowController.h"
#import "SimpleSearchWindowController.h"
#import "PluginManager.h"
#import "PianoKeyboardViewController.h"
#include <Quartz/Quartz.h>

static int sErrorLine;
static int sErrorCol;
static NSString* sErrorMsg;

void abcError( const char* msg, int line, int col )
{
  [sErrorMsg release];
  sErrorMsg = [NSString stringWithFormat:@"Error: Line %d, column %d - %s", line, col, msg];
  [sErrorMsg retain];
  sErrorLine = line;
  sErrorCol = col;
}

@interface TuneItem : TunePlayerItem
{
  Tune* tune;
}

- (id)initWithTune:(Tune*)tune;
- (NSData*)data;
@end


#define TunebookPrivateTableViewDataType @"TunebookPrivateTableViewDataType"
#define TunebookPrivateTunesDataType @"TunebookPrivateTunesDataType"

@implementation TuneListViewController

@synthesize database;
@synthesize abcTextView;
@synthesize currentTune;

- (void)awakeFromNib
{
  [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(databaseChanged:) name:@"databaseChanged" object:nil]; 
  [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(databaseChanged:) name:@"playlistsChanged" object:nil]; 
  [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(finishedPlaying:) name:kTunePlayerFinishedNotification object:nil]; 
  database = [[TuneDatabase alloc] init];
  [database initializeDatabase];
  tuneListView.delegate = self;
  searchField.delegate = self;
  abcTextView.delegate = self;

  // Replace NSClipView of scrollView with a CenteringClipView
  id docView = [tuneScrollView documentView];
  NSClipView* clipView = [[CenteringClipView alloc] initWithFrame:[docView frame]];
  [tuneScrollView setContentView:clipView];
  [tuneScrollView setDocumentView:docView];

  [tunebookView registerForDraggedTypes:[NSArray arrayWithObjects: NSFilenamesPboardType, NSURLPboardType, TunebookPrivateTableViewDataType, nil]];
  [tuneListView registerForDraggedTypes:[NSArray arrayWithObjects: NSFilenamesPboardType, NSURLPboardType, TunebookPrivateTableViewDataType, nil]];

  [self setNextResponder:[self.view window]];
  [tunebookView setNextResponder:self];
  [tuneListView setNextResponder:self];
  [tuneView setNextResponder:self];
  [abcTextView setNextResponder:self];

  NSFont *font;
  NSTableColumn *col;
  NSEnumerator *e;
  font = [NSFont systemFontOfSize:10.0];
  e = [[tunebookView tableColumns] objectEnumerator];
  while ( col = [e nextObject] )
    [[col dataCell] setFont:font];

  font = [NSFont systemFontOfSize:10.0];
  e = [[tuneListView tableColumns] objectEnumerator];
  while ( col = [e nextObject] )
    [[col dataCell] setFont:font];

	col = [[tunebookView tableColumns] objectAtIndex: 0];
	ImageAndTextCell* cell = [[ImageAndTextCell alloc] init];
  [cell setEditable:YES];
	[col setDataCell: cell];

  tuneModified = NO;
  pendingTune = nil;

  [tunebookView reloadData];
  [tunebookView selectRowIndexes:[NSIndexSet indexSetWithIndex:1] byExtendingSelection:NO];
  [self setPlaylist:nil];
  [self setTune:nil];
  [tunebookView becomeFirstResponder];
}

- (void)dealloc {
	[[NSNotificationCenter defaultCenter] removeObserver:self];
//	[tableView dealloc];
//	[searchField release];
	[listContent release];
	[filteredListContent release];
	[musicData release];
  [pendingTune release];

  [super dealloc];
}

- (BOOL)acceptsFirstResponder
{
  return YES;
}

- (BOOL)becomeFirstResponder
{
  return YES;
}

- (void)databaseChanged:(id)sender
{
  if ( playlist )
    listContent = [playlist loadTunes];
  [self filterTunes];
  [filteredListContent sortUsingDescriptors:[tuneListView sortDescriptors]];
	[tunebookView reloadData];
	[tuneListView reloadData];
}

- (void)setPlaylist:(Playlist *)aPlaylist select:(BOOL)aSelect
{
  if ( aSelect ) {
    int row = [self rowForTunebook:aPlaylist];
    if ( row >= 0 ) {
      [tunebookView selectRowIndexes:[NSIndexSet indexSetWithIndex:row] byExtendingSelection:NO];
      return;
    }
  }
  [tunebookView deselectAll:self];
  [self setPlaylist:aPlaylist];
}

- (void)setPlaylist:(Playlist *)aPlaylist
{
  Playlist *old = playlist;
  playlist = aPlaylist;
  [playlist retain];
  [old release];

  if ( playlist ) {
    listContent = [playlist loadTunes];
  } else {
    listContent = database.tunes;
  }

	NSTableColumn* col = [tuneListView tableColumnWithIdentifier:@"order"];
  [col setHidden:playlist == nil];

  [filteredListContent release];
  filteredListContent = [[NSMutableArray alloc] initWithCapacity: [listContent count]];
  [self filterTunes];
  [filteredListContent sortUsingDescriptors:[tuneListView sortDescriptors]];
	[tuneListView reloadData];
  if ( [filteredListContent count] > 0 ) {
    [tuneListView selectRowIndexes:[NSIndexSet indexSetWithIndex:0] byExtendingSelection:NO];
    [self setTune:[filteredListContent objectAtIndex:0]];
  } else {
    [tuneListView deselectAll:self];
    [self setTune:nil];
  }
}

- (Playlist *)playlist
{
  return playlist;
}

- (void)sheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void  *)contextInfo
{
//  if ( returnCode == NSAlertOtherReturn ) // Cancel
//    [tuneListView selectRow:currentRow byExtendingSelection:NO];
  tuneModified = NO;
  if ( returnCode == NSAlertDefaultReturn ) { // Save
    [self saveEditedTune:self];
    [self setTune:pendingTune];
  }
  if ( returnCode == NSAlertAlternateReturn ) // Don't Save
    [self setTune:pendingTune];
  [pendingTune release];
  pendingTune = nil;
}

- (IBAction)saveEditedTune:(id)sender
{
  tuneModified = NO;
  parsedTunes = [[NSMutableArray alloc] init];
  [parsedTunes retain];
  TuneParser *parser = [[TuneParser alloc] init];
  parser.delegate = self;
  NSString *dots = abcTextView.string;
  [parser parseString:dots];
  if ( [parsedTunes count] == 0 ) {
    NSBeginAlertSheet(@"Bad ABC", @"OK", nil, nil, [self.view window], self, nil, nil, nil,
      @"The ABC does not contain any valid tunes." );
    [parsedTunes release];
  } else if ( [parsedTunes count] > 1 ) {
    NSBeginAlertSheet(@"Multiple tunes in ABC", @"OK", nil, nil, [self.view window], self, nil, nil, nil,
      @"Add Extra Tunes?" );
  } else {
    Tune* editTune = self.currentTune;
    Tune* t = (Tune *)[parsedTunes objectAtIndex:0];
    editTune.title = t.title;//FIXME-method to do this...
    editTune.dots = t.dots;
    editTune.rating = t.rating;
    editTune.key = t.key;
    editTune.numSharps = t.numSharps;
    editTune.timeSigNum = t.timeSigNum;
    editTune.timeSigDen = t.timeSigDen;
    if ( [editTune inDatabase] ) {
      [editTune dehydrate];
      [editTune hydrate];
    } else
      [database addTune:editTune];
    [parsedTunes release];
    [[NSNotificationCenter defaultCenter] postNotificationName:@"databaseChanged" object:self]; 
    [self setTune:editTune];
  }
  [_saveButton setEnabled:false];
}

- (void)setTune:(Tune*)tune
{
  if ( tuneModified ) {
     NSBeginAlertSheet(@"You have made changes to the tune.", @"Save", @"Don't Save", nil, [self.view window], self, NULL,
      @selector(sheetDidEnd:returnCode:contextInfo:),
      NULL,
      @"Do you want to save your changes?" );
    pendingTune = tune;
    [pendingTune retain];
    tuneModified = NO;
    return;
  }

  if ( self.currentTune != nil && tune.inDatabase )
    [self.currentTune dehydrate];

  self.currentTune = tune;
  currentRow = [tuneListView selectedRow];
  if ( tune ) {
    if ( tune.inDatabase )
      [tune hydrate];
    tuneView.tune = tune.dots;

    NSRect bounds;
    int numPages = 1;
    if ( tune ) {
      NSImage* image = [[NSImage alloc] initWithSize:NSMakeSize(8, 8)];
      [image lockFocus];
      CGContextRef context = (CGContextRef)[[NSGraphicsContext currentContext] graphicsPort];
      numPages = [Tune drawTune:tune.dots intoContext:context bounds:CGRectMake(0, 0, 8, 8) scale:1.0f paginate:YES];
      if ( numPages < 1 )
        numPages = 1;
      bounds = CGRectMake( 0, 0, PaperWidth, numPages * PaperHeight );
      [image unlockFocus];
    } else
      bounds = CGRectMake( 0, 0, PaperWidth, PaperHeight );
    [tuneView setFrame:bounds];
    NSPoint newScrollOrigin = NSMakePoint(0.0, NSMaxY([[tuneScrollView documentView] frame]) - NSHeight([[tuneScrollView contentView] bounds]));

    [[tuneScrollView documentView] scrollPoint:newScrollOrigin];

    [abcTextView setString:tune.dots];
    [abcTextView setEditable:true];
  }
  else {
    tuneView.tune = nil;
    [abcTextView setString:@""];
    [abcTextView setEditable:false];
  }
  [_saveButton setEnabled:false];
	[musicData release];
  musicData = nil;
  [[TunePlayer sharedInstance] stop];
  [tuneView setNeedsDisplay:true];
}

- (void)setTuneDots:(NSString*)dots
{
  if ( dots ) {
    tuneView.tune = dots;
  }
  else {
    tuneView.tune = nil;
  }
  [[TunePlayer sharedInstance] stop];
  [tuneView setNeedsDisplay:true];
}

- (Tune*)selectedTune
{
  int row = [tuneListView selectedRow];
  if ( row >= 0 && row < [filteredListContent count] )
    return [filteredListContent objectAtIndex:row];
  return nil;
}

- (NSArray*)selectedTunes
{
  NSInteger count = [tuneListView numberOfSelectedRows];
  if ( count == 0 )
    return nil;
  NSMutableArray *tunes = [[NSMutableArray alloc] initWithCapacity:count];
  NSInteger rowCount = [tuneListView numberOfRows];

  for ( int i = 0; i < rowCount; i++ ) {
    if ( [tuneListView isRowSelected:i] ) {
      Tune* t = [filteredListContent objectAtIndex:i];
      [tunes addObject:t];
    }
  }
  [tunes autorelease];
  return tunes;
}

#define kMaxTunesInSet 100

- (Tune*)tuneWithSelectedTunes
{
  NSArray* tunes = [self selectedTunes];
  NSMutableString* dots = [[NSMutableString alloc] init];
  Tune *tune = [[Tune alloc] init];
  tune.title = playlist ? playlist.name :@"Tunes";
  tune.key = @"D";
  tune.hydrated = YES;
  tune.temporary = YES;
  int count = MIN( kMaxTunesInSet, [tunes count] );
  for ( int i = 0; i < count; i++ ) {
    Tune* t = [tunes objectAtIndex:i];
    [t hydrate];
    NSString* s = [[t dots] stringByReplacingOccurrencesOfString:@"X:1" withString:[NSString stringWithFormat:@"X:%d", i+1]];
    [dots appendString:s];
    [dots appendString:@"\n"];
    [t dehydrate];
  }
  tune.dots = dots;
  [dots release];
  [tune autorelease];
  return tune;
}

- (Tune*)tuneFromTunebook:(Playlist*)tunebook
{
  NSArray* tunes = [tunebook loadTunes];
  NSMutableString* dots = [[NSMutableString alloc] init];
  Tune *tune = [[Tune alloc] init];
  tune.title = playlist ? playlist.name :@"Tunes";
  tune.key = @"D";
  tune.hydrated = YES;
  tune.temporary = YES;
  int count = MIN( kMaxTunesInSet, [tunes count] );
  for ( int i = 0; i < count; i++ ) {
    Tune* t = [tunes objectAtIndex:i];
    [t hydrate];
    NSString* s = [[t dots] stringByReplacingOccurrencesOfString:@"X:1" withString:[NSString stringWithFormat:@"X:%d", i+1]];
    [dots appendString:s];
    [dots appendString:@"\n"];
    [t dehydrate];
  }
  tune.dots = dots;
  [dots release];
  [tune autorelease];
  return tune;
}

- (Playlist*) tunebookForRow:(NSInteger)row
{
  row -= 3;
  if ( row >= 0 && row < [database.playlists count] )
    return [database.playlists objectAtIndex:row];
  return nil;
}

- (int) rowForTunebook:(Playlist*)tunebook
{
  int count = [database.playlists count];
  for ( int i = 0; i < count; i++ )
    if ( [database.playlists objectAtIndex:i] == tunebook )
      return i + 3;
  return -1;
}

- (Playlist*)selectedTunebook
{
  int row = [tunebookView selectedRow];
  return [self tunebookForRow:row];
}

// TuneParserDelegate methods
- (void) parsedTune:(TuneParser *)parser tune:(Tune *)t
{
  [parsedTunes addObject:t];
}

- (void)tableView:(NSTableView *)view willDisplayCell:(id)cell forTableColumn:(NSTableColumn *)col row:(NSInteger)row
{
  if ( view == tunebookView ) {
    static NSImage* sLibrary;
    if ( !sLibrary  )
      sLibrary = [[NSImage imageNamed:@"notes.png"] retain];

    static NSImage* sFolder;
    if ( !sFolder  )
      sFolder = [[NSImage imageNamed:@"book.png"] retain];

    static NSImage* sSmartFolder;
    if ( !sSmartFolder  )
      sSmartFolder = [[NSImage imageNamed:@"gear.png"] retain];

    Playlist* tunebook = [self tunebookForRow:row];
    BOOL title = (row == 0 || row == 2);
    [cell setFont: [NSFont systemFontOfSize:10.0]];
		[cell setImage:title ? nil : (row == 1 ? sLibrary : (tunebook.type == kPlaylistTypeSearch ? sSmartFolder : sFolder))];
		[cell setIndent:title ? 0 : 16];
	}
}

- (NSDragOperation)tableView:(NSTableView *)tableView validateDrop:(id <NSDraggingInfo>)info proposedRow:(NSInteger)row proposedDropOperation:(NSTableViewDropOperation)dropOperation
{
  if ( tableView == tunebookView ) {
    if ( row >= 3 || row == 1 ) {
      if ( dropOperation == NSTableViewDropOn || dropOperation == NSTableViewDropAbove ) {
        NSPasteboard *pboard = [info draggingPasteboard];
        if ( [[pboard types] containsObject:NSFilenamesPboardType] ) {
          return NSDragOperationCopy;
        }
        if ( [[pboard types] containsObject:NSURLPboardType] ) {
          return NSDragOperationCopy;
        }
        else if ( info.draggingSource == tunebookView && [[pboard types] containsObject:TunebookPrivateTableViewDataType] ) {
          if ( dropOperation == NSTableViewDropAbove )
            return NSDragOperationMove;
        }
        else if ( info.draggingSource == tuneListView && [[pboard types] containsObject:TunebookPrivateTableViewDataType] ) {
          return NSDragOperationCopy;
        }
      }
    }
  }
  else if ( tableView == tuneListView ) {
    if ( dropOperation == NSTableViewDropOn || dropOperation == NSTableViewDropAbove ) {
      NSPasteboard *pboard = [info draggingPasteboard];
      if ( [[pboard types] containsObject:NSFilenamesPboardType] ) {
        return NSDragOperationCopy;
      }
      if ( [[pboard types] containsObject:NSURLPboardType] ) {
        return NSDragOperationCopy;
      }
      if ( info.draggingSource == tuneListView && [[pboard types] containsObject:TunebookPrivateTableViewDataType] ) {
        if ( dropOperation == NSTableViewDropAbove && playlist && playlist.type == kPlaylistTypeSimple && [filteredListContent count] == [listContent count] ) {
          // Don't allow rerdering if we're not sorted by order
          NSArray* sortDescriptors = [tuneListView sortDescriptors];
          if ( [sortDescriptors count] == 0 )
            return NSDragOperationMove;
          NSSortDescriptor* sortDescriptor = [sortDescriptors objectAtIndex:0];
          if ( [[sortDescriptor key] isEqualToString:@"order"] && [sortDescriptor ascending] )
            return NSDragOperationMove;
        }
      }
    }
  }

  return NSDragOperationNone;
}

- (BOOL)tableView:(NSTableView *)tableView acceptDrop:(id <NSDraggingInfo>)info row:(NSInteger)row dropOperation:(NSTableViewDropOperation)dropOperation
{
  if ( ((tableView == tunebookView && row >= 3) || row == 1) || tableView == tuneListView ) {
    if ( dropOperation == NSTableViewDropOn || dropOperation == NSTableViewDropAbove ) {
      Playlist* droppedPlaylist = tableView == tuneListView ? playlist : [self tunebookForRow:row];
      NSPasteboard *pboard = [info draggingPasteboard];
      if ( [[pboard types] containsObject:NSFilenamesPboardType] ) {
        NSArray *files = [pboard propertyListForType:NSFilenamesPboardType];

        for ( NSString* path in files ) {
          parsedTunes = [[NSMutableArray alloc] init];
          [parsedTunes retain];
          TuneParser *parser = [[TuneParser alloc] init];
          parser.delegate = self;
          NSData *data = [NSData dataWithContentsOfFile:path options:0 error:nil];
          if ( data ) {
            [parser parseData:data];
            if ( [parsedTunes count] == 0 ) {
              NSBeginAlertSheet(@"Can't import the file", @"OK", nil, nil, [self.view window], self, nil, nil, nil,
                @"The file does not contain any valid ABC tunes (perhaps it wasn't an ABC file after all)." );
              return NO;
            }
            [database startTransaction];
            Playlist* p = nil;
            if ( row != 1 ) {
              if ( dropOperation == NSTableViewDropOn )
                p = droppedPlaylist;
              else {
                p = [[Playlist alloc] init];
                p.name = [path lastPathComponent];
                [database addPlaylist:p];
              }
            }
            for ( Tune* tune in parsedTunes ) {
              [database addTune:tune];
              if ( p )
                [p addTune:tune];
            }
            [database commitTransaction];
            [database updateTunebookSortOrder];
          }
          [parsedTunes release];
        }
      }
      else if ( [[pboard types] containsObject:NSURLPboardType] ) {
        NSURL *url = [NSURL URLFromPasteboard:pboard];
        Playlist *tunebook;
        if ( droppedPlaylist )
          tunebook = droppedPlaylist;
        else {
          tunebook = [[Playlist alloc] init];
          tunebook.name = [[url path] lastPathComponent];
        }
        ImportProgressController *importProgressController = [[ImportProgressController alloc] initWithNibName:@"ImportProgress" bundle:nil];
        [importProgressController import:url intoTunebook:tunebook withDatabase:database window:[self.view window]];
      }
      else if ( [[pboard types] containsObject:TunebookPrivateTableViewDataType] ) {
        NSData* rowData = [pboard dataForType:TunebookPrivateTableViewDataType];
        NSIndexSet* rowIndexes = [NSKeyedUnarchiver unarchiveObjectWithData:rowData];

        if ( info.draggingSource == tunebookView && dropOperation == NSTableViewDropAbove ) {
          // Reordering rows in the tunebook view
          NSInteger count = [rowIndexes count];
          NSMutableArray *movedRows = [NSMutableArray arrayWithCapacity:count];
          NSUInteger currentIndex = [rowIndexes firstIndex];
          while ( currentIndex != NSNotFound ) {
            int index = currentIndex-3;
            [movedRows addObject:[database.playlists objectAtIndex:index]];
            [database.playlists replaceObjectAtIndex:index withObject:[NSNull null]];
            currentIndex = [rowIndexes indexGreaterThanIndex:currentIndex];
          }
          [database.playlists replaceObjectsInRange:NSMakeRange(row-3, 0) withObjectsFromArray:movedRows];
          [database.playlists removeObjectIdenticalTo:[NSNull null]];
          [tunebookView deselectAll:nil];
          [tunebookView reloadData];
          [database updateTunebookSortOrder];
        }
        else if ( tableView == tuneListView && info.draggingSource == tuneListView && [[pboard types] containsObject:TunebookPrivateTableViewDataType] ) {
          // Reordering rows in the tune view
          NSInteger count = [rowIndexes count];
          NSMutableArray *movedRows = [NSMutableArray arrayWithCapacity:count];
          NSUInteger currentIndex = [rowIndexes firstIndex];
          while ( currentIndex != NSNotFound ) {
            int index = currentIndex;
            [movedRows addObject:[listContent objectAtIndex:index]];
            [listContent replaceObjectAtIndex:index withObject:[NSNull null]];
            currentIndex = [rowIndexes indexGreaterThanIndex:currentIndex];
          }
          [listContent replaceObjectsInRange:NSMakeRange(row, 0) withObjectsFromArray:movedRows];
          [listContent removeObjectIdenticalTo:[NSNull null]];
          [self filterTunes];
          [tuneListView deselectAll:nil];
          [tuneListView reloadData];
          [playlist updateSortOrder:listContent];
        }
        else {
          // Dragging tunes into the tunebook view
          Playlist *tunebook;
          if ( droppedPlaylist && dropOperation == NSTableViewDropOn )
            tunebook = droppedPlaylist;
          else {
            tunebook = [[Playlist alloc] init];
            tunebook.name = @"New Tunebook";
            [database addPlaylist:tunebook];
            row -= 3;
            // Add playlist adds to the end. Move it to the new row.
            if ( row < [database.playlists count]-1 ) {
              [database.playlists removeObject:tunebook];
              [database.playlists insertObject:tunebook atIndex:row];
            }
            [database updateTunebookSortOrder];
          }
          NSUInteger currentIndex = [rowIndexes firstIndex];
          while ( currentIndex != NSNotFound ) {
            [tunebook addTune:[filteredListContent objectAtIndex:currentIndex]];
            currentIndex = [rowIndexes indexGreaterThanIndex:currentIndex];
          }
        }
      }
      [tuneListView reloadData];
      [[NSNotificationCenter defaultCenter] postNotificationName:@"databaseChanged" object:database]; //FIXME-database should do this
      [[NSNotificationCenter defaultCenter] postNotificationName:@"playlistsChanged" object:database]; //FIXME-database should do this
    }
  }

  return YES;
}

- (NSInteger)numberOfRowsInTableView:(NSTableView *)tableView
{
  if ( tableView == tuneListView )
    return [filteredListContent count];
  if ( tableView == tunebookView )
    return [database.playlists count]+3;
  return 0;
}

- (BOOL)tableView:(NSTableView *)tableView isGroupRow:(NSInteger)row
{
  if ( tableView == tunebookView )
    return row == 0 || row == 2;
  return NO;
}

- (BOOL)tableView:(NSTableView *)tableView shouldSelectRow:(NSInteger)row
{
  return ![self tableView:tableView isGroupRow:row];
}

- (id)tableView:(NSTableView *)tableView objectValueForTableColumn:(NSTableColumn *)tableColumn row:(NSInteger)row
{
  if ( tableView == tuneListView ) {
    Tune* tune = (Tune*)[filteredListContent objectAtIndex:row];
    if ( [[tableColumn identifier] isEqualToString:@"key"] )
      return tune.key;
    if ( [[tableColumn identifier] isEqualToString:@"meter"] ) {
      if ( tune.timeSigDen != 0 )
        return [NSString stringWithFormat:@"%d/%d", tune.timeSigNum, tune.timeSigDen];
      else
        return @"?";
    }
    if ( [[tableColumn identifier] isEqualToString:@"order"] )
      return [NSNumber numberWithInt:tune.order];
    return tune.title;
  }
  if ( tableView == tunebookView ) {
    if ( row == 0 )
      return @"LIBRARY";
    if ( row == 1 )
      return @"Tunes";
    if ( row == 2 )
      return @"TUNEBOOKS";
    Playlist* p = [database.playlists objectAtIndex:row-3];
    return [NSString stringWithFormat:@"%@", [p name]];
  }
  return @"";
}

- (void)tableView:(NSTableView *)tableView setObjectValue:(id)object forTableColumn:(NSTableColumn *)tableColumn row:(NSInteger)row
{
  if ( tableView == tunebookView ) {
    Playlist* p = [self tunebookForRow:row];
    if ( p ) {
      NSString* name = [object stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
      if ( ![p.name isEqualToString:name] ) {
        [p updateName:name];
        [tunebookView reloadData];
      }
    }
  }
}

- (BOOL)tableView:(NSTableView *)tv shouldEditTableColumn:(NSTableColumn *)tableColumn row:(NSInteger)row
{
  return tv == tunebookView && [self tunebookForRow:row] != nil;
}

- (BOOL)tableView:(NSTableView *)tv writeRowsWithIndexes:(NSIndexSet *)rowIndexes toPasteboard:(NSPasteboard*)pboard
{
  NSData *zNSIndexSetData = [NSKeyedArchiver archivedDataWithRootObject:rowIndexes];
  [pboard declareTypes:[NSArray arrayWithObject:TunebookPrivateTableViewDataType] owner:self];
  [pboard setData:zNSIndexSetData forType:TunebookPrivateTableViewDataType];
  return YES;
}

- (void)tableViewSelectionDidChange:(NSNotification *)notification;
{
  NSTableView *tableView = [notification object];
  int row = [tableView selectedRow];
  if ( tableView == tuneListView ) {
    NSInteger count = [tuneListView numberOfSelectedRows];
    if ( count > 1 ) {
      [self setTune:[self tuneWithSelectedTunes]];
      [abcTextView setEditable:NO];
    }
    else if ( row >= 0 ) {
      Tune* tune = (Tune*)[filteredListContent objectAtIndex:row];
//      [tune hydrate];
      [self setTune:tune];
      [abcTextView setEditable:YES];
    }
    else {
      [self setTune:nil];
    }
  }
  else if ( tableView == tunebookView ) {
    if ( row == 1 )
      [self setPlaylist:nil];
    else if ( row >= 3 ) {
      Playlist* p = [self tunebookForRow:row];
      [self setPlaylist:p];
    }
  }
}

- (void)tableView:(NSTableView *)tableView sortDescriptorsDidChange:(NSArray *)oldDescriptors
{
  if ( tableView == tuneListView ) {
    [filteredListContent sortUsingDescriptors:[tableView sortDescriptors]];
    [tableView reloadData];
  }
}

- (void)filterTunes
{
  [filteredListContent removeAllObjects];
  [filteredListContent addObjectsFromArray: listContent];
  int index = 0;
  for ( Tune* tune in listContent )
    tune.order = ++index;
}

- (void)controlTextDidChange:(NSNotification *)notification
{
	if ( notification.object == searchField ) {
    [filteredListContent removeAllObjects];
    [database searchForTunes:[searchField stringValue] tunes:listContent results:filteredListContent];
    [tuneListView deselectAll:self];
    [tuneListView reloadData];
  }
}

- (NSRange)findLineNumber:(int)line
{
    int current = 1;
    NSRange r;

    /* get the data as a c string, fill out an NSRange corresponding to the desired line... */

    char *data = (char*)[[abcTextView string] UTF8String];

    if(*data == 0) return NSMakeRange(0, 1);

    r.location = r.length = 0;

    /* find the start of the desired line */

    while(data[r.location] && current < line) {
        if(data[r.location] == '\n') current++;
        r.location++;
    }   

    if(!data[r.location]) {
        /* if we're at \0, then we've reached the end of the string without */
        /* finding enough lines--step backwards to select the final line.   */

        while(r.location && data[r.location--] != '\n') r.length++;
    } else {
        /* if we're here, then we've found the line we want--step forward  */
        /* until we find a newline or the end of the string */

        while(data[r.location + r.length] && data[r.location + r.length] != '\n')
            r.length++;
    }

    return r;
}

- (void)textDidChange:(NSNotification *)notification
{
	if ( notification.object == abcTextView ) {
    tuneModified = YES;
    [_saveButton setEnabled:tuneModified];
//printf("setdots\n");
    [self setTuneDots:[abcTextView string]];
//printf("after setdots\n");
    //FIXME-experimental
//    if ( sErrorLine != -1 ) {
//      NSRange range = [self findLineNumber:sErrorLine];
//      [abcTextView setTextColor:[NSColor redColor] range:range];
//      sErrorLine = -1;
//    }
  }
}

- (void)addTunebook:(id)sender
{
  Playlist* p = [[Playlist alloc] init];
  p.name = @"New Tunebook";
  [database addPlaylist:p];
  [database updateTunebookSortOrder];
  [tunebookView reloadData];
}

- (void)removeTunebook:(id)sender
{
}

- (void)newTune:(id)sender
{
  Tune *tune = [[Tune alloc] init];
  tune.title = @"New Tune";
  tune.key = @"D";
  tune.dots = @"X: 1\nT: New Tune\nL: 1/8\nM:4/4\nK: D\n|";
  [database addTune:tune];
  if ( playlist ) {
    [playlist addTune:tune];
    [self setPlaylist:playlist];
  }
  [[NSNotificationCenter defaultCenter] postNotificationName:@"databaseChanged" object:database]; //FIXME-database should do this
  int row = [filteredListContent indexOfObject:tune];
  if ( row >= 0 )
    [tuneListView selectRowIndexes:[NSIndexSet indexSetWithIndex:row] byExtendingSelection:NO];
}

- (void)removeTune:(id)sender
{
}

- (void)printTune:(id)sender
{
  NSResponder* responder = [[self.view window] firstResponder];
  NSString* dots = nil;
  if ( responder == tunebookView ) {
    Playlist* tunebook = [self selectedTunebook];
    if ( tunebook )
      dots = [self tuneFromTunebook:tunebook].dots;
  }
  else if ( responder == tuneListView )
    dots = abcTextView.string;
  if ( dots ) {
    NSData* data = [Tune pdfDataForTune:dots];
    PDFDocument *pdfDocument = [[[PDFDocument alloc] initWithData:(NSData*)data] autorelease];
    BOOL autoRotate = YES;
    NSPrintInfo *printInfo = [NSPrintInfo sharedPrintInfo];
    NSPrintOperation *op = [pdfDocument printOperationForPrintInfo:printInfo scalingMode:kPDFPrintPageScaleNone autoRotate:autoRotate];
    [op setShowsPrintPanel:YES];
    [op setShowsProgressPanel:YES];
    [op runOperation];
  }
}

- (void) keyDown:(NSEvent *)event
{
  NSString *chars = [event characters];
  unichar character = [chars characterAtIndex: 0];

  if (character == NSDeleteCharacter) {
    [self delete:self];
  }
  
  //FIXME...
//  TunePlayer* tunePlayer = [TunePlayer sharedInstance];
//  if (character == 'z') {
//    [tunePlayer transpose:-100];
//  }
//  if (character == 'x') {
//    [tunePlayer transpose:100];
//  }
}

-(IBAction) play:(id)sender
{
  if ( tuneView.tune ) {
    TunePlayer* tunePlayer = [TunePlayer sharedInstance];
    if ( [tunePlayer isPlaying] )
      [tunePlayer stop];
    else {
      if ( !musicData )
        musicData = [[Tune midiDataForTune:tuneView.tune guitarChords:YES] retain];
      if ( musicData ) {
        [tunePlayer setSMFData:musicData];
        [tunePlayer play];
      }
    }
  }
}

- (IBAction) showHelp:(id)sender
{
  NSString *helpFilePath = [[NSBundle mainBundle] pathForResource:@"tunebook" ofType:@"html" inDirectory:@"TunebookHelp"];
  NSURL *helpFileURL = [NSURL fileURLWithPath:helpFilePath];
  [[NSWorkspace sharedWorkspace]openURL:helpFileURL];
}

- (IBAction) showPlaySettings:(id)sender
{
  PlaySettingsWindowController* playSettingsWindowController = [PlaySettingsWindowController sharedInspector];
  if ( [[playSettingsWindowController window] isVisible] )
    [playSettingsWindowController close];
  else {
    [playSettingsWindowController showWindow:playSettingsWindowController];
    [[playSettingsWindowController window] orderFront:playSettingsWindowController];
  }
}

-(IBAction) delete:(id)sender
{
  NSResponder* responder = [[self.view window] firstResponder];
  if ( responder == tunebookView ) {
    int row = [tunebookView selectedRow];
    Playlist* p = [self tunebookForRow:row];
    if ( p )
      [database removePlaylist:p];
    [self setPlaylist:[self selectedTunebook]];
  }
  else if ( responder == tuneListView ) {
    NSArray* tunes = [self selectedTunes];
    if ( tunes ) {
      for ( Tune* tune in tunes ) {
        if ( playlist )
          [playlist removeTune:tune];
        else
          [database removeTune:tune];
      }
      [[NSNotificationCenter defaultCenter] postNotificationName:@"databaseChanged" object:database]; //FIXME-database should do this
      [self setTune:[self selectedTune]];
    }
  }
}

-(IBAction) cut:(id)sender
{
}

-(IBAction) copy:(id)sender
{
}

-(IBAction) paste:(id)sender
{
}

-(IBAction) importTunes:(id)sender
{
}

-(IBAction) search:(id)sender
{
//  SearchWindowController *search = [[SearchWindowController alloc] initWithWindowNibName:@"Search"];
//  search.delegate = self;
//  NSPredicate* predicate = [NSPredicate predicateWithFormat:@"(title = '') AND dots CONTAINS ''"];
////  predicate = [NSCompoundPredicate andPredicateWithSubpredicates:[NSArray arrayWithObject:predicate]];
//  [search showSearch:predicate window:[self.view window]];
  Playlist* tunebook = [[Playlist alloc] initWithName:@"Search Results" abc:@"" title:@"" key:@"" meter:@"" database:database];
  tunebook.database = database;
    SimpleSearchWindowController *search = [[SimpleSearchWindowController alloc] initWithWindowNibName:@"SimpleSearch"];
  search.delegate = self;
  [search showSearch:tunebook window:[self.view window]];
#if 0//FIXME
    PianoKeyboardViewController* controller = [[PianoKeyboardViewController alloc] initWithNibName:@"PianoKeyboardView" bundle:nil];
    [self.view addSubview:controller.view];
#endif
}

-(void) searchWithPredicate:(NSPredicate*)aPredicate
{
  Playlist *tunebook = [[Playlist alloc] initWithPredicate:aPredicate database:database];
  [self setPlaylist:tunebook select:YES];
}

- (IBAction)saveTunes:(id)sender
{
  NSResponder* responder = [[self.view window] firstResponder];
  NSString* name = @"Tunes.abc";
  Playlist* tunebook = nil;
  if ( responder == tunebookView ) {
    tunebook = [self selectedTunebook];
    if ( tunebook ) {
      name = tunebook.name;
      if ( ![name hasSuffix:@".abc"] )
        name = [name stringByAppendingString:@".abc"];
    }
  }
  NSSavePanel *savePanel = [NSSavePanel savePanel];
  NSString *path = @"/Documents";
  [savePanel setDirectory:[path stringByExpandingTildeInPath]];
  [savePanel setTitle:tunebook ? @"Save Tunebook as an ABC file" : @"Save Tunes as an ABC file"];
  [savePanel setRequiredFileType:@"abc"];
  [savePanel beginSheetForDirectory:NSHomeDirectory()
    file:name
    modalForWindow:[self.view window]
    modalDelegate:self
    didEndSelector:@selector(didEndSaveSheet:returnCode:contextInfo:) contextInfo:responder];
}

-(void)didEndSaveSheet:(NSSavePanel *)savePanel returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
  if ( returnCode == NSModalResponseOK ) {
    NSString* path = [savePanel filename];
    NSResponder* responder = (NSResponder*)contextInfo;
    NSString* dots = nil;
    if ( responder == tunebookView ) {
      Playlist* tunebook = [self selectedTunebook];
      if ( tunebook )
        dots = [self tuneFromTunebook:tunebook].dots;
    }
    else if ( responder == tuneListView )
      dots = abcTextView.string;
    if ( dots ) {
      NSError* error = nil;
      if ( ![dots writeToFile:path atomically:NO encoding:NSUTF8StringEncoding error:&error] )
        [[NSApplication sharedApplication] presentError:error];        
    }
  }
}

- (IBAction)saveTunesAsMIDI:(id)sender
{
  NSResponder* responder = [[self.view window] firstResponder];
  NSString* name = @"Tunes.mid";
  Playlist* tunebook = nil;
  if ( responder == tunebookView ) {
    tunebook = [self selectedTunebook];
    if ( tunebook ) {
      name = tunebook.name;
      if ( ![name hasSuffix:@".mid"] )
        name = [name stringByAppendingString:@".mid"];
    }
  }
  NSSavePanel *savePanel = [NSSavePanel savePanel];
  NSString *path = @"/Documents";
  [savePanel setDirectory:[path stringByExpandingTildeInPath]];
  [savePanel setTitle:tunebook ? @"Save Tunebook as a MIDI file" : @"Save Tunes as an MIDI file"];
  [savePanel setRequiredFileType:@"mid"];
  [savePanel beginSheetForDirectory:NSHomeDirectory()
    file:name
    modalForWindow:[self.view window]
    modalDelegate:self
    didEndSelector:@selector(didEndSaveSheetMIDI:returnCode:contextInfo:) contextInfo:responder];
}

-(void)didEndSaveSheetMIDI:(NSSavePanel *)savePanel returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
  if ( returnCode == NSModalResponseOK ) {
    NSString* path = [savePanel filename];
    NSResponder* responder = (NSResponder*)contextInfo;
    NSString* dots = nil;
    if ( responder == tunebookView ) {
      Playlist* tunebook = [self selectedTunebook];
      if ( tunebook )
        dots = [self tuneFromTunebook:tunebook].dots;
    }
    else if ( responder == tuneListView )
      dots = abcTextView.string;
    if ( dots ) {
      NSError* error = nil;
      NSData* data = [Tune midiDataForTune:dots guitarChords:YES];
      if ( ![data writeToFile:path options:NSAtomicWrite error:&error] )
        [[NSApplication sharedApplication] presentError:error];        
    }
  }
}

- (IBAction)saveTunesAsPDF:(id)sender
{
  NSResponder* responder = [[self.view window] firstResponder];
  NSString* name = @"Tunes.pdf";
  Playlist* tunebook = nil;
  if ( responder == tunebookView ) {
    tunebook = [self selectedTunebook];
    if ( tunebook ) {
      name = tunebook.name;
      if ( ![name hasSuffix:@".pdf"] )
        name = [name stringByAppendingString:@".pdf"];
    }
  }
  NSSavePanel *savePanel = [NSSavePanel savePanel];
  NSString *path = @"/Documents";
  [savePanel setDirectory:[path stringByExpandingTildeInPath]];
  [savePanel setTitle:tunebook ? @"Save Tunebook as a PDF file" : @"Save Tunes as a PDF file"];
  [savePanel setRequiredFileType:@"pdf"];
  [savePanel beginSheetForDirectory:NSHomeDirectory()
    file:name
    modalForWindow:[self.view window]
    modalDelegate:self
    didEndSelector:@selector(didEndSaveSheetPDF:returnCode:contextInfo:) contextInfo:responder];
}

-(void)didEndSaveSheetPDF:(NSSavePanel *)savePanel returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
  if ( returnCode == NSModalResponseOK ) {
    NSString* path = [savePanel filename];
    NSResponder* responder = (NSResponder*)contextInfo;
    NSString* dots = nil;
    if ( responder == tunebookView ) {
      Playlist* tunebook = [self selectedTunebook];
      if ( tunebook )
        dots = [self tuneFromTunebook:tunebook].dots;
    }
    else if ( responder == tuneListView )
      dots = abcTextView.string;
    if ( dots ) {
      NSError* error = nil;
      NSData* data = [Tune pdfDataForTune:dots];
      if ( ![data writeToFile:path options:NSAtomicWrite error:&error] )
        [[NSApplication sharedApplication] presentError:error];        
    }
  }
}

//- (IBAction)saveTunesAsImages:(id)sender
//{
//  NSResponder* responder = [[self.view window] firstResponder];
//  NSString* name = @"Tunes";
//  Playlist* tunebook = nil;
//  if ( responder == tunebookView ) {
//    tunebook = [self selectedTunebook];
//    if ( tunebook ) {
//      name = tunebook.name;
////      if ( ![name hasSuffix:@".mid"] )
////        name = [name stringByAppendingString:@".mid"];
//    }
//  }
//  NSSavePanel *savePanel = [NSSavePanel savePanel];
//  NSString *path = @"/Documents";
//  [savePanel setDirectory:[path stringByExpandingTildeInPath]];
//  [savePanel setTitle:tunebook ? @"Save Tunebook as a series of images" : @"Save Tunes as a series of images"];
////  [savePanel setRequiredFileType:@"mid"];
//  [savePanel beginSheetForDirectory:NSHomeDirectory()
//                               file:name
//                     modalForWindow:[self.view window]
//                      modalDelegate:self
//                     didEndSelector:@selector(didEndSaveSheetImages:returnCode:contextInfo:) contextInfo:responder];
//}
//
//-(void)didEndSaveSheetImages:(NSSavePanel *)savePanel returnCode:(int)returnCode contextInfo:(void *)contextInfo
//{
//  if ( returnCode == NSModalResponseOK ) {
//    NSString* path = [savePanel filename];
//    NSResponder* responder = (NSResponder*)contextInfo;
//    NSString* dots = nil;
//    if ( responder == tunebookView ) {
//      Playlist* tunebook = [self selectedTunebook];
//      NSArray* tunes = [tunebook loadTunes];
//      for ( Tune* tune in tunes ) {
//        BOOL hydrated = tune.hydrated;
//        [tune hydrate];
//        //
//        if ( !hydrated )
//          [tune dehydrate];
//      }
//    }
//    else if ( responder == tuneListView )
//      dots = abcTextView.string;
//    if ( dots ) {
//      NSError* error = nil;
//      NSData* data = [Tune midiDataForTune:dots guitarChords:YES];
//      if ( ![data writeToFile:path options:NSAtomicWrite error:&error] )
//        [[NSApplication sharedApplication] presentError:error];        
//    }
//  }
//}

- (BOOL) validateUserInterfaceItem:(id <NSValidatedUserInterfaceItem>)item
{
//  NSLog(@"validateUserInterfaceItem %@", item);
  if ( [item action] == @selector(newTune:) )
    return YES;
  if ( [item action] == @selector(addTunebook:) )
    return YES;
  if ( [item action] == @selector(showHelp:) )
    return YES;
  if ( [item action] == @selector(search:) )
    return YES;
  if ( [item action] == @selector(runScript:) )
    return YES;

  NSResponder* responder = [[self.view window] firstResponder];
  if ( responder == tunebookView ) {
    Playlist* p = [self selectedTunebook];
    if ( p )
      return YES;
    return NO;
  }
  if ( responder == tuneListView ) {
    NSInteger count = [tuneListView numberOfSelectedRows];
//    Tune* tune = [self selectedTune];
    if ( [item action] == @selector(saveTunesAsMIDI:) )
      return count == 1;
    if ( count != 0 )
      return YES;
    return NO;
  }
  return NO;
}

- (BOOL) validateToolbarItem:(NSToolbarItem *)theItem;
{
//  NSLog(@"validateToolbarItem %@", [theItem label]);
//  NSResponder* responder = [[self.view window] firstResponder];
  BOOL hasTune = tuneView.tune != nil;
  if ( hasTune ) {
    if ( [theItem action] == @selector(printTune:) )
      return hasTune;
  }
  if ( [theItem action] == @selector(play:) ) {
    if ( [[TunePlayer sharedInstance] isPlaying] ) {
      [theItem setLabel:@"Stop"];
      [theItem setImage:[NSImage imageNamed:@"Stop"]];
    }
    else {
      [theItem setLabel:@"Play"];
      [theItem setImage:[NSImage imageNamed:@"Play"]];
      return hasTune;
    }
    return YES;
  }
  if ( [theItem action] == @selector(showPlaySettings:) )
    return YES;
  if ( [theItem action] == @selector(search:) )
    return YES;
  return NO;
}

- (void)finishedPlaying:(id)sender
{
  NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
  [[[self.view window] toolbar] validateVisibleItems];
  [pool release];
}

- (void)runScript:(NSMenuItem*)sender
{
  PluginContext* context = [[[PluginContext alloc] initWithController:self] autorelease];
	PluginManager *pluginManager = [PluginManager sharedManager];
	NSString *path = [sender representedObject];
	BOOL success = [pluginManager loadScriptAtPath:path runFunction:@"main" withArguments:[NSArray arrayWithObject:context]];
	if (!success)
		NSRunAlertPanel(@"Can't Run Plugin", @"The plugin returned an unknown error.", nil, nil, nil);
}

@end

@implementation TuneItem
- (id)initWithTune:(Tune*)aTune
{
  self = [super init];
  if ( self ) {
    tune = aTune;
  }
  return self;
}

- (void)dealloc {
  [tune release];
  [super dealloc];
}

- (NSData*)data
{
  BOOL hydrated = tune.hydrated;
  [tune hydrate];
  NSData* data = [Tune midiDataForTune:tune.dots guitarChords:YES];
  if ( !hydrated )
    [tune dehydrate];
  return data;
}
@end

