//
//  Tunelayer.m
//  Tunebook-Mac
//
//  Created by Jerry on 22/07/2011.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import "TunePlayer.h"
#import "Tunebook_MacAppDelegate.h"

@implementation TunePlayer

@synthesize instrument;

NSString* kTunePlayerFinishedNotification = @"TunePlayerFinished";

enum {
    kMidiMessageNoteOn        = 0x90,
    kMidiMessageNoteOff       = 0x80,
    kMidiMessageBassOn        = 0x91,
    kMidiMessageBassOff       = 0x81,
    kMidiMessageChordOn       = 0x92,
    kMidiMessageChordOff      = 0x82,
    kMidiMessageControlChange = 0xB0,
    kMidiMessageProgramChange = 0xC0,
    kMidiMessage_PitchBend    = 0x0E,
};

static void TunePlayerCallback(	void *inClientData,
    MusicSequence inSequence,
    MusicTrack inTrack,
    MusicTimeStamp inEventTime,
    const MusicEventUserData* inEventData,
    MusicTimeStamp inStartSliceBeat,
    MusicTimeStamp inEndSliceBeat)
{
  [[TunePlayer sharedInstance] performSelectorOnMainThread:@selector(endOfSequence) withObject:nil waitUntilDone:NO];
}

- (id)init
{
  self = [super init];
  if ( self ) {
    musicPlayer = 0;
    musicSequence = 0;
    if ( NewMusicPlayer( &musicPlayer ) == noErr ) {
      graph = 0;
      OSStatus result;
      AUNode synthNode, limiterNode, outNode;
      
      AudioComponentDescription cd;
      cd.componentManufacturer = kAudioUnitManufacturer_Apple;
      cd.componentFlags = 0;
      cd.componentFlagsMask = 0;
   
      result = NewAUGraph( &graph );
   
      cd.componentType = kAudioUnitType_MusicDevice;
      cd.componentSubType = kAudioUnitSubType_DLSSynth;
   
      result = AUGraphAddNode( graph, &cd, &synthNode);
   
      cd.componentType = kAudioUnitType_Effect;
      cd.componentSubType = kAudioUnitSubType_PeakLimiter;  
   
      result = AUGraphAddNode (graph, &cd, &limiterNode);
   
      cd.componentType = kAudioUnitType_Output;
      cd.componentSubType = kAudioUnitSubType_DefaultOutput;  
      result = AUGraphAddNode (graph, &cd, &outNode);
      
      result = AUGraphOpen (graph);
      
      result = AUGraphConnectNodeInput (graph, synthNode, 0, limiterNode, 0);
      result = AUGraphConnectNodeInput (graph, limiterNode, 0, outNode, 0);
      
      result = AUGraphNodeInfo(graph, synthNode, 0, &synthUnit);

      result = AUGraphInitialize(graph);
    }

    items = nil;
    currentItem = 0;

    instrument = [[NSUserDefaults standardUserDefaults] integerForKey:kPlayerVoiceKey];
    chordInstrument = [[NSUserDefaults standardUserDefaults] integerForKey:kPlayerChordVoiceKey];
    speed = [[NSUserDefaults standardUserDefaults] floatForKey:kPlayerSpeedKey];
    [self setInstrument:instrument];
    [self setChordInstrument:chordInstrument];
    [self setSpeed:speed];
  }
  return self;
}

-(void)dealloc
{
  if ( musicPlayer )
    DisposeMusicPlayer( musicPlayer );
  if ( musicSequence )
    DisposeMusicSequence( musicSequence );
  if ( graph )
    DisposeAUGraph( graph );
  [items release];
  [super dealloc];
}

+ (TunePlayer *)sharedInstance
{
  static TunePlayer *gInstance;

  if ( gInstance == nil ) {
    gInstance = [[TunePlayer alloc] init];
  }

  return gInstance;
}

- (NSInteger)instrument
{
  return instrument;
}

- (void)setInstrument:(NSInteger)i
{
  instrument = i;
  OSStatus result;
  result = MusicDeviceMIDIEvent(synthUnit, kMidiMessageProgramChange + 0, instrument, 0, 0);
  [[NSUserDefaults standardUserDefaults] setInteger:i forKey:kPlayerVoiceKey];
}

- (NSInteger)chordInstrument
{
  return chordInstrument;
}

- (void)setChordInstrument:(NSInteger)i
{
  chordInstrument = i;
  OSStatus result;
  result = MusicDeviceMIDIEvent(synthUnit, kMidiMessageProgramChange + 1, chordInstrument, 0, 0);
  [[NSUserDefaults standardUserDefaults] setInteger:i forKey:kPlayerChordVoiceKey];
}

- (void)transpose:(NSInteger)delta
{
//FIXME...
  int value = delta;
  int groupID = 0;
  UInt32 status = 0xB0 | groupID;
  OSStatus result;
  result = MusicDeviceMIDIEvent(synthUnit, status, 0x64, 0x00, 0); // RPN pitch bend range.
  result = MusicDeviceMIDIEvent(synthUnit, status, 0x65, 0x00, 0);
  result = MusicDeviceMIDIEvent(synthUnit, status, 0x06, (UInt32)value, 0); // Data entry MSB
  result = MusicDeviceMIDIEvent(synthUnit, status, 0x26, 0x00, 0); // Data entry LSB (optional)
  result = MusicDeviceMIDIEvent(synthUnit, status, 0x64, 0x7F, 0); // RPN reset
  result = MusicDeviceMIDIEvent(synthUnit, status, 0x65, 0x7F, 0);

//value+=(short)0x2000;	// center value
//unsigned char byte1=(unsigned char)(value & 0x7f); // 7 bit bytes
//unsigned char byte2=(unsigned char)( (value >> 7 ) & 0x7f );
//
//int channel = 0;
//UInt32 pitchChange = kMidiMessage_PitchBend << 4 | channel;
//
//result = MusicDeviceMIDIEvent(synthUnit, pitchChange, byte1, byte2, 0);
}

- (void)endOfSequence
{
  if ( !loop )
    [self stop];
}

- (void)play
{
  [self setLoop:loop];
  [self setChords:chords];
  MusicPlayerStart( musicPlayer );
}

- (void)stop
{
  if ( musicPlayer && musicSequence ) {
    if ( [self isPlaying] )
      MusicPlayerStop( musicPlayer );
    [[NSNotificationCenter defaultCenter] postNotificationName:kTunePlayerFinishedNotification object:[TunePlayer sharedInstance]];
  }
}

- (BOOL)isPlaying
{
  if ( musicPlayer ) {
    Boolean isPlaying;
    if ( MusicPlayerIsPlaying( musicPlayer, &isPlaying ) == noErr && isPlaying )
      return YES;
  }
  return NO;
}

- (void)setSMFData:(NSData*)smfData
{
  MusicSequence newMusicSequence = 0;
  if ( NewMusicSequence( &newMusicSequence ) == noErr ) {
    if ( MusicSequenceFileLoadData( newMusicSequence, (CFDataRef)smfData, kMusicSequenceFile_MIDIType, kMusicSequenceLoadSMF_ChannelsToTracks ) == noErr ) {
      if ( graph )
        MusicSequenceSetAUGraph( newMusicSequence, graph );
      if ( MusicPlayerSetSequence( musicPlayer, newMusicSequence ) == noErr) {
        MusicSequenceSetUserCallback(	newMusicSequence, TunePlayerCallback, NULL );

        UInt32 ntracks;
        MusicSequenceGetTrackCount( newMusicSequence, &ntracks );
        MusicTimeStamp sequenceLength = 0;
        for (UInt32 i = 0; i < ntracks; ++i) {
          MusicTrack track;
          MusicTimeStamp trackLength;
          UInt32 propsize = sizeof(MusicTimeStamp);
          MusicSequenceGetIndTrack( newMusicSequence, i, &track );
          MusicTrackGetProperty(track, kSequenceTrackProperty_TrackLength, &trackLength, &propsize);
          if ( trackLength > sequenceLength )
            sequenceLength = trackLength;
        }

        MusicTrack musicTrack;
        if ( MusicSequenceGetIndTrack( newMusicSequence, 0, &musicTrack ) == noErr ) {
          MusicEventUserData eventData = { 0 };
          MusicTrackNewUserEvent( musicTrack, sequenceLength, &eventData );
        }
        MusicPlayerPreroll( musicPlayer );
      }
    }
  }
  if ( musicSequence )
    DisposeMusicSequence( musicSequence );
  musicSequence = newMusicSequence;
}

- (float)speed
{
  return speed;
}

- (void)setSpeed:(float)aSpeed
{
  speed = aSpeed;
  MusicPlayerSetPlayRateScalar(	musicPlayer, (Float64)pow( 2.0, speed ) );
  [[NSUserDefaults standardUserDefaults] setFloat:speed forKey:kPlayerSpeedKey];
}

- (BOOL)loop
{
  return loop;
}

- (void)setLoop:(BOOL)aLoop
{
  loop = aLoop;
  [[NSUserDefaults standardUserDefaults] setInteger:loop forKey:kPlayerLoopKey];

  if ( musicPlayer && musicSequence ) {
    UInt32 ntracks;
    MusicSequenceGetTrackCount( musicSequence, &ntracks );
    for (UInt32 i = 0; i < ntracks; ++i) {
      MusicTrack track;
      MusicTimeStamp trackLength;
      UInt32 propsize = sizeof(MusicTimeStamp);
      MusicSequenceGetIndTrack( musicSequence, i, &track );
      MusicTrackGetProperty(track, kSequenceTrackProperty_TrackLength, &trackLength, &propsize);

      MusicTrackLoopInfo loopInfo;
      loopInfo.loopDuration = trackLength;
      loopInfo.numberOfLoops = loop ? 0 : 1;
      MusicTrackSetProperty(track, kSequenceTrackProperty_LoopInfo, &loopInfo, sizeof(MusicTrackLoopInfo));
    }
  }
}

- (BOOL)shuffle
{
  return shuffle;
}

- (void)setShuffle:(BOOL)aShuffle
{
  shuffle = aShuffle;
  [[NSUserDefaults standardUserDefaults] setInteger:shuffle forKey:kPlayerShuffleKey];
}

- (BOOL)chords
{
  return chords;
}

- (void)setChords:(BOOL)aChords
{
  chords = aChords;
  [[NSUserDefaults standardUserDefaults] setFloat:speed forKey:kGuitarChordsKey];

  if ( musicPlayer && musicSequence ) {
    MusicTrack track;
    if ( MusicSequenceGetIndTrack( musicSequence, 1, &track ) == noErr ) {
      Boolean mute = !chords;
      MusicTrackSetProperty( track, kSequenceTrackProperty_MuteStatus, &mute, sizeof(Boolean) );
    }
  }
}

- (NSArray*)items
{
  return items;
}

- (void)setItems:(NSArray*)aItems
{
  NSArray* old = items;
  items = [aItems retain];
  [old release];
  currentItem = 0;
}

@end

@implementation TunePlayerItem
- (NSData*)data
{
  return nil;
}
@end
