from Foundation import *
from AppKit import *

import webbrowser
import cgi
import sys

from collections import defaultdict
import re

def orderChords(input):
    for chord in input:
        chord.sort(lambda x,y : cmp(x[0],y[0]))


class ParseError(Exception):
    def __init__(self, message):
        self.message = message

    def __str__(self):
        return repr(self.message)


class AbcParser:
    convert = {}
    convert['C'] = 0
    convert['D'] = 2
    convert['E'] = 4
    convert['F'] = 5
    convert['G'] = 7
    convert['A'] = 9
    convert['B'] = 11

    convert['c'] = 12
    convert['d'] = 14
    convert['e'] = 16
    convert['f'] = 17
    convert['g'] = 19
    convert['a'] = 21
    convert['b'] = 23

    convert['z'] = 999
    convert['x'] = 999

    invconvert = {}
    for key, v in convert.iteritems():
        invconvert[v] = key

    def loadFromFile(self, filename):
        f = open(filename)
        self.lines = f.readlines()
        f.close()

    def loadFromString(self, inputstr):
#        print inputstr
        self.lines = inputstr.splitlines()
        print self.lines

    def parseNote(self, content, accidental):
        m = re.match(">?([\^_=]*)([xza-gA-G])(,*)('*)([0-9]*)(/[0-9]*)?(\-)?", content)

        if not m:
            print "Unknown character(s) near %s" % content[:7]
            raise ParseError("Unknown character(s) near %s" % content[:7])

        #alterations
        gr = m.group(1)
        if gr == "^":
            acc = 1
        elif gr == "_":
            acc = -1
        elif gr == "=":
            acc = 0
        else:
            acc = accidental[m.group(2)]

        #pitch
        pitch = self.__class__.convert[m.group(2)] + acc

        if m.group(3) != "":
            pitch -= 12*len(m.group(3))
        elif m.group(4) != "":
            pitch += 12*len(m.group(4))

        #speed
        speed = 1.0

        if m.group(5) != "":
            speed *= int(m.group(5))

        if m.group(6) != None:
            a = int(m.group(6)[1:])
            if self.divisor % a != 0:
                self.divisor *= a

            speed /= a

        #tied
        tied = (m.group(7) != None)

        content = content[len(m.group(0)):]

        return ((pitch, speed, tied), content)

    ##
    #Return the score of an abc code opened with loadFromString or loadFromFile
    def parse(self):
        #header
        self.header = ""
        for line in self.lines:
            if not re.match("[A-Z]:", line) and not re.match("%", line):
                break
            self.header += line

        #cleaning
        content = ""
        for line in self.lines:
            if not re.match("[A-Z]:", line) and not re.match("%", line):
                content += re.sub("[<\n\\\ ]|\([1-9]+","", line)

        #score
        self.divisor = 1
        score = []
        accidental = defaultdict(int)
        while len(content) > 0:
            if content[0] == "|":
                accidental = defaultdict(int)
                content = content[1:]
            elif content[0] == "[":
                content = content[1:]
                chord = []
                while content[0] != "]":
                    note, content = self.parseNote(content, accidental)
                    chord.append(note)
                content = content[1:]
                score.append(chord)
            else:
                note, content = self.parseNote(content, accidental)
                score.append([note]) #one-note chord

        #speeds
        for note in score:
            for i,n in enumerate(note):
                note[i] = (n[0], int(n[1]*self.divisor), n[2])

        return score

    def processNote(self, (pitch, speed, tied)):
        if pitch == 999:
            self.content += "z"
        else:
            if pitch % 12 not in [0,2,4,5,7,9,11]:
                self.content += "^"
                pitch -= 1
            else:
                self.content += "="

            dec = ""
            while pitch < 0:
                dec += ","
                pitch += 12

            while pitch > 23:
                dec += "'"
                pitch -= 12

            self.content += self.__class__.invconvert[pitch]
            self.content += dec

        if speed != self.divisor:
            if speed % self.divisor == 0:
                self.content += str(speed/self.divisor)
            else:
                self.content +=  str(speed) + "/" + str(self.divisor)

        if tied == 1:
            self.content += "-"

    #transform a score (in our internal representation) into abc notation
    def process(self, score):
        self.content = self.header

        for chord in score:
            if len(chord) == 1:
                self.processNote(chord[0])
            else:
                self.content += "["
                for note in chord:
                    self.processNote(note)
                self.content += "]"
            self.content += " "

    def saveToFile(self, filename):
        f = open(filename, "wb")
        f.write(self.content)
        f.close()

    def getAbcCode(self):
        return self.content
        
def main(context):
  print "Running plugin!"
  tunes = context.tunes()
  tune = tunes[6];
  tune.hydrate();
  

#  from abcparser import AbcParser

  abcparser = AbcParser()
  abcparser.loadFromString( tune.dots().UTF8String() )
  try:
      score = abcparser.parse()

      abcparser.process(score)
      print abcparser.content
  except:
      print "Error"
      print sys.exc_info()[0]
  print score

#  for t in tunes:
#    print t.title().UTF8String()
#    t.hydrate()
#    print t.dots().UTF8String()
#  title = context.selectedText()
#  url = 'http://www.google.com/search?q=' + cgi.escape(title) + '&as_sitesearch=thesession.org'
#  print url
# webbrowser.open(url)
