/*
 *  This file is part of abc2ps,
 *  Copyright (C) 1996-2007  Michael Methfessel
 *  See file abc2ps.c for details.
 */

#include "abc2ps.h"
#include "buffer.h"
#include "pssubs.h"
#include "subs.h"
#include "util.h"
#include <string.h>

/*  subroutines to handle output buffer  */

static int   ln_num;                  /* number of lines in buffer */
static float ln_pos[BUFFLN];          /* vertical positions of buffered lines */
static int   ln_buf[BUFFLN];          /* buffer location of buffered lines */

/* ----- bskip(h): translate down by h points in output buffer ---- */
void bskip(float h)
{
  if (h * h > 0.0001) {
    graphicsTranslate( gcx, 0, -h );
    bposy = bposy - h;
  }
}

/* ----- init_pdims: initialize page dimensions ----- */
void init_pdims ()
{
  if (in_page)
    return;
  posx = cfmt.leftmargin;
  posy = cfmt.pageheight - cfmt.topmargin;

}

/* ----- clear_buffer ------- */
void clear_buffer ()
{
  nbuf   = 0;
  bposy  = 0.0;
  ln_num = 0;
}

/* ----- write_buffer: write buffer contents, break at full pages ---- */
void write_buffer (FILE* fp)
{
  int l, b1, b2, nb;
  float p1, dp;

  writenum++;

  nb = 0;
  for (l = 0; l < ln_num; l++) {
    b1 = 0;
    p1 = 0;
    if (l > 0) {
      b1 = ln_buf[l - 1];
      p1 = ln_pos[l - 1];
    }
    b2 = ln_buf[l];
    dp = ln_pos[l] - p1;
    if ((posy + dp < cfmt.botmargin)) {
      write_pagebreak (fp);
    }
    graphicsFlushBuffer( gcx, buf, b1, b2 );
    posy = posy + dp;
    nb = ln_buf[l];
  }

  if (nb < nbuf) {
    graphicsFlushBuffer( gcx, buf, nb, nbuf );
  }

  clear_buffer();
}

/* ----- buffer_eob: handle completed block in buffer ------- */
/* if the added stuff does not fit on current page, write it out
   after page break and change buffer handling mode to pass though */
void buffer_eob (FILE* fp)
{
  int do_break;

  if (ln_num >= BUFFLN)
    rx("max number off buffer lines exceeded", " -- check BUFFLN");

  ln_buf[ln_num] = nbuf;
  ln_pos[ln_num] = bposy;
  ln_num++;

  if (!use_buffer) {
    write_buffer (fp);
    return;
  }

  do_break = 0;
  if (posy + bposy < cfmt.botmargin)
    do_break = 1;
  if (cfmt.oneperpage)
    do_break = 1;

  if (do_break) {
    if (tunenum != 1 )
      write_pagebreak (fp);
    write_buffer (fp);
    use_buffer = 0;
  }
}

/* ----- check_buffer: dump buffer if less than nb bytes avilable --- */
void check_buffer (FILE* fp, int nb)
{
  char mm[81];

  if (nbuf + nb > BUFFSZ) {
    sprintf (mm, "BUFFSZ exceeded at line %d", ln_num);
    wng("possibly bad page breaks, ", mm);
    write_buffer (fp);
    use_buffer = 0;
  }
}
