/*
 *  This file is part of abc2ps,
 *  Copyright (C) 1996-2007  Michael Methfessel
 *  See file abc2ps.c for details.
 */

#include <ctype.h>
#include <stdlib.h>
#include <string.h>

#include "abc2ps.h"
#include "style.h"
#include "buffer.h"
#include "util.h"
#include "format.h"
#include "pssubs.h"
#include "parse.h"

#include "subs.h"


/* ----- is_xrefstr: check if string ok for xref selection ---- */
int is_xrefstr (char* str)
{

  char* c;
  c = str;
  while (*c != '\0') {
    if ((!isdigit(*c)) && (*c != '-') && (*c != ',') && (*c != ' '))
      return 0;
    c++;
  }
  return 1;
}


/* ----- init_ops ----- */
void init_ops (int job)
{
  int j;

  musicSettings.one_per_page         = -1;
  musicSettings.landscape            = -1;
  musicSettings.scalefac             = -1.0;
  musicSettings.lmargin              = -1.0;
  musicSettings.swidth               = -1.0;
  musicSettings.write_history        = -1;
  musicSettings.staffsep             = -1;
  musicSettings.dstaffsep            =  0;
  musicSettings.break_continues      = -1;
  musicSettings.continue_lines       = -1;
  musicSettings.include_xrefs        = -1;
  musicSettings.alfa_c               = -1.0;
  musicSettings.strict1              = -1.0;
  musicSettings.strict2              = -1.0;
  musicSettings.barnums              = -1;
  musicSettings.barnums_cl           = -1;
  musicSettings.make_index           =  0;
  musicSettings.make_audio           =  0;
  musicSettings.playtempo            =  0;
  musicSettings.au_nomusic           =  0;
  musicSettings.select_all           =  0;
  musicSettings.pb_start = musicSettings.pb_end    = -1;
  musicSettings.write_text           =  0;
  musicSettings.do_acc               =  0;
  strcpy (musicSettings.acc_pat, "NONE");

  musicSettings.do_mode              = DO_OUTPUT;
  musicSettings.make_ps              = 1;

  musicSettings.pagenumbers          = 0;
  strcpy (styf, "");
  musicSettings.ntrp = 0;
  strcpy (musicSettings.transpose[0], "");
  strcpy (musicSettings.vcselstr, "");
  strcpy (musicSettings.au_qstr, "");

  musicSettings.force_ktype     = 0;
  musicSettings.force_add_pitch = 0;
  for (j = 0; j < MAXSVC; j++)
    musicSettings.frc_ktype[j] = musicSettings.frc_add_pitch[j] = 0;

  if (job) {
    strcpy (styd, DEFAULT_FDIR);
    strcpy (outf, OUTPUTFILE);
    musicSettings.invert_chords        = 1;
    musicSettings.pretty               = 0;
    musicSettings.gmode                = G_FILL;
    vb                   = VERBOSE0;
    musicSettings.search_field0        = S_TITLE;
  }
}


/* ----- ops_into_fmt ----- */
void ops_into_fmt(Format* fmt)
{
  if (musicSettings.landscape >= 0)
    fmt->landscape = musicSettings.landscape;
  if (musicSettings.scalefac >= 0)
    fmt->scale = musicSettings.scalefac;
  if (musicSettings.lmargin >= 0)
    fmt->leftmargin = musicSettings.lmargin;
  if (musicSettings.swidth >= 0)
    fmt->staffwidth = musicSettings.swidth;
  if (musicSettings.continue_lines >= 0)
    fmt->continueall = musicSettings.continue_lines;
  if (musicSettings.break_continues >= 0)
    fmt->breakall = musicSettings.break_continues;
  if (musicSettings.write_history >= 0)
    fmt->writehistory = musicSettings.write_history;
  if (musicSettings.bars_per_line > 0)
    fmt->barsperstaff = musicSettings.bars_per_line;
  if (musicSettings.include_xrefs >= 0)
    fmt->withxrefs = musicSettings.include_xrefs;
  if (musicSettings.one_per_page >= 0)
    fmt->oneperpage = musicSettings.one_per_page;
  if (musicSettings.alfa_c >= 0)
    fmt->maxshrink = musicSettings.alfa_c;
  if (musicSettings.staffsep >= 0)
    fmt->staffsep = musicSettings.staffsep;
  if (musicSettings.strict1 >= 0)
    fmt->strict1 = musicSettings.strict1;
  if (musicSettings.strict2 >= 0)
    fmt->strict2 = musicSettings.strict2;
  if (musicSettings.barnums >= 0)
    fmt->barnums = musicSettings.barnums;
  if (musicSettings.playtempo > 0)
    fmt->playtempo = musicSettings.playtempo;
  if (strlen(musicSettings.au_qstr) > 0)
    strcpy(fmt->auquality, musicSettings.au_qstr);
  if (strcmp(musicSettings.acc_pat, "NONE"))
    fmt->accomp = musicSettings.acc_pat[0];
  if (musicSettings.barnums_cl >= 0)
    fmt->barnums_cl = musicSettings.barnums_cl;
  fmt->staffsep    = fmt->staffsep    + musicSettings.dstaffsep;
  fmt->sysstaffsep = fmt->sysstaffsep + musicSettings.dstaffsep;

}

#if !ABC_LIBRARY
/* ----- parse_args: parse list of arguments, interpret flags ----- */
int parse_args (int ac, char* av[])
{
  int i, m, k, nsel, sel_arg, j, ok, f_pos, got_value, nn;
  char c, aaa[201], ext[41], cc, * s;
  KeySignature tmp_key;

  help_me      = 0;
  ninf         = 0;
  nsel         = 0;
  sel_arg      = 0;
  respect_case = 0;
  f_pos        = -1;
  s_field[0]   = search_field0;
  strcpy (sel_str[0], "");
  strcpy (znote, "");
  strcpy (global_transpose, "");
  musicSettings.global_halftones = 0;

  i = 1;
  while (i < ac) {

    if (av[i][0] == '-') {         /* interpret a flag with '-'*/

      /* identify fullword options first */
      if (!strcmp(av[i], "-maxv")) {
        if ((i == ac - 1) || (av[i + 1][0] == '-'))
          rx("missing parameter for ", av[i]);
        sscanf(av[++i], "%d", &maxVc);
        if (maxVc > 100)
          maxVc = 100;
      }
      else if (!strcmp(av[i], "-maxs")) {
        if ((i == ac - 1) || (av[i + 1][0] == '-'))
          rx("missing parameter for ", av[i]);
        sscanf(av[++i], "%d", &maxSyms);
      }
      else if (!strcmp(av[i], "-hh")) {
        help_me = 3;
      }
  #if ABC_USE_CAIRO
      else if (!strcmp(av[i], "-format")) {
        if ((i == ac - 1) || (av[i + 1][0] == '-'))
          rx("missing parameter for ", av[i]);
        outputFormat = av[++i];
      }
  #endif

      else {
        m = 1;
        k = strlen (av[i]);
        while (m < k) {
          c = av[i][m];
          if      (c == 'h')
            help_me = 1;                                   /* simple flags */
          else if (c == 'H')
            help_me = 2;
          /*|           else if (c == 'e')  sel_arg = 1; |*/
          else if (c == 'f') {
            nsel++;
            strcpy (sel_str[nsel], "");
            if (ninf == 0) {             /* selector before first file */
              strcpy (sel_str[nsel], sel_str[nsel - 1]);
              s_field[nsel] = s_field[nsel - 1];
            }
            s_field[nsel] = search_field0;
            sel_arg = 0;
            f_pos = i;
          }
          else if (c == 'b') {
            break_continues = 1;
            continue_lines = 0;
          }
          else if (c == 'c') {
            continue_lines = 1;
            break_continues = 0;
          }
          else if (c == 'x')
            musicSettings.include_xrefs = 1;
          else if (c == '1')
            musicSettings.one_per_page = 1;
          else if (c == 'i')
            invert_chords = 0;
          else if (c == 'n')
            write_history = 1;
          else if (c == 'l')
            musicSettings.landscape = 1;
          else if (c == 'p')
            pretty = 1;
          else if (c == 'P')
            pretty = 2;
          else if (c == 'E')
            epsf = 1;
          else if (c == 'o') {
            musicSettings.do_mode = DO_OUTPUT;
            make_ps = 1;
          }
          else if (c == 'N')
            pagenumbers = 1;
          else if (c == 'I')
            musicSettings.make_index = 1;
          else if (c == 'a') {
            musicSettings.do_mode = DO_OUTPUT;
            make_audio = 1;
          }
          else if (c == 'G')
            au_nomusic = 1;
          else if (c == 'T')
            write_text = 1;
          else if (c == 'C') {
            make_audio = make_ps = 1;
            do_mode = DO_OUTPUT;
          }

          /* -ex only with sticky arg */
          else if (c == 'e') {
            sel_arg = 1;
            respect_case = 0;
            s_field[nsel] = S_TITLE;
            if (av[i][m + 1]) {
              cc = av[i][m + 1];
              if      (cc == 't')
                s_field[nsel] = S_TITLE;
              else if (cc == 'c')
                s_field[nsel] = S_COMPOSER;
              else if (cc == 'r')
                s_field[nsel] = S_RHYTHM;
              else if (cc == 's')
                s_field[nsel] = S_SOURCE;
              else if (cc == 'T') {
                respect_case = 1;
                s_field[nsel] = S_TITLE;
              }
              else if (cc == 'C') {
                respect_case = 1;
                s_field[nsel] = S_COMPOSER;
              }
              else if (cc == 'R') {
                respect_case = 1;
                s_field[nsel] = S_RHYTHM;
              }
              else if (cc == 'S') {
                respect_case = 1;
                s_field[nsel] = S_SOURCE;
              }
              else {
                printf ("Invalid parameter for flag -e: %c\n", cc);
                return 1;
              }
            }
            break;
          }

          /* -Ax only with sticky arg */
          else if (c == 'A') {
            if (av[i][m + 1]) {
              acc_pat[0] = av[i][m + 1];
              acc_pat[1] = '\0';
            }
            musicSettings.do_mode = DO_OUTPUT;
            do_acc = 1;
            make_audio = 1;
            make_ps = 1;
            break;
          }


          /* --- flags with parameters */
          else if (strchr("vsdwmgtkqAQDFYBVXOZS", c)) {
            strlcpy (aaa, &av[i][m + 1], 200);
            if ((strlen(aaa) > 0) && strchr("glO", c)) {     /* no sticky arg */
              printf ("+++ Incorrect usage of flag -%c\n", c);
              return 1;
            }

            got_value = 1;                            /* check for value */
            if (strlen(aaa) == 0) {
              got_value = 0;
              i++;
              if ((i >= ac) || (av[i][0] == '-'))
                i--;
              else {
                strlcpy (aaa, av[i], 200);
                got_value = 1;
              }
            }

            if (got_value && strchr("vsYB", c)) {      /* check num args */
              ok = 1;
              for (j = 0; j < strlen(aaa); j++)
                if (!strchr("0123456789.", aaa[j]))
                  ok = 0;
              if (!ok) {
                printf ("+++ Invalid parameter <%s> for flag -%c\n", aaa, c);
                return 1;
              }
            }


            /* --- flags which require a value */
            if (!got_value) {                   /* check value was given */
              printf ("+++ Missing parameter after flag -%c\n", c);
              return 1;
            }

            if (c == 'S') {
              s = strchr (aaa, '=');
              if (s) {
                *s++ = 0;
                if (set_keysig (aaa, &tmp_key, 1))
                  rx ("Invalid argument for flag -S: ", aaa);
                for (j = 0; j < MAXSVC; j++) {
                  if (str_sel (s, j + 1)) {
                    frc_ktype[j]     = tmp_key.ktype;
                    frc_add_pitch[j] = tmp_key.add_pitch;
                  }
                }
              }
              else {
                if (set_keysig (aaa, &tmp_key, 1))
                  rx ("Invalid argument for flag -S: ", aaa);
                force_ktype     = tmp_key.ktype;
                force_add_pitch = tmp_key.add_pitch;
                for (j = 0; j < MAXSVC; j++) {
                  frc_ktype[j]     = tmp_key.ktype;
                  frc_add_pitch[j] = tmp_key.add_pitch;
                }
              }
              break;
            }

            if (c == 'Z') {
              strlcpy (znote, aaa, 200);
              break;
            }

            if (c == 'k') {
              nn = 0;
              sscanf (aaa, "%d%n", &barnums, &nn);
              if (aaa[nn] == 'c')
                barnums_cl = 1;
              break;
            }

            if (c == 'Q') {
              sscanf(aaa, "%f", &playtempo);
              if (playtempo < 6)
                playtempo = 6;
              break;
            }

            if (c == 'q') {
              strlcpy (au_qstr, aaa, 20);
              break;
            }

            if (c == 'V') {                               /* -V flag */
              ok = 1;
              strlcpy (musicSettings.vcselstr, aaa, 100);
            }

            if (c == 'X') {                  /* -X flag now selects pb bars */
              ok = 1;
              pb_start = pb_end = -1;

              if (*aaa == '-') {
                pb_start = 1;
                sscanf (aaa + 1, "%d", &pb_end);
              }
              else {
                sscanf (aaa, "%d-%d", &pb_start, &pb_end);
                if (pb_end < 0)
                  pb_end   = 100000;
                if (!strchr(aaa, '-'))
                  pb_end = pb_start;
              }
              if (pb_start == 0)
                pb_start = 1;
              if (pb_end < pb_start)
                pb_end = pb_start;
            }

            if (c == 'O') {                               /* -O flag  */
              if (!strcmp(aaa, "=")) {
                choose_outname = 1;
              }
              else {
                getext (aaa, ext);
                if (strcmp(ext, "ps") && strcmp(ext, "eps") && strcmp(ext, "")) {
                  printf ("Wrong extension for output file: %s\n", aaa);
                  return 1;
                }
                strext (outf, aaa, "ps", 1);
                choose_outname = 0;
              }
            }

            if (c == 'B') {
              sscanf (aaa, " %d", &bars_per_line);
              continue_lines = 0;
            }

            if (c == 't') {
              if (musicSettings.ntrp >= MAXTRP)
                rx ("Too many transpose flags", "");
              strlcpy (musicSettings.transpose[musicSettings.ntrp], aaa, 20);
              if (!strchr(aaa, '='))
                strlcpy (musicSettings.global_transpose, aaa, 20);
              musicSettings.ntrp++;
            }

            if (c == 'v')
              sscanf(aaa, "%d", &vb);
            if (c == 's')
              sscanf(aaa, "%f", &scalefac);
            if (c == 'd') {
              if (aaa[0] == '+' || aaa[0] == '-')
                dstaffsep = scan_u(aaa);
              else
                staffsep = scan_u(aaa);
            }
            if (c == 'w')
              swidth   = scan_u(aaa);
            if (c == 'm')
              lmargin  = scan_u(aaa);
            if (c == 'F')
              strlcpy (styf, aaa, STRL1 - 1);
            if (c == 'D')
              strlcpy (styd, aaa, STRL1 - 1);
            if (c == 'g') {
              if      (abbrev(aaa, "shrink", 2))
                musicSettings.gmode = G_SHRINK;
              else if (abbrev(aaa, "stretch", 2))
                musicSettings.gmode = G_STRETCH;
              else if (abbrev(aaa, "space",  2))
                musicSettings.gmode = G_SPACE;
              else if (abbrev(aaa, "fill",   2))
                musicSettings.gmode = G_FILL;
              else {
                printf ("+++ Bad parameter for flag -g: %s\n", aaa);
                return 1;
              }
            }
            break;
          }
          else {
            printf ("+++ Unknown flag: -%c\n", c);
            return 1;
          }
          m++;
        }
      }
    }

    else {
      if (strstr(av[i], ".fmt")) {     /* implict -F */
        strlcpy (styf, av[i], STRL1 - 1);
      }
      else {
        if (strstr(av[i], ".abc") && sel_arg) {     /* file if .abc */
          nsel++;
          strcpy (sel_str[nsel], "");
          s_field[nsel] = S_TITLE;
          if (ninf == 0) {                /* selector before first file */
            strcpy (sel_str[nsel], sel_str[nsel - 1]);
            s_field[nsel] = s_field[nsel - 1];
          }
          sel_arg = 0;
        }
        if (is_xrefstr(av[i]) && (!sel_arg)) {    /* sel if xref numbers */
          if (i - 1 != f_pos)
            sel_arg = 1;
        }

        if (!sel_arg) {             /* this arg is a file name */
          if (ninf >= MAXINF) {
            printf ("+++ Too many input files, max is %d\n", MAXINF);
            return 1;
          }
          strlcpy (in_file[ninf], av[i], STRL1 - 1);
          psel[ninf] = nsel;
          ninf++;
        }
        else {                             /* this arg is a selector */
          strcat(sel_str[nsel], av[i]);
          strcat(sel_str[nsel], " ");
        }
      }
    }
    i++;
  }

  /*|   for (j=0; j<maxVc; j++) |*/
  /*|     printf ("voice %d  ktype=%d add=%d\n", j+1, |*/
  /*|             frc_ktype[j], frc_add_pitch[j]); |*/

  return 0;
}
#endif

/* ----- alloc_structs ----- */
/* Thanks to Henrik Norbeck for this */
void alloc_structs ()
{
  int j;

  sym = (Symbol*)calloc(maxSyms, sizeof(Symbol));
  if (sym == NULL)
    rx("Out of memory", "");

  symv = (Symbol**)calloc(maxVc, sizeof(Symbol*));
  if (symv == NULL)
    rx("Out of memory", "");

  for (j = 0; j < maxVc; j++) {
    symv[j] = (Symbol*)calloc(maxSyms, sizeof(Symbol));
    if (symv[j] == NULL)
      rx("Out of memory", "");
  }

  xp = (XPosition*)calloc(maxSyms + 1, sizeof(XPosition));
  if (xp == NULL)
    rx("Out of memory", "");

  for (j = 0; j < maxSyms + 1; j++) {
    xp[j].p = (int*)calloc(maxVc, sizeof(int));
    if (xp[j].p == NULL)
      rx("Out of memory", "");
  }

  voice = (Voice*)calloc(maxVc, sizeof(Voice));
  if (voice == NULL)
    rx("Out of memory", "");

  sym_st = (Symbol**)calloc(maxVc, sizeof(Symbol*));
  if (sym_st == NULL)
    rx("Out of memory", "");

  for (j = 0; j < maxVc; j++) {
    sym_st[j] = (Symbol*)calloc(MAXSYMST, sizeof(Symbol));
    if (sym_st[j] == NULL)
      rx("Out of memory", "");
  }

  nsym_st = (int*)calloc(maxVc, sizeof(int));
  if (nsym_st == NULL)
    rx("Out of memory", "");

}

#if ABC_LIBRARY
void free_structs ()
{
  int j;

  if ( sym )
    free( sym );

  if ( symv ) {
    for (j = 0; j < maxVc; j++)
      free( symv[j] );
    free( symv );
  }

  if ( xp ) {
    for (j = 0; j < maxSyms + 1; j++)
      free( xp[j].p );
    free( xp );
  }

  if ( voice )
    free( voice );

  if ( sym_st ) {
    for (j = 0; j < maxVc; j++)
      free( sym_st[j] );
    free( sym_st );
  }

  if ( nsym_st )
    free( nsym_st );
}
#endif

/* ----- set_page_format ----- */
int set_page_format ()
{
  int i, j;

  if (musicSettings.pretty == 1)
    set_pretty_format (&cfmt);
  else if (musicSettings.pretty == 2)
    set_pretty2_format (&cfmt);
  else
    set_standard_format (&cfmt);

  i = read_fmt_file ("fonts.fmt", styd, &cfmt);
  j = 0;
  if (strlen(styf) > 0) {
    strext(styf, styf, "fmt", 1);
    j = read_fmt_file (styf, styd, &cfmt);
    if (j == 0) {
      printf ("\n+++ Cannot open file: %s\n", styf);
      return 0;
    }
    strlcpy (cfmt.name, styf, STRLFMT - 1);
  }
  if (i || j)
    printf("\n");
  ops_into_fmt (&cfmt);

  make_font_list (&cfmt);
  sfmt = cfmt;
  dfmt = cfmt;
  return 1;
}


/* ----- set_font ----- */
void set_font (FILE* fp, FontSpec font, int add_bracket)
{
  int i, fnum;

  fnum = -1;
  for (i = 0; i < nfontnames; i++) {
    if (!strcmp(font._name, fontnames[i]))
      fnum = i;
  }
  if (fnum < 0) {
#if !TARGET_OS_IPHONE
    printf ("\n+++ Font \"%s\" not predefined; using first in list\n",
            font._name);
#endif
    fnum = 0;
  }
  #if 1 //FIXME-fonts
  graphicsSetFont( gcx, fnum, font._size );
  #else
  if (strstr (font._name, "Helvetica"))
    graphicsSetFont( gcx, kSansSerifFont, font._size );
  else
    graphicsSetFont( gcx, kSerifFont, font._size );
  #endif
}

/* ----- close_output_file ------ */
void close_output_file ()
{
  if (!file_open)
    return;
  close_page(fout);
  file_open = 0;
  file_initialized = 0;
}


/* ----- open_output_file ------ */
void open_output_file (char* fnam, char* tstr)
{
  pagenum = 0;
  tunenum = 0;
  file_open = 1;
  file_initialized = 0;
}
